@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -76,9 +76,11 @@ using MatrixInt = Matrix<Integer, true>;
  *
  * \ingroup algebra
  */
-class AbelianGroup : public ShortOutput<AbelianGroup, true> {
+class AbelianGroup :
+        public ShortOutput<AbelianGroup, true>,
+        public TightEncodable<AbelianGroup> {
     protected:
-        unsigned rank_ { 0 };
+        size_t rank_ { 0 };
             /**< The rank of the group (the number of Z components). */
         std::vector<Integer> revInvFactors_;
             /**< The invariant factors <i>d0</i>,...,<i>dn</i> as
@@ -93,7 +95,7 @@ class AbelianGroup : public ShortOutput<AbelianGroup, true> {
         /**
          * Creates a new trivial group.
          */
-        AbelianGroup();
+        AbelianGroup() = default;
         /**
          * Creates a clone of the given group.
          */
@@ -110,7 +112,7 @@ class AbelianGroup : public ShortOutput<AbelianGroup, true> {
          *
          * @param rank the rank of the new group.
          */
-        AbelianGroup(unsigned rank);
+        AbelianGroup(size_t rank);
         /**
          * Creates a new group with the given rank and invariant factors.
          *
@@ -131,7 +133,7 @@ class AbelianGroup : public ShortOutput<AbelianGroup, true> {
          * is greater than 1 and divides the invariant factor after it.
          */
         template <typename T>
-        AbelianGroup(unsigned rank, std::initializer_list<T> invFac);
+        AbelianGroup(size_t rank, std::initializer_list<T> invFac);
         /**
          * Creates a new group with the given rank and invariant factors.
          *
@@ -152,7 +154,7 @@ class AbelianGroup : public ShortOutput<AbelianGroup, true> {
          * is greater than 1 and divides the invariant factor after it.
          */
         template <typename Container>
-        AbelianGroup(unsigned rank, const Container& invFac);
+        AbelianGroup(size_t rank, const Container& invFac);
         /**
          * Creates the abelian group defined by the given presentation matrix.
          *
@@ -231,7 +233,7 @@ class AbelianGroup : public ShortOutput<AbelianGroup, true> {
          *
          * @param extraRank the extra rank to add; this defaults to 1.
          */
-        void addRank(int extraRank = 1);
+        void addRank(long extraRank = 1);
         /**
          * Adds the given torsion element to the group.
          *
@@ -247,76 +249,6 @@ class AbelianGroup : public ShortOutput<AbelianGroup, true> {
          * must be strictly positive.
          */
         void addTorsion(Integer degree);
-        /**
-         * Deprecated routine that adds the given torsion element to the group.
-         *
-         * As of Regina 7.0, this routine is much faster than it used to be.
-         * In particular, if you have many torsion elements to add, it is now
-         * efficient just to call addTorsion() for each new torsion element,
-         * one at a time.
-         *
-         * In this routine we add a specified number of copies of
-         * Z_<i>d</i>, where <i>d</i> is some given degree.
-         *
-         * \deprecated Use addTorsion() instead, multiple times if
-         * necessary (which is exactly how this routine is implemented).
-         *
-         * \pre The given degree is strictly positive, and the
-         * given multiplicity is at least 1.
-         *
-         * @param degree <i>d</i>, where we are adding copies of
-         * <i>Z_d</i> to the torsion.
-         * @param mult the multiplicity <i>m</i>, where we are adding
-         * precisely <i>m</i> copies of <i>Z_d</i>; this defaults to 1.
-         */
-        [[deprecated]] void addTorsionElement(const Integer& degree,
-            unsigned mult = 1);
-        /**
-         * Deprecated routine that adds the given torsion element to the group.
-         *
-         * As of Regina 7.0, this routine is much faster than it used to be.
-         * In particular, if you have many torsion elements to add, it is now
-         * efficient just to call addTorsion() for each new torsion element,
-         * one at a time.
-         *
-         * In this routine we add a specified number of copies of
-         * Z_<i>d</i>, where <i>d</i> is some given degree.
-         *
-         * \deprecated Use addTorsion() instead, multiple times if
-         * necessary (which is exactly how this routine is implemented).
-         *
-         * \pre The given degree is strictly positive, and the
-         * given multiplicity is at least 1.
-         *
-         * @param degree <i>d</i>, where we are adding copies of
-         * <i>Z_d</i> to the torsion.
-         * @param mult the multiplicity <i>m</i>, where we are adding
-         * precisely <i>m</i> copies of <i>Z_d</i>; this defaults to 1.
-         */
-        [[deprecated]] void addTorsionElement(unsigned long degree,
-            unsigned mult = 1);
-        /**
-         * Deprecated routine that adds the given set of torsion elements to
-         * this group.
-         *
-         * The torsion elements to add are described by a list of
-         * integers <i>k1</i>,...,<i>km</i>, where we are adding
-         * Z_<i>k1</i>,...,Z_<i>km</i>.  Unlike invariant factors, the
-         * <i>ki</i> are not required to divide each other.
-         *
-         * \deprecated This routine uses an old implementation, and it is now
-         * much faster just to add each torsion element one at a time using
-         * addTorsion().
-         *
-         * \pre Each integer in the given list is strictly greater than 1.
-         *
-         * \ifacespython This routine takes a python list as its argument.
-         *
-         * @param torsion a list containing the torsion elements to add,
-         * as described above.
-         */
-        [[deprecated]] void addTorsionElements(
-            const std::multiset<Integer>& torsion);
         /**
          * Adds the abelian group defined by the given presentation to this
          * group.  Note that this routine might be slow since calculating the
@@ -348,7 +280,7 @@ class AbelianGroup : public ShortOutput<AbelianGroup, true> {
          *
          * @return the number of included copies of \e Z.
          */
-        unsigned rank() const;
+        size_t rank() const;
         /**
          * Returns the rank in the group of the torsion term of given degree.
          * If the given degree is <i>d</i>, this routine will return the
@@ -365,7 +297,7 @@ class AbelianGroup : public ShortOutput<AbelianGroup, true> {
          * @param degree the degree of the torsion term to query.
          * @return the rank in the group of the given torsion term.
          */
-        unsigned torsionRank(const Integer& degree) const;
+        size_t torsionRank(const Integer& degree) const;
         /**
          * Returns the rank in the group of the torsion term of given degree.
          * If the given degree is <i>d</i>, this routine will return the
@@ -382,7 +314,7 @@ class AbelianGroup : public ShortOutput<AbelianGroup, true> {
          * @param degree the degree of the torsion term to query.
          * @return the rank in the group of the given torsion term.
          */
-        unsigned torsionRank(unsigned long degree) const;
+        size_t torsionRank(unsigned long degree) const;
         /**
          * Returns the number of invariant factors that describe the
          * torsion elements of this group.
@@ -425,7 +357,7 @@ class AbelianGroup : public ShortOutput<AbelianGroup, true> {
          * @return \c true if and only if this is the free abelian group
          * of rank \a r.
          */
-        bool isFree(unsigned r) const;
+        bool isFree(size_t r) const;
         /**
          * Determines whether this is the non-trivial cyclic group on
          * the given number of elements.
@@ -438,7 +370,7 @@ class AbelianGroup : public ShortOutput<AbelianGroup, true> {
          * @param n the number of elements of the cyclic group in question.
          * @return \c true if and only if this is the cyclic group Z_n.
          */
-        bool isZn(unsigned long n) const;
+        bool isZn(size_t n) const;
         /**
          * Determines whether this and the given abelian group have
          * identical presentations (which means they are isomorphic).
@@ -500,7 +432,8 @@ class AbelianGroup : public ShortOutput<AbelianGroup, true> {
          * Writes the tight encoding of this abelian group to the given output
          * stream.  See the page on \ref tight "tight encodings" for details.
          *
-         * \ifacespython Not present; use tightEncoding() instead.
+         * \ifacespython Not present; use tightEncoding() instead, which
+         * returns a string.
          *
          * @param out the output stream to which the encoded string will
          * be written.
@@ -508,12 +441,28 @@ class AbelianGroup : public ShortOutput<AbelianGroup, true> {
         void tightEncode(std::ostream& out) const;
 
         /**
-         * Returns the tight encoding of this abelian group.
+         * Reconstructs an abelian group from its given tight encoding.
          * See the page on \ref tight "tight encodings" for details.
          *
-         * @return the resulting encoded string.
+         * The tight encoding will be read from the given input stream.
+         * If the input stream contains leading whitespace then it will be
+         * treated as an invalid encoding (i.e., this routine will throw an
+         * exception).  The input routine \e may contain further data: if this
+         * routine is successful then the input stream will be left positioned
+         * immediately after the encoding, without skipping any trailing
+         * whitespace.
+         *
+         * \exception InvalidInput the given input stream does not begin with
+         * a tight encoding of an abelian group.
+         *
+         * \ifacespython Not present; use tightDecoding() instead, which takes
+         * a string as its argument.
+         *
+         * @param input an input stream that begins with the tight encoding
+         * for an abelian group.
+         * @return the abelian group represented by the given tight encoding.
          */
-        std::string tightEncoding() const;
+        static AbelianGroup tightDecode(std::istream& input);
 
         /**
          * Writes a chunk of XML containing this abelian group.
@@ -560,14 +509,11 @@ void swap(AbelianGroup& lhs, AbelianGroup& rhs) noexcept;
 
 // Inline functions for AbelianGroup
 
-inline AbelianGroup::AbelianGroup() {
-}
-
-inline AbelianGroup::AbelianGroup(unsigned rank) : rank_(rank) {
+inline AbelianGroup::AbelianGroup(size_t rank) : rank_(rank) {
 }
 
 template <typename T>
-inline AbelianGroup::AbelianGroup(unsigned rank,
+inline AbelianGroup::AbelianGroup(size_t rank,
         std::initializer_list<T> invFac) : rank_(rank) {
     if (invFac.size() > 0) {
         auto it = std::rbegin(invFac);
@@ -587,7 +533,7 @@ inline AbelianGroup::AbelianGroup(unsigned rank,
 }
 
 template <typename Container>
-inline AbelianGroup::AbelianGroup(unsigned rank, const Container& invFac) :
+inline AbelianGroup::AbelianGroup(size_t rank, const Container& invFac) :
         rank_(rank) {
     if (! invFac.empty()) {
         auto it = invFac.rbegin();
@@ -611,28 +557,15 @@ inline void AbelianGroup::swap(AbelianGroup& other) noexcept {
     revInvFactors_.swap(other.revInvFactors_);
 }
 
-inline void AbelianGroup::addRank(int extraRank) {
+inline void AbelianGroup::addRank(long extraRank) {
     rank_ += extraRank;
 }
 
-inline void AbelianGroup::addTorsionElement(const Integer& degree,
-        unsigned mult) {
-    for ( ; mult > 0; --mult)
-        addTorsion(degree);
-}
-
-inline void AbelianGroup::addTorsionElement(unsigned long degree,
-        unsigned mult) {
-    Integer d(degree);
-    for ( ; mult > 0; --mult)
-        addTorsion(d);
-}
-
-inline unsigned AbelianGroup::rank() const {
+inline size_t AbelianGroup::rank() const {
     return rank_;
 }
 
-inline unsigned AbelianGroup::torsionRank(unsigned long degree) const {
+inline size_t AbelianGroup::torsionRank(unsigned long degree) const {
     return torsionRank(Integer(degree));
 }
 
@@ -652,11 +585,11 @@ inline bool AbelianGroup::isZ() const {
     return (rank_ == 1 && revInvFactors_.empty());
 }
 
-inline bool AbelianGroup::isFree(unsigned r) const {
+inline bool AbelianGroup::isFree(size_t r) const {
     return (rank_ == r && revInvFactors_.empty());
 }
 
-inline bool AbelianGroup::isZn(unsigned long n) const {
+inline bool AbelianGroup::isZn(size_t n) const {
     return (n == 0 ? isZ() : n == 1 ? isTrivial() :
         (rank_ == 0 && revInvFactors_.size() == 1 &&
             revInvFactors_.front() == n));
@@ -677,12 +610,6 @@ inline void AbelianGroup::tightEncode(std::ostream& out) const {
     regina::tightEncode(out, 0);
 }
 
-inline std::string AbelianGroup::tightEncoding() const {
-    std::ostringstream out;
-    tightEncode(out);
-    return out.str();
-}
-
 inline void swap(AbelianGroup& a, AbelianGroup& b) noexcept {
     a.swap(b);
 }
