@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -363,22 +363,6 @@ class HomGroupPresentation : public Output<HomGroupPresentation> {
          */
         HomGroupPresentation operator * (HomGroupPresentation&& rhs) const;
 
-        /**
-         * Deprecated routine that composes this homomorphism with the
-         * given homomorphism.
-         *
-         * \deprecated Instead of <tt>a.composeWith(b)</tt>, use the
-         * multiplication operator <tt>a * b</tt>.
-         *
-         * \pre the codomain of \a rhs must be the same as the domain of this
-         * homomorphism.
-         *
-         * @param rhs the homomorphism to compose this with.
-         * @return the composition of both homomorphisms.
-         */
-        [[deprecated]] HomGroupPresentation composeWith(
-            const HomGroupPresentation& rhs) const;
-
         /**
          * Inverts the homomorphism.
          *
@@ -560,11 +544,6 @@ inline GroupExpression HomGroupPresentation::invEvaluate(unsigned long i)
     return (*inv_)[i];
 }
 
-inline HomGroupPresentation HomGroupPresentation::composeWith(
-        const HomGroupPresentation& rhs) const {
-    return (*this) * rhs;
-}
-
 inline void swap(HomGroupPresentation& a, HomGroupPresentation& b) noexcept {
     a.swap(b);
 }
