@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -133,8 +133,7 @@ class MarkedAbelianGroup : public ShortOutput<MarkedAbelianGroup, true> {
         /** Internal change of basis. M_ = MCi*SNF(M_)*MRi_ */
         MatrixInt MRi_;
         /** Internal rank of M */
-        unsigned long rankM_; // this is the index of the first zero entry
-                              // in SNF(M_)
+        size_t rankM_; // this is the index of the first zero entry in SNF(M_)
 
         /* Internal reduced N matrix: */
         // In the notes below, pres refers to the internal presentation
@@ -152,11 +151,11 @@ class MarkedAbelianGroup : public ShortOutput<MarkedAbelianGroup, true> {
         /** Internal list of invariant factors. */
         std::vector<Integer> invFac_;
         /** The number of free generators, from SNF(pres) */
-        unsigned long snfFreeRank_;
+        size_t snfFreeRank_;
         /** The row index of the first zero along the diagonal in SNF(pres). */
-        unsigned long snfFreeIndex_;
+        size_t snfFreeIndex_;
         /** Row index of first invariant factor (ie entry > 1) in SNF(pres) */
-        unsigned long ifLoc_;
+        size_t ifLoc_;
 
         // These variables store information for mod-p homology computations.
         /** coefficients to use in homology computation **/
@@ -167,7 +166,7 @@ class MarkedAbelianGroup : public ShortOutput<MarkedAbelianGroup, true> {
 
         /** invariant factor data in the tensor product presentation matrix
             SNF */
-        unsigned long tensorIfLoc_;
+        size_t tensorIfLoc_;
         std::vector<Integer> tensorInvFac_;
 
     public:
@@ -236,7 +235,7 @@ class MarkedAbelianGroup : public ShortOutput<MarkedAbelianGroup, true> {
          * @param p describes the type of ring that we use to talk about
          * the "free" module.
          */
-        MarkedAbelianGroup(unsigned long rank, const Integer &p);
+        MarkedAbelianGroup(size_t rank, const Integer &p);
 
         /**
          * Creates a clone of the given group.
@@ -289,15 +288,6 @@ class MarkedAbelianGroup : public ShortOutput<MarkedAbelianGroup, true> {
          */
         const MatrixInt& m() const;
 
-        /**
-         * Deprecated alias for m().
-         *
-         * \deprecated This routine has been renamed to lower-case m().
-         *
-         * @return a reference to the defining matrix \a M.
-         */
-        [[deprecated]] const MatrixInt& M() const;
-
         /**
          * Returns the "left" matrix that was used to define the chain complex.
          * Our group was defined as the kernel of \a M modulo the image of \a N.
@@ -311,15 +301,6 @@ class MarkedAbelianGroup : public ShortOutput<MarkedAbelianGroup, true> {
          */
         const MatrixInt& n() const;
 
-        /**
-         * Deprecated alias for n().
-         *
-         * \deprecated This routine has been renamed to lower-case n().
-         *
-         * @return a reference to the defining matrix \a N.
-         */
-        [[deprecated]] const MatrixInt& N() const;
-
         /**
          * Returns the coefficients used for the computation of homology.
          * That is, this routine returns the integer \a p where we use
@@ -340,7 +321,7 @@ class MarkedAbelianGroup : public ShortOutput<MarkedAbelianGroup, true> {
          *
          * @return the rank of the group.
          */
-        unsigned long rank() const;
+        size_t rank() const;
 
         /**
          * Returns the rank in the group of the torsion term of given degree.
@@ -358,7 +339,7 @@ class MarkedAbelianGroup : public ShortOutput<MarkedAbelianGroup, true> {
          * @param degree the degree of the torsion term to query.
          * @return the rank in the group of the given torsion term.
          */
-        unsigned long torsionRank(const Integer& degree) const;
+        size_t torsionRank(const Integer& degree) const;
 
         /**
          * Returns the rank in the group of the torsion term of given degree.
@@ -376,7 +357,7 @@ class MarkedAbelianGroup : public ShortOutput<MarkedAbelianGroup, true> {
          * @param degree the degree of the torsion term to query.
          * @return the rank in the group of the given torsion term.
          */
-        unsigned long torsionRank(unsigned long degree) const;
+        size_t torsionRank(unsigned long degree) const;
 
         /**
          * Returns the number of invariant factors that describe the
@@ -407,16 +388,7 @@ class MarkedAbelianGroup : public ShortOutput<MarkedAbelianGroup, true> {
          *
          * @return the rank of the chain complex.
          */
-        unsigned long ccRank() const;
-
-        /**
-         * Deprecated alias for ccRank().
-         *
-         * \deprecated This routine has been renamed to ccRank().
-         *
-         * @return the rank of the chain complex.
-         */
-        [[deprecated]] unsigned long rankCC() const;
+        size_t ccRank() const;
 
         /**
          * Returns the minimum number of generators for this group.
@@ -426,16 +398,7 @@ class MarkedAbelianGroup : public ShortOutput<MarkedAbelianGroup, true> {
          *
          * @return the minimum number of generators.
          */
-        unsigned long snfRank() const;
-
-        /**
-         * Deprecated alias for snfRank().
-         *
-         * \deprecated This routine has been renamed to snfRank().
-         *
-         * @return the minimum number of generators.
-         */
-        [[deprecated]] unsigned long minNumberOfGenerators() const;
+        size_t snfRank() const;
 
         /**
          * Returns the given invariant factor describing the torsion
@@ -504,19 +467,6 @@ class MarkedAbelianGroup : public ShortOutput<MarkedAbelianGroup, true> {
          */
         bool operator != (const MarkedAbelianGroup& other) const;
 
-        /**
-         * Deprecated routine that determines whether this and the given
-         * group were formed from identical chain complex constructions.
-         *
-         * \deprecated This routine is now offered as the equality
-         * comparison operator (==).
-         *
-         * @param other the group with which this should be compared.
-         * @return \c true if and only if the this and the given group have
-         * identical chain complex definitions.
-         */
-        [[deprecated]] bool equalTo(const MarkedAbelianGroup& other) const;
-
         /**
          * Returns the requested free generator of this group,
          * represented in the original chain complex defining the group.
@@ -534,7 +484,7 @@ class MarkedAbelianGroup : public ShortOutput<MarkedAbelianGroup, true> {
          * @return the (\a index)th free generator, presented in
          * chain complex coordinates.
          */
-        Vector<Integer> freeRep(unsigned long index) const;
+        Vector<Integer> freeRep(size_t index) const;
 
         /**
          * Returns the requested generator of the torsion subgroup,
@@ -553,7 +503,7 @@ class MarkedAbelianGroup : public ShortOutput<MarkedAbelianGroup, true> {
          * @return the (\a index)th torsion generator, presented in
          * chain complex coordinates.
          */
-        Vector<Integer> torsionRep(unsigned long index) const;
+        Vector<Integer> torsionRep(size_t index) const;
 
         /**
          * A combination of freeRep and torsionRep, this routine takes
@@ -593,7 +543,7 @@ class MarkedAbelianGroup : public ShortOutput<MarkedAbelianGroup, true> {
          * SNF coordinates; this must be between 0 and snfRank()-1 inclusive.
          * @return a corresponding vector in chain complex coordinates.
          */
-        Vector<Integer> ccRep(unsigned long snfGen) const;
+        Vector<Integer> ccRep(size_t snfGen) const;
 
         /**
          * Expresses the given cycle as a combination of free and torsion
@@ -663,7 +613,7 @@ class MarkedAbelianGroup : public ShortOutput<MarkedAbelianGroup, true> {
          * @return the resulting projection, in the chain complex
          * coordinates.
          */
-        Vector<Integer> cycleProjection(unsigned long ccindx) const;
+        Vector<Integer> cycleProjection(size_t ccindx) const;
 
         /**
          * Determines whether the given vector represents a cycle in the
@@ -688,20 +638,6 @@ class MarkedAbelianGroup : public ShortOutput<MarkedAbelianGroup, true> {
          */
         Vector<Integer> boundaryOf(const Vector<Integer>& chain) const;
 
-        /**
-         * Deprecated alias for boundaryOf().
-         *
-         * \deprecated This routine has been renamed to boundaryOf().
-         *
-         * \exception InvalidArgument The given vector is not in chain
-         * complex coordinates; that is, its length is not M.columns().
-         *
-         * @param chain any vector in chain complex coordinates.
-         * @return the differential, expressed as a vector of length M.rows().
-         */
-        [[deprecated]] Vector<Integer> boundaryMap(const Vector<Integer>& chain)
-            const;
-
         /**
          * Determines whether the given vector represents a boundary in the
          * chain complex.
@@ -730,35 +666,13 @@ class MarkedAbelianGroup : public ShortOutput<MarkedAbelianGroup, true> {
          */
         Vector<Integer> asBoundary(const Vector<Integer>& bdry) const;
 
-        /**
-         * Deprecated alias for asBoundary().
-         *
-         * \deprecated This routine has been renamed to asBoundary().
-         *
-         * \exception InvalidArgument The given vector is not a boundary.
-         *
-         * @param bdry a boundary vector, given in chain complex coordinates.
-         * @return a vector \a v such that <tt>N*v=bdry</tt>.
-         */
-        [[deprecated]] Vector<Integer> writeAsBoundary(
-            const Vector<Integer>& bdry) const;
-
         /**
          * Returns the number of generators of the kernel of \a M, where
          * \a M is the "right" matrix used to define the chain complex.
          *
          * @return the number of generators of ker(\a M).
          */
-        unsigned long cycleRank() const;
-
-        /**
-         * Deprecated alias for cycleRank().
-         *
-         * \deprecated This routine has been renamed to cycleRank().
-         *
-         * @return the number of generators of ker(\a M).
-         */
-        [[deprecated]] unsigned long minNumberCycleGens() const;
+        size_t cycleRank() const;
 
         /**
          * Returns the requested generator of the cycles, i.e., the kernel of
@@ -775,7 +689,7 @@ class MarkedAbelianGroup : public ShortOutput<MarkedAbelianGroup, true> {
          * @return the (\a index)th generator of the cycles, expressed in
          * chain complex coordinates.
          */
-        Vector<Integer> cycleGen(unsigned long index) const;
+        Vector<Integer> cycleGen(size_t index) const;
 
         /**
          * Returns the torsion subgroup of this group.
@@ -1242,25 +1156,6 @@ class HomMarkedAbelianGroup : public Output<HomMarkedAbelianGroup> {
          */
         HomMarkedAbelianGroup torsionSubgroup() const;
 
-        /**
-         * Deprecated routine that writes a human-readable version of the
-         * reduced matrix to the given output stream.  This is a description
-         * of the homomorphism in some specific coordinates at present only
-         * meant to be internal to HomMarkedAbelianGroup.  At present, these
-         * coordinates have the torsion factors of the group appearing first,
-         * followed by the free factors.
-         *
-         * \deprecated All of the information that this routine outputs
-         * is now written by writeTextLong() and returned in string form
-         * by detail().  Use those routines instead.
-         *
-         * \ifacespython Not present; instead use detail(), which returns a
-         * string including all this information (plus a little more).
-         *
-         * @param out the output stream.
-         */
-        [[deprecated]] void writeReducedMatrix(std::ostream& out) const;
-
     private:
         /**
          * For those situations where you want to define an
@@ -1320,8 +1215,7 @@ inline const AbelianGroup& HomMarkedAbelianGroup::cokernel() const {
 
 // Inline functions for MarkedAbelianGroup
 
-inline unsigned long MarkedAbelianGroup::torsionRank(unsigned long degree)
-        const {
+inline size_t MarkedAbelianGroup::torsionRank(unsigned long degree) const {
     return torsionRank(Integer(degree));
 }
 
@@ -1334,7 +1228,7 @@ inline const Integer& MarkedAbelianGroup::invariantFactor(
     return invFac_[index];
 }
 
-inline unsigned long MarkedAbelianGroup::rank() const {
+inline size_t MarkedAbelianGroup::rank() const {
     return snfFreeRank_;
 }
 
@@ -1342,30 +1236,18 @@ inline AbelianGroup MarkedAbelianGroup::unmarked() const {
     return AbelianGroup(snfFreeRank_, invFac_);
 }
 
-inline unsigned long MarkedAbelianGroup::snfRank() const {
+inline size_t MarkedAbelianGroup::snfRank() const {
     return snfFreeRank_ + invFac_.size();
 }
 
-inline unsigned long MarkedAbelianGroup::minNumberOfGenerators() const {
-    return snfRank();
-}
-
-inline unsigned long MarkedAbelianGroup::ccRank() const {
+inline size_t MarkedAbelianGroup::ccRank() const {
     return M_.columns();
 }
 
-inline unsigned long MarkedAbelianGroup::rankCC() const {
-    return M_.columns();
-}
-
-inline unsigned long MarkedAbelianGroup::cycleRank() const {
+inline size_t MarkedAbelianGroup::cycleRank() const {
     return M_.columns() - (rankM_ - TORVec_.size());
 }
 
-inline unsigned long MarkedAbelianGroup::minNumberCycleGens() const {
-    return cycleRank();
-}
-
 inline bool MarkedAbelianGroup::isTrivial() const {
     return ( (snfFreeRank_==0) && invFac_.empty() );
 }
@@ -1384,10 +1266,6 @@ inline bool MarkedAbelianGroup::operator != (const MarkedAbelianGroup& other)
     return (M_ != other.M_) || (N_ != other.N_) || (coeff_ != other.coeff_);
 }
 
-inline bool MarkedAbelianGroup::equalTo(const MarkedAbelianGroup& other) const {
-    return (M_ == other.M_) && (N_ == other.N_) && (coeff_ == other.coeff_);
-}
-
 inline bool MarkedAbelianGroup::isIsomorphicTo(
         const MarkedAbelianGroup &other) const {
     return ((invFac_ == other.invFac_) && (snfFreeRank_ == other.snfFreeRank_));
@@ -1396,24 +1274,13 @@ inline bool MarkedAbelianGroup::isIsomorphicTo(
 inline const MatrixInt& MarkedAbelianGroup::m() const {
     return M_;
 }
-inline const MatrixInt& MarkedAbelianGroup::M() const {
-    return M_;
-}
 inline const MatrixInt& MarkedAbelianGroup::n() const {
     return N_;
 }
-inline const MatrixInt& MarkedAbelianGroup::N() const {
-    return N_;
-}
 inline const Integer& MarkedAbelianGroup::coefficients() const {
     return coeff_;
 }
 
-inline Vector<Integer> MarkedAbelianGroup::boundaryMap(
-        const Vector<Integer>& CCrep) const {
-    return boundaryOf(CCrep);
-}
-
 inline void swap(MarkedAbelianGroup& a, MarkedAbelianGroup& b) noexcept {
     a.swap(b);
 }
