@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -236,16 +236,6 @@ class AngleStructure : public ShortOutput<AngleStructure> {
         AngleStructure(const SnapshotRef<Triangulation<3>>& triang,
             Vector<Integer>&& vector);
 
-        /**
-         * Deprecated routine that creates a newly allocated clone of this
-         * angle structure.
-         *
-         * \deprecated Simply use the copy constructor instead.
-         *
-         * @return a clone of this angle structure.
-         */
-        [[deprecated]] AngleStructure* clone() const;
-
         /**
          * Sets this to be a copy of the given angle structure.
          *
@@ -476,15 +466,6 @@ class AngleStructure : public ShortOutput<AngleStructure> {
          */
         bool operator < (const AngleStructure& other) const;
 
-        /**
-         * A deprecated alias for vector().
-         *
-         * \deprecated This routine has been renamed to vector().
-         *
-         * @return the underlying integer vector.
-         */
-        [[deprecated]] const Vector<Integer>& rawVector() const;
-
         /**
          * Writes a short text representation of this object to the
          * given output stream.
@@ -568,10 +549,6 @@ inline AngleStructure::AngleStructure(const AngleStructure& src,
         flags_(src.flags_) {
 }
 
-inline AngleStructure* AngleStructure::clone() const {
-    return new AngleStructure(*this);
-}
-
 inline void AngleStructure::swap(AngleStructure& other) noexcept {
     vector_.swap(other.vector_);
     triangulation_.swap(other.triangulation_);
@@ -612,10 +589,6 @@ inline const Vector<Integer>& AngleStructure::vector() const {
     return vector_;
 }
 
-inline const Vector<Integer>& AngleStructure::rawVector() const {
-    return vector_;
-}
-
 inline void swap(AngleStructure& a, AngleStructure& b) noexcept {
     a.swap(b);
 }
