@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -316,12 +316,22 @@ class AngleStructures :
         size_t size() const;
         /**
          * Returns the angle structure at the requested index in this list.
+         * This is identical to using the square bracket operator.
          *
          * @param index the index of the requested angle structure in
          * this list; this must be between 0 and size()-1 inclusive.
          * @return the angle structure at the requested index.
          */
         const AngleStructure& structure(size_t index) const;
+        /**
+         * Returns the angle structure at the requested index in this list.
+         * This is identical to calling structure().
+         *
+         * @param index the index of the requested angle structure in
+         * this list; this must be between 0 and size()-1 inclusive.
+         * @return the angle structure at the requested index.
+         */
+        const AngleStructure& operator [](size_t index) const;
 
         /**
          * Returns an iterator at the beginning of this list of angle
@@ -442,88 +452,6 @@ class AngleStructures :
          */
         bool operator != (const AngleStructures& other) const;
 
-        /**
-         * Deprecated routine to enumerate angle structures on a given
-         * triangulation.
-         *
-         * This static routine is identical to calling the class
-         * constructor with the given arguments, but with two differences:
-         *
-         * - If a progress tracker is passed, this routine will start the
-         *   enumeration in a detached background thread and return immediately
-         *   (unlike the class constructor, which does not return until the
-         *   enumeration is finished).
-         *
-         * - This routine wraps the new angle structure list in a packet and
-         *   inserts it beneath \a owner in the packet tree (unlike the class
-         *   constructor, which creates a plain AngleStructures object).
-         *   If a progress tracker is passed (i.e., the enumeration runs in a
-         *   background thread) then this tree insertion will not happen until
-         *   the enumeration has finished, and if the user cancels the
-         *   operation then the insertion will not happen at all.
-         *
-         * This function is safe to use even if \a owner is a "pure"
-         * Triangulation<3> or SnapPeaTriangulation, not a packet type.
-         * In such a scenario, this routine will still build the angle
-         * structure list, but the resulting packet will be orphaned.
-         *
-         * See the class constructor for details on how this routine
-         * works and what the various arguments mean.
-         *
-         * \deprecated Just call the AngleStructures constructor.
-         *
-         * \ifacespython For this deprecated function, the progress tracker
-         * argument is omitted.  It is still possible to enumerate in
-         * the background with a progress tracker, but for that you will
-         * need to call the class constructor instead and create the new
-         * thread yourself.
-         *
-         * @param owner the triangulation for which the vertex
-         * angle structures will be enumerated.
-         * @param tautOnly \c true if only taut structures are to be
-         * enuemrated, or \c false if we should enumerate all vertices
-         * of the angle structure solution space; this defaults to \c false.
-         * @param tracker a progress tracker through which progress will
-         * be reported, or \c null if no progress reporting is required.
-         * @return the new angle structure list.
-         */
-        [[deprecated]] static std::shared_ptr<PacketOf<AngleStructures>>
-            enumerate(Triangulation<3>& owner, bool tautOnly = false,
-                ProgressTracker* tracker = nullptr);
-
-        /**
-         * Deprecated routine to enumerate all taut angle structures
-         * on the given triangulation using the double description method.
-         *
-         * This static routine is almost identical to calling the class
-         * constructor with the \c tautOnly argument set to \c true and
-         * the \a algHints argument set to AS_ALG_DD.  The only difference is
-         * that, unlike the class constructor, this routine will also wrap
-         * the new angle structure list in a packet and insert it beneath
-         * \a owner in the packet tree.
-         *
-         * This function is safe to use even if \a owner is a "pure"
-         * Triangulation<3> or SnapPeaTriangulation, not a packet type.
-         * In such a scenario, this routine will still build the angle
-         * structure list, but the resulting packet will be orphaned.
-         *
-         * Note that default algorithm used by the class constructor for
-         * taut angle structures is the tree traversal method, which is
-         * based on linear programming, and which is typically \e much faster
-         * than the double description method for larger triangulations.
-         *
-         * \warning This routine is slow, and users will not want to call it
-         * unless they have some specialised need.
-         *
-         * \deprecated Just call the AngleStructures constructor.
-         *
-         * @param owner the triangulation for which the taut angle structures
-         * will be enumerated.
-         * @return the new angle structure list.
-         */
-        [[deprecated]] static std::shared_ptr<PacketOf<AngleStructures>>
-            enumerateTautDD(Triangulation<3>& owner);
-
         /**
          * Writes a short text representation of this object to the
          * given output stream.
@@ -691,6 +619,10 @@ inline const AngleStructure& AngleStructures::structure(size_t index) const {
     return structures_[index];
 }
 
+inline const AngleStructure& AngleStructures::operator [](size_t index) const {
+    return structures_[index];
+}
+
 inline auto AngleStructures::begin() const {
     return structures_.begin();
 }
