@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -125,10 +125,10 @@ class GluingPermSearcher<2> : public ShortOutput<GluingPermSearcher<2>> {
                  of the two identified triangle edges (unless otherwise
                  specified by a subclass that uses a specialised search
                  algorithm. */
-        int orderSize;
+        size_t orderSize;
             /**< The total number of edges in the edge pairing graph, i.e.,
                  the number of elements of interest in the order[] array. */
-        int orderElt;
+        ssize_t orderElt;
             /**< Marks which element of order[] we are currently examining
                  at this stage of the search. */
 
@@ -253,7 +253,7 @@ class GluingPermSearcher<2> : public ShortOutput<GluingPermSearcher<2>> {
          * \a maxDepth levels of this search tree.  Rather than
          * producing complete gluing permutation sets, this search will
          * produce a series of partially-constructed permutation sets.
-         * A partial searche can be continued by calling runSearch()
+         * A partial search can be continued by calling runSearch()
          * again on the underlying GluingPermSearcher (perhaps after being
          * frozen, or passed on to a different processor via taggedData() and
          * fromTaggedData()).  If necessary, the \a action routine may call
@@ -293,18 +293,6 @@ class GluingPermSearcher<2> : public ShortOutput<GluingPermSearcher<2>> {
          */
         bool isComplete() const;
 
-        /**
-         * Deprecated function that determines whether this search manager
-         * holds a complete gluing permutation set or just a partially
-         * completed search state.
-         *
-         * \deprecated This routine has been renamed to isComplete().
-         *
-         * @return \c true if a complete gluing permutation set is held,
-         * or \c false otherwise.
-         */
-        [[deprecated]] bool completePermSet() const;
-
         /**
          * Dumps all internal data in a plain text format, along with a
          * marker to signify which precise class the data belongs to.
@@ -413,7 +401,7 @@ class GluingPermSearcher<2> : public ShortOutput<GluingPermSearcher<2>> {
          * input stream constructor is not.  Python users should use
          * taggedData() and fromTaggedData() instead.
          *
-         * @param all of this object's internal data in plain text format.
+         * @return all of this object's internal data in plain text format.
          */
         std::string data() const;
 
@@ -601,11 +589,7 @@ inline void GluingPermSearcher<2>::partialSearch(long maxDepth,
 }
 
 inline bool GluingPermSearcher<2>::isComplete() const {
-    return (orderElt == orderSize);
-}
-
-inline bool GluingPermSearcher<2>::completePermSet() const {
-    return (orderElt == orderSize);
+    return (orderElt == static_cast<ssize_t>(orderSize));
 }
 
 inline void GluingPermSearcher<2>::dumpTaggedData(std::ostream& out) const {
