@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -276,8 +276,7 @@ struct OutputBase {
          * str() or detail() at all.  Instead, it is based purely on the
          * inheritance condition as stated above.
          */
-        using type = typename std::remove_reference<
-            decltype(test(std::declval<T>()))>::type;
+        using type = std::remove_reference_t<decltype(test(std::declval<T>()))>;
 };
 
 // Inline functions
