@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 2011-2021, Ben Burton                                   *
+ *  Copyright (c) 2011-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -270,7 +270,7 @@ class LPConstraintBase {
          */
         template <typename IntType>
         static void constrain(LPData<LPConstraintNone, IntType>& lp,
-            unsigned numCols);
+            size_t numCols);
 
         /**
          * Ensures that the given normal surface satisfies the extra
@@ -398,7 +398,7 @@ class LPConstraintNone : public LPConstraintSubspace {
             const LPInitialTableaux<LPConstraintNone>& init);
         template<typename IntType>
         static void constrain(
-            LPData<regina::LPConstraintNone, IntType>&, unsigned);
+            LPData<regina::LPConstraintNone, IntType>&, size_t);
         static bool verify(const NormalSurface&);
         static bool verify(const AngleStructure&);
         static bool supported(NormalEncoding enc);
@@ -463,24 +463,12 @@ class LPConstraintEulerPositive : public LPConstraintBase {
         template<typename IntType>
         static void constrain(
             LPData<regina::LPConstraintEulerPositive, IntType>& lp,
-            unsigned numCols);
+            size_t numCols);
         static bool verify(const NormalSurface& s);
         static bool verify(const AngleStructure&);
         static bool supported(NormalEncoding enc);
 };
 
-/**
- * A deprecated type alias for LPConstraintEulerPositive.
- *
- * The old name LPConstraintEuler should no longer be used, since Regina
- * now provides multiple constraint types relating to Euler characteristic.
- *
- * \ifacespython Not present; use LPConstraintEulerPositive instead.
- *
- * \ingroup enumerate
- */
-using LPConstraintEuler [[deprecated]] = LPConstraintEulerPositive;
-
 /**
  * A class that constraints the tableaux of normal surface matching equations
  * to ensure that Euler characteristic is zero.
@@ -526,7 +514,7 @@ class LPConstraintEulerZero : public LPConstraintSubspace {
         template<typename IntType>
         static void constrain(
             LPData<regina::LPConstraintEulerZero, IntType>& lp,
-            unsigned numCols);
+            size_t numCols);
         static bool verify(const NormalSurface& s);
         static bool verify(const AngleStructure&);
         static bool supported(NormalEncoding enc);
@@ -591,7 +579,7 @@ class LPConstraintNonSpun : public LPConstraintSubspace {
         template <typename IntType>
         static void constrain(
             LPData<regina::LPConstraintNonSpun, IntType>& lp,
-            unsigned numCols);
+            size_t numCols);
         static bool verify(const NormalSurface& s);
         static bool verify(const AngleStructure&);
         static bool supported(NormalEncoding enc);
@@ -647,11 +635,13 @@ class LPConstraintNonSpun : public LPConstraintSubspace {
  * coordinates).  See LPInitialTableaux for a more detailed discussion of this.
  *
  * This base class provides limited functionality (as documented below).
- * Subclasses \e must implement a constructor (which, like this base
- * class, takes an initial tableaux and determines which coordinates are
- * banned and/or marked), and must implement supported(), which indicates
- * which normal or angle structure coordinate system this constraint class
- * can work with.
+ * Subclasses \e must implement a constructor (which, like this base class,
+ * takes an initial tableaux and determines which coordinates are banned and/or
+ * marked), and must implement supported(), which indicates which normal or
+ * angle structure coordinate system this constraint class can work with.
+ * The constructor may take additional arguments beyond the initial tableaux;
+ * if so, then the tree traversal classes (mentioned below) will forward
+ * these arguments at runtime from their own class constructors.
  *
  * These ban constraint classes are designed mainly to act as C++ template
  * arguments, and end users will typically not need to construct their own
@@ -861,8 +851,8 @@ class BanNone : public ShortOutput<BanNone> {
         template <class LPConstraint, typename IntType>
         void enforceBans(LPData<LPConstraint, IntType>&) const {}
 
-        bool operator == (const BanNone& other) const { return true; }
-        bool operator != (const BanNone& other) const { return false; }
+        bool operator == (const BanNone&) const { return true; }
+        bool operator != (const BanNone&) const { return false; }
 
         void writeTextShort(std::ostream& out) const {
             out << "Nothing banned or marked";
@@ -935,6 +925,70 @@ class BanBoundary : public BanConstraintBase {
         static bool supported(NormalEncoding enc);
 };
 
+/**
+ * A class that bans normal disc types that meet a particular edge of the
+ * underlying triangulation.  No disc types are marked at all.
+ *
+ * This class is only for use with normal or almost normal surfaces, not
+ * angle structures.
+ *
+ * \warning This class only works as expected with vector encodings that
+ * explicitly include triangles (e.g., encodings for standard normal or
+ * almost normal coordinates).  In quadrilateral or quadrilateral-octagon
+ * coordinates it will only ban quadrilaterals or octagons that meet the
+ * given edge, but it will still allow \e triangles that meet the edge
+ * (since triangle types are not counted in these coordinate systems).
+ * The supported() routine will only return \c true for encodings that
+ * include triangles.
+ *
+ * See the BanConstraintBase class notes for details on all member
+ * functions and structs.
+ *
+ * These ban constraint classes are designed mainly to act as C++ template
+ * arguments, and end users will typically not need to construct their own
+ * object of these classes.  Instead, to use a ban constraint class, pass it
+ * as a template parameter to one of the tree traversal subclasses
+ * (e.g., TreeEnumeration, TreeSingleSolution, or TautEnumeration).
+ *
+ * \headers Some templated parts of this class are implemented in a separate
+ * header (treeconstraint-impl.h), which is not included automatically by this
+ * file.  Most end users should not need this extra header, since Regina's
+ * calculation engine already includes explicit instantiations for common
+ * template arguments.
+ *
+ * \ifacespython It is rare that you would need to access this class directly
+ * through Python.  Instead, to use a ban constraint class, you would typically
+ * create a tree traversal object with the appropriate class suffix (e.g., one
+ * such Python class is \c TreeEnumeration_BanEdge).  See the
+ * BanConstraintBase class notes for further details.
+ *
+ * \apinotfinal
+ *
+ * \ingroup enumerate
+ */
+class BanEdge : public BanConstraintBase {
+    public:
+        /**
+         * Constructs a new set of banning and marking constraints.
+         *
+         * This base class constructor will construct the \a banned_ and
+         * \a marked_ arrays to be the correct size based on the given
+         * tableaux, and will initialise their contents to ban disc types
+         * that meet the given edge.
+         *
+         * No disc types will be marked.
+         *
+         * @param init the original starting tableaux being used for this
+         * enumeration task.  This tableaux must work with normal or almost
+         * normal surface coordinates (not angle structure coordinates).
+         * @param edge the specific edge that our normal discs must not meet.
+         */
+        template <class LPConstraint>
+        BanEdge(const LPInitialTableaux<LPConstraint>& init, Edge<3>* edge);
+
+        static bool supported(NormalEncoding enc);
+};
+
 /**
  * A class that bans and marks disc types associated with torus boundary
  * components.  Here we refer exclusively to real torus boundary
@@ -1019,7 +1073,7 @@ inline void LPConstraintNone::addRows(
 
 template <typename IntType>
 inline void LPConstraintNone::constrain(
-        LPData<regina::LPConstraintNone, IntType>&, unsigned) {
+        LPData<regina::LPConstraintNone, IntType>&, size_t) {
 }
 
 inline bool LPConstraintNone::verify(const NormalSurface&) {
@@ -1037,7 +1091,7 @@ inline bool LPConstraintNone::supported(NormalEncoding) {
 template <typename IntType>
 inline void LPConstraintEulerPositive::constrain(
         LPData<regina::LPConstraintEulerPositive, IntType>& lp,
-        unsigned numCols) {
+        size_t numCols) {
     lp.constrainPositive(numCols - 1);
 }
 
@@ -1056,7 +1110,7 @@ inline bool LPConstraintEulerPositive::supported(NormalEncoding enc) {
 
 template <typename IntType>
 inline void LPConstraintEulerZero::constrain(
-        LPData<regina::LPConstraintEulerZero, IntType>& lp, unsigned numCols) {
+        LPData<regina::LPConstraintEulerZero, IntType>& lp, size_t numCols) {
     lp.constrainZero(numCols - 1);
 }
 
@@ -1075,7 +1129,7 @@ inline bool LPConstraintEulerZero::supported(NormalEncoding enc) {
 
 template <typename IntType>
 inline void LPConstraintNonSpun::constrain(
-        LPData<regina::LPConstraintNonSpun, IntType>& lp, unsigned numCols) {
+        LPData<regina::LPConstraintNonSpun, IntType>& lp, size_t numCols) {
     lp.constrainZero(numCols - 2);
     lp.constrainZero(numCols - 1);
 }
@@ -1111,7 +1165,7 @@ inline BanConstraintBase::~BanConstraintBase() {
 template <class LPConstraint, typename IntType>
 inline void BanConstraintBase::enforceBans(LPData<LPConstraint, IntType>& lp)
         const {
-    for (unsigned i = 0; i < lp.coordinateColumns(); ++i)
+    for (size_t i = 0; i < lp.coordinateColumns(); ++i)
         if (banned_[i])
             lp.constrainZero(i);
 }
@@ -1140,6 +1194,11 @@ inline bool BanBoundary::supported(NormalEncoding enc) {
     return enc.storesTriangles();
 }
 
+inline bool BanEdge::supported(NormalEncoding enc) {
+    // Note: storesTriangles() will ensure we are not using angle structures.
+    return enc.storesTriangles();
+}
+
 inline bool BanTorusBoundary::supported(NormalEncoding enc) {
     // Note: storesTriangles() will ensure we are not using angle structures.
     return enc.storesTriangles();
