@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 2011-2021, Ben Burton                                   *
+ *  Copyright (c) 2011-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -185,11 +185,11 @@ class TreeTraversal : public ShortOutput<
                  same encoding; see LPInitialTableaux for details. */
         const BanConstraint ban_;
             /**< Details of any banning/marking constraints that are in use. */
-        const int nTets_;
+        const size_t nTets_;
             /**< The number of tetrahedra in the underlying triangulation. */
-        const int nTypes_;
+        const size_t nTypes_;
             /**< The total length of a type vector. */
-        const int nTableaux_;
+        const size_t nTableaux_;
             /**< The maximum number of tableaux that we need to keep in memory
                  at any given time during the backtracking search. */
 
@@ -199,7 +199,7 @@ class TreeTraversal : public ShortOutput<
                  we modify this type vector in-place.  Any types beyond
                  the current level in the search tree will always be set
                  to zero. */
-        int* typeOrder_;
+        size_t* typeOrder_;
             /**< A permutation of 0,...,\a nTypes_-1 that indicates in
                  which order we select types: the first type we select
                  (at the root of the tree) is type_[typeOrder_[0]], and the
@@ -208,11 +208,11 @@ class TreeTraversal : public ShortOutput<
                  allowed to change as the algorithm runs (though of
                  course you can only change sections of the permutation
                  that correspond to types not yet selected). */
-        int level_;
+        ssize_t level_;
             /**< The current level in the search tree.
                  As the search runs, this holds the index into
                  typeOrder_ corresponding to the last type that we chose. */
-        int octLevel_;
+        ssize_t octLevel_;
             /**< The level at which we are enforcing an octagon type (with a
                  strictly positive number of octagons).  If we are working with
                  angle structures or normal surfaces only (and so we do not
@@ -457,13 +457,18 @@ class TreeTraversal : public ShortOutput<
          * (e.g., 2 for a vanilla normal surface tree traversal algorithm).
          * If the underlying coordinate system does not support triangles
          * then this argument will be ignored.
+         * @param banArgs any additional arguments to be passed to the
+         * BanConstraint constructor, after the initial starting tableaux.
+         * For most ban constrainst classes, this list of arguments is empty.
          * @param enumeration \c true if we should optimise the tableaux
          * for a full enumeration of vertex surfaces or taut angle structures,
          * or \c false if we should optimise the tableaux for an existence test
          * (such as searching for a non-trivial normal disc or sphere).
          */
+        template <typename... BanArgs>
         TreeTraversal(const Triangulation<3>& tri, NormalEncoding enc,
-                int branchesPerQuad, int branchesPerTri, bool enumeration);
+                int branchesPerQuad, int branchesPerTri, bool enumeration,
+                BanArgs&&... banArgs);
 
         /**
          * Rearranges the search tree so that \a nextType becomes the next
@@ -480,7 +485,7 @@ class TreeTraversal : public ShortOutput<
          *
          * @param nextType the next type to process.
          */
-        void setNext(int nextType);
+        void setNext(size_t nextType);
 
         /**
          * Returns the next unmarked triangle type from a given starting
@@ -507,7 +512,7 @@ class TreeTraversal : public ShortOutput<
          * triangle type from \a startFrom onwards, or -1 if there are no
          * more remaining.
          */
-        int nextUnmarkedTriangleType(int startFrom);
+        ssize_t nextUnmarkedTriangleType(size_t startFrom);
 
         /**
          * Determines how many different values we could assign to the given
@@ -535,7 +540,7 @@ class TreeTraversal : public ShortOutput<
          * feasible system; this will be between 0 and 4 inclusive for
          * quadrilateral types, or between 0 and 3 inclusive for angle types.
          */
-        int feasibleBranches(int quadType);
+        int feasibleBranches(size_t quadType);
 
         /**
          * Gives a rough estimate as to what percentage of the way the
@@ -684,7 +689,7 @@ class TreeEnumeration :
         size_t nSolns_;
             /**< The number of vertex surfaces found so far. */
 
-        int lastNonZero_;
+        ssize_t lastNonZero_;
             /**< The index into typeOrder_ corresponding to the last non-zero
                  type that was selected, or -1 if we still have the zero
                  vector.  Here "last" means "last chosen"; that is, the
@@ -734,8 +739,13 @@ class TreeEnumeration :
          * vertex surfaces.
          * @param enc the normal (or almost normal) surface vector encoding
          * that we are working with.
+         * @param banArgs any additional arguments to be passed to the
+         * BanConstraint constructor, after the initial starting tableaux.
+         * For most ban constrainst classes, this list of arguments is empty.
          */
-        TreeEnumeration(const Triangulation<3>& tri, NormalEncoding enc);
+        template <typename... BanArgs>
+        TreeEnumeration(const Triangulation<3>& tri, NormalEncoding enc,
+            BanArgs&&... banArgs);
 
         /**
          * Returns the total number of vertex normal or almost normal surfaces
@@ -1049,8 +1059,12 @@ class TautEnumeration :
          *
          * @param tri the triangulation in which we wish to enumerate
          * taut angle structures.
+         * @param banArgs any additional arguments to be passed to the
+         * BanConstraint constructor, after the initial starting tableaux.
+         * For most ban constrainst classes, this list of arguments is empty.
          */
-        TautEnumeration(const Triangulation<3>& tri);
+        template <typename... BanArgs>
+        TautEnumeration(const Triangulation<3>& tri, BanArgs&&... banArgs);
 
         /**
          * Returns the total number of taut angle structures
@@ -1381,12 +1395,13 @@ class TreeSingleSoln :
         using TreeTraversal<LPConstraint, BanConstraint, IntType>::setNext;
 
     private:
-        int nextZeroLevel_;
+        ssize_t nextZeroLevel_;
             /**< The next level in the search tree at which we will force some
                  triangle coordinate to zero.  We use this to avoid vertex
                  links by dynamically reorganising the search tree as we run
                  to ensure that at least one relevant triangle coordinate is
-                 set to zero at all stages and all levels of the search. */
+                 set to zero at all stages and all levels of the search.
+                 We make this type signed for consistenty with \a level_. */
 
         bool cancelled_;
             /**< Has the search been cancelled by another thread?
@@ -1429,8 +1444,13 @@ class TreeSingleSoln :
          * non-trivial surface.
          * @param enc the normal (or almost normal) surface vector encoding
          * that we are working with.
+         * @param banArgs any additional arguments to be passed to the
+         * BanConstraint constructor, after the initial starting tableaux.
+         * For most ban constrainst classes, this list of arguments is empty.
          */
-        TreeSingleSoln(const Triangulation<3>& tri, NormalEncoding enc);
+        template <typename... BanArgs>
+        TreeSingleSoln(const Triangulation<3>& tri, NormalEncoding enc,
+            BanArgs&&... banArgs);
 
         /**
          * Runs the tree traversal algorithm until it finds some non-trivial
@@ -1501,7 +1521,7 @@ inline size_t TreeTraversal<LPConstraint, BanConstraint, IntType>::visited()
 template <class LPConstraint, typename BanConstraint, typename IntType>
 inline void TreeTraversal<LPConstraint, BanConstraint, IntType>::dumpTypes(
         std::ostream& out) const {
-    for (unsigned i = 0; i < nTypes_; ++i)
+    for (size_t i = 0; i < nTypes_; ++i)
         out << static_cast<int>(type_[i]);
 }
 
@@ -1520,10 +1540,10 @@ void TreeTraversal<LPConstraint, BanConstraint, IntType>::writeTextShort(
 
     // We assume the possible types are all single-digit.
     char* c = new char[nTypes_ + 1];
-    int i = 0;
+    ssize_t i = 0;
     for ( ; i <= level_; ++i)
         c[typeOrder_[i]] = char(type_[typeOrder_[i]] + '0');
-    for ( ; i < nTypes_; ++i)
+    for ( ; i < static_cast<ssize_t>(nTypes_); ++i)
         c[typeOrder_[i]] = '_';
     c[i] = 0;
 
@@ -1532,20 +1552,74 @@ void TreeTraversal<LPConstraint, BanConstraint, IntType>::writeTextShort(
 }
 
 template <class LPConstraint, typename BanConstraint, typename IntType>
-inline int TreeTraversal<LPConstraint, BanConstraint, IntType>::
-        nextUnmarkedTriangleType(int startFrom) {
-    while (startFrom < nTypes_ && ban_.marked(2 * nTets_ + startFrom))
-        ++startFrom;
-    return (startFrom == nTypes_ ? -1 : startFrom);
+template <typename... BanArgs>
+TreeTraversal<LPConstraint, BanConstraint, IntType>::TreeTraversal(
+        const Triangulation<3>& tri, NormalEncoding enc,
+        int branchesPerQuad, int branchesPerTri, bool enumeration,
+        BanArgs&&... banArgs) :
+        origTableaux_(tri, enc, enumeration),
+        enc_(enc),
+        ban_(origTableaux_, std::forward<BanArgs>(banArgs)...),
+        nTets_(tri.size()),
+        nTypes_(enc_.storesTriangles() ? 5 * nTets_ : nTets_),
+        /* Each time we branch, one LP can be solved in-place:
+           therefore we use branchesPerQuad-1 and branchesPerTri-1.
+           The final +1 is for the root node. */
+        nTableaux_(enc_.storesTriangles() ?
+            (branchesPerQuad - 1) * nTets_ +
+                (branchesPerTri - 1) * nTets_ * 4 + 1 :
+            (branchesPerQuad - 1) * nTets_ + 1),
+        type_(new char[nTypes_ + 1]),
+        typeOrder_(new size_t[nTypes_]),
+        level_(0),
+        octLevel_(enc_.storesOctagons() ? -1 : static_cast<ssize_t>(nTypes_)),
+        lp_(new LPData<LPConstraint, IntType>[nTableaux_]),
+        lpSlot_(new LPData<LPConstraint, IntType>*[nTypes_ + 1]),
+        nextSlot_(new LPData<LPConstraint, IntType>*[nTypes_ + 1]),
+        nVisited_(0) {
+    // Initialise the type vector to the zero vector.
+    std::fill(type_, type_ + nTypes_ + 1, 0);
+
+    // Set a default type order.
+    for (size_t i = 0; i < nTypes_; ++i)
+        typeOrder_[i] = i;
+
+    // Reserve space for all the tableaux that we will ever need.
+    for (size_t i = 0; i < nTableaux_; ++i)
+        lp_[i].reserve(origTableaux_);
+
+    // Mark the location of the initial tableaux at the root node.
+    lpSlot_[0] = lp_;
+    nextSlot_[0] = lp_ + 1;
+
+    // Reserve space for our additional temporary tableaux.
+    tmpLP_[0].reserve(origTableaux_);
+    tmpLP_[1].reserve(origTableaux_);
+    tmpLP_[2].reserve(origTableaux_);
+    tmpLP_[3].reserve(origTableaux_);
 }
 
 template <class LPConstraint, typename BanConstraint, typename IntType>
+inline ssize_t TreeTraversal<LPConstraint, BanConstraint, IntType>::
+        nextUnmarkedTriangleType(size_t startFrom) {
+    while (startFrom < nTypes_ && ban_.marked(2 * nTets_ + startFrom))
+        ++startFrom;
+    // Don't use ( ? : ), since we are combining signed/unsigned return values.
+    if (startFrom == nTypes_)
+        return -1;
+    else
+        return startFrom;
+}
+
+template <class LPConstraint, typename BanConstraint, typename IntType>
+template <typename... BanArgs>
 inline TreeEnumeration<LPConstraint, BanConstraint, IntType>::TreeEnumeration(
-        const Triangulation<3>& tri, NormalEncoding enc) :
+        const Triangulation<3>& tri, NormalEncoding enc, BanArgs&&... banArgs) :
         TreeTraversal<LPConstraint, BanConstraint, IntType>(tri, enc,
             (enc.storesOctagons() ? 7 : 4) /* branches per quad */,
             2 /* branches per triangle */,
-            true /* enumeration */),
+            true /* enumeration */,
+            std::forward<BanArgs>(banArgs)...),
         nSolns_(0),
         lastNonZero_(-1) {
 }
@@ -1584,12 +1658,14 @@ inline bool TreeEnumeration<LPConstraint, BanConstraint, IntType>::
 }
 
 template <class LPConstraint, typename BanConstraint, typename IntType>
+template <typename... BanArgs>
 inline TautEnumeration<LPConstraint, BanConstraint, IntType>::TautEnumeration(
-        const Triangulation<3>& tri) :
+        const Triangulation<3>& tri, BanArgs&&... banArgs) :
         TreeTraversal<LPConstraint, BanConstraint, IntType>(tri, NS_ANGLE,
             3 /* branches per quad */,
             0 /* branches per triangle; irrelevant here */,
-            true /* enumeration */),
+            true /* enumeration */,
+            std::forward<BanArgs>(banArgs)...),
         nSolns_(0) {
 }
 
@@ -1627,12 +1703,14 @@ inline bool TautEnumeration<LPConstraint, BanConstraint, IntType>::
 }
 
 template <class LPConstraint, typename BanConstraint, typename IntType>
+template <typename... BanArgs>
 inline TreeSingleSoln<LPConstraint, BanConstraint, IntType>::TreeSingleSoln(
-        const Triangulation<3>& tri, NormalEncoding enc) :
+        const Triangulation<3>& tri, NormalEncoding enc, BanArgs&&... banArgs) :
         TreeTraversal<LPConstraint, BanConstraint, IntType>(tri, enc,
             (enc.storesOctagons() ? 6 : 3) /* branches per quad */,
             2 /* branches per triangle */,
-            false /* enumeration */),
+            false /* enumeration */,
+            std::forward<BanArgs>(banArgs)...),
         nextZeroLevel_(0),
         cancelled_(false) {
 }
