@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -465,14 +465,14 @@ inline void ValidityConstraints::addLocal(iterator begin, iterator end) {
     local_.emplace_back(begin, end);
 }
 inline void ValidityConstraints::addLocal(std::initializer_list<int> pattern) {
-    local_.emplace_back(std::move(pattern));
+    local_.emplace_back(pattern);
 }
 template <typename iterator>
 inline void ValidityConstraints::addGlobal(iterator begin, iterator end) {
     global_.emplace_back(begin, end);
 }
 inline void ValidityConstraints::addGlobal(std::initializer_list<int> pattern) {
-    global_.emplace_back(std::move(pattern));
+    global_.emplace_back(pattern);
 }
 
 inline void ValidityConstraints::swap(ValidityConstraints& other) noexcept {
