@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -145,6 +145,14 @@ class NormalHypersurface : public ShortOutput<NormalHypersurface> {
         mutable std::optional<AbelianGroup> H1_;
             /**< First homology group of the hypersurface.
                  This is std::nullopt if it has not yet been computed. */
+        mutable uint8_t linkOf_ { 0 };
+            /**< Indicates which dimensions of face a positive rational multiple
+                 of this hypersurface is a thin or normalised link of.  This is
+                 treated as a bitmask: for each i=0,1,2,3, the (2i+1)th bit
+                 indicates whether this hypersurface scales to the link of an
+                 i-face, and the (2i)th bit indicates whether this information
+                 has actually been computed yet; if it has not been computed,
+                 then the (2i+1)th bit will be zero. */
 
     public:
         /**
@@ -466,19 +474,6 @@ class NormalHypersurface : public ShortOutput<NormalHypersurface> {
         NormalHypersurface(const SnapshotRef<Triangulation<4>>& triang,
             HyperCoords coords, Vector<LargeInteger>&& vector);
 
-        /**
-         * Deprecated routine that creates a newly allocated clone of this
-         * normal hypersurface.
-         *
-         * The name of the normal hypersurface will \e not be copied to the
-         * clone; instead the clone will have an empty name.
-         *
-         * \deprecated Simply use the copy constructor instead.
-         *
-         * @return a clone of this normal hypersurface.
-         */
-        [[deprecated]] NormalHypersurface* clone() const;
-
         /**
          * Sets this to be a copy of the given normal hypersurface.
          *
@@ -527,11 +522,15 @@ class NormalHypersurface : public ShortOutput<NormalHypersurface> {
         void swap(NormalHypersurface& other) noexcept;
 
         /**
-         * Returns the double of this hypersurface.
+         * Deprecated routine that returns the double of this hypersurface.
+         *
+         * \deprecated Normal hypersurfaces can now be multiplied by integer
+         * constants.  In particular, this routine has exactly the same
+         * effect as multiplying the hypersurface by 2.
          *
          * @return the double of this normal hypersurface.
          */
-        NormalHypersurface doubleHypersurface() const;
+        [[deprecated]] NormalHypersurface doubleHypersurface() const;
 
         /**
          * Returns the sum of this and the given hypersurface.  This will
@@ -551,6 +550,46 @@ class NormalHypersurface : public ShortOutput<NormalHypersurface> {
          */
         NormalHypersurface operator + (const NormalHypersurface& rhs) const;
 
+        /**
+         * Returns the given integer multiple of this hypersurface.
+         *
+         * The resulting hypersurface will use the same internal vector
+         * encoding as this hypersurface.
+         *
+         * @param coeff the coefficient to multiply this hypersurface by;
+         * this must be non-negative.
+         * @return the resulting multiple of this hypersurface.
+         */
+        NormalHypersurface operator * (const LargeInteger& coeff) const;
+
+        /**
+         * Converts this hypersurface into the given integer multiple of itself.
+         *
+         * The internal vector encoding used by this hypersurface will not
+         * change.
+         *
+         * @param coeff the coefficient to multiply this hypersurface by;
+         * this must be non-negative.
+         * @return a reference to this hypersurface.
+         */
+        NormalHypersurface& operator *= (const LargeInteger& coeff);
+
+        /**
+         * Converts this hypersurface into its smallest positive rational
+         * multiple with integer coordinates.
+         *
+         * Note that the scaling factor will be independent of which
+         * internal vector encoding is used.  This is essentially because
+         * integer prism coordinates (which are stored in every encoding)
+         * are enough to guarantee integer tetrahedron coordinates (which
+         * might or might not be stored).
+         *
+         * @return the integer by which the original hypersurface was divided
+         * (i.e., the gcd of all normal coordinates in the original
+         * hypersurface).  This will always be strictly positive.
+         */
+        LargeInteger scaleDown();
+
         /**
          * Returns the number of tetrahedron pieces of the given type in
          * this normal hypersurface.
@@ -648,21 +687,6 @@ class NormalHypersurface : public ShortOutput<NormalHypersurface> {
          * @param out the output stream to which to write.
          */
         void writeTextShort(std::ostream& out) const;
-        /**
-         * Deprecated routine that writes the underlying coordinate vector
-         * to the given output stream in text format.
-         * No indication will be given as to which coordinate
-         * system is being used or what each coordinate means.
-         * No newline will be written.
-         *
-         * \deprecated Just write vector() directly to the output stream.
-         *
-         * \ifacespython Not present; instead just write vector() to the
-         * appropriate output stream.
-         *
-         * @param out the output stream to which to write.
-         */
-        [[deprecated]] void writeRawVector(std::ostream& out) const;
 
         /**
          * Writes a chunk of XML containing this normal hypersurface and all
@@ -699,7 +723,10 @@ class NormalHypersurface : public ShortOutput<NormalHypersurface> {
         bool isCompact() const;
         /**
          * Returns whether or not this hypersurface is orientable.
-         *·
+         *
+         * For our purposes, the empty hypersurface is considered to be
+         * orientable.
+         *
          * This routine caches its results, which means that once it has
          * been called for a particular surface, subsequent calls return
          * the answer immediately.
@@ -717,6 +744,9 @@ class NormalHypersurface : public ShortOutput<NormalHypersurface> {
         /**
          * Returns whether or not this hypersurface is two-sided.
          *
+         * For our purposes, the empty hypersurface is considered to be
+         * two-sided.
+         *
          * This routine caches its results, which means that once it has
          * been called for a particular surface, subsequent calls return
          * the answer immediately.
@@ -734,6 +764,9 @@ class NormalHypersurface : public ShortOutput<NormalHypersurface> {
         /**
          * Returns whether or not this hypersurface is connected.
          *
+         * For our purposes, the empty hypersurface is considered to be
+         * connected.
+         *
          * This routine caches its results, which means that once it has
          * been called for a particular surface, subsequent calls return
          * the answer immediately.
@@ -763,43 +796,280 @@ class NormalHypersurface : public ShortOutput<NormalHypersurface> {
          * Determines whether or not this hypersurface is vertex linking.
          * A <i>vertex linking</i> hypersurface contains only tetrahedra.
          *
+         * This behaves differently from isVertexLink(), which only detects
+         * the link of a single vertex (or a multiple of such a link).
+         * In contrast, this routine will also detect the union of
+         * several \e different vertex links.
+         *
          * Note that the results of this routine are not cached.
          * Thus the results will be reevaluated every time this routine is
          * called.
          *
-         * \todo \opt Cache results.
-         *
          * @return \c true if and only if this hypersurface is vertex linking.
          */
         bool isVertexLinking() const;
         /**
-         * Determines whether or not a rational multiple of this hypersurface
-         * is the link of a single vertex.
+         * Determines whether or not a positive rational multiple of this
+         * hypersurface is the link of a single vertex.
+         *
+         * This behaves differently from isVertexLinking(), which will also
+         * detect a union of several different vertex links.  In contrast,
+         * this routine will only identify the link of a \e single vertex
+         * (or a multiple of such a link).
          *
          * Note that the results of this routine are not cached.
          * Thus the results will be reevaluated every time this routine is
          * called.
          *
-         * \todo \opt Cache results.
-         *
-         * @return the vertex linked by this hypersurface, or \c null if this
-         * hypersurface is not the link of a single vertex.
+         * @return the vertex linked by a positive rational multiple of this
+         * hypersurface, or \c null if this hypersurface is not a multiple of
+         * a single vertex link.
          */
         const Vertex<4>* isVertexLink() const;
         /**
-         * Determines whether or not a rational multiple of this hypersurface
-         * is the thin link of a single edge.
+         * Determines whether or not a positive rational multiple of this
+         * hypersurface is the thin link of a single edge.
+         *
+         * Here a \e thin edge link is a normal hypersurface which appears
+         * naturally as the frontier of a regular neighbourhood of an edge,
+         * with no need for any further normalisation.
+         *
+         * This behaves differently from isNormalEdgeLink(), which tests for a
+         * \e normalised edge link (which could end up far away from the
+         * edge, or could be normalised into a hypersurface with different
+         * topology, or could even be normalised away to nothing).
+         * Although isNormalEdgeLink() will also indicate thin edge links,
+         * this test has significantly less overhead (and so should be faster).
+         *
+         * A hypersurface (or its positive rational multiple) can be the thin
+         * edge link of at most one edge.
          *
          * Note that the results of this routine are not cached.
          * Thus the results will be reevaluated every time this routine is
          * called.
          *
-         * \todo \opt Cache results.
-         *
-         * @return the edge linked by this hypersurface, or \c null if this
-         * hypersurface is not a thin edge link.
+         * @return the edge linked by a positive rational multiple of this
+         * hypersurface, or \c null if this hypersurface is not a multiple
+         * of a single thin edge link.
          */
         const Edge<4>* isThinEdgeLink() const;
+        /**
+         * Determines whether or not a positive rational multiple of this
+         * hypersurface is the normalised link of a single edge.
+         *
+         * Here the phrase \e normalised link of an edge \a e means the
+         * frontier of a regular neighbourhood of \a e, converted into a
+         * normal hypersurface by expanding away from the edge using some basic
+         * normalisation moves.  It could be that there is no normalisation
+         * required at all (in which case it is also a \e thin edge link).
+         * However, it could be that the normalisation process expands
+         * the hypersurface far away from the edge itself, or changes its
+         * topology, or disconnects the hypersurface, or even normalises it
+         * away to an empty hypersurface.
+         *
+         * In particular, this test behaves differently from isThinEdgeLink(),
+         * which tests for thin edge links only (where no additional
+         * normalisation is required).  If you are only interested in thin
+         * edge links, then you should call isThinEdgeLink(), which has much
+         * less overhead.
+         *
+         * A hypersurface (or its positive rational multiple) could be the
+         * normalised link of many edges.  The return value will be a pair
+         * (\a v, \a thin), where:
+         *
+         * - \a v is a vector containing all such edges.  This will begin with
+         *   the edges for which this hypersurface is a thin link, followed by
+         *   the edges where normalisation was required; within each category
+         *   the edges will be ordered by their index within the triangulation.
+         *
+         * - \a thin is either 0 or 1, indicating how many edges this
+         *   hypersurface is a thin link for.  This uses an unsigned type,
+         *   since it will often be compared to <tt>v.size()</tt>.
+         *
+         * If no positive rational multiple of this hypersurface is the
+         * normalised link of any edge, then \a link will be 0 and \a v will be
+         * the empty vector.
+         *
+         * Note that the results of this routine are not cached.
+         * Thus the results will be reevaluated every time this routine is
+         * called.
+         *
+         * @return a vector containing the edge(s) linked by a positive rational
+         * multiple of this hypersurface and an integer indicating how many
+         * of these links are thin, as described above.
+         */
+        std::pair<std::vector<const Edge<4>*>, unsigned> isNormalEdgeLink()
+            const;
+        /**
+         * Determines whether or not a positive rational multiple of this
+         * hypersurface is the thin link of a single triangle.
+         *
+         * Here a \e thin triangle link is a normal hypersurface which appears
+         * naturally as the frontier of a regular neighbourhood of a triangle,
+         * with no need for any further normalisation.
+         *
+         * This behaves differently from isNormalTriangleLink(), which tests
+         * for a \e normalised triangle link (which could end up far away from
+         * the triangle, or could be normalised into a hypersurface with
+         * different topology, or could even be normalised away to nothing).
+         * Unlike the tests for edge links, the routines isThinTriangleLink()
+         * and isNormalTriangleLink() use essentially the same implementation
+         * (so testing for only thin links may be a little faster, but not by
+         * much).
+         *
+         * A hypersurface (or its positive rational multiple) can be the thin
+         * triangle link of at most two triangles.  If there are indeed two
+         * different triangles \a t1 and \a t2 for which a multiple of this
+         * hypersurface can be expressed as the thin triangle link, then
+         * the pair (\a t1, \a t2) will be returned.  If there is only one
+         * such triangle \a t, then the pair (\a t, \c null) will be returned.
+         * If no positive rational multiple of this hypersurface is the thin
+         * link of any triangle, then the pair (\c null, \c null) will be
+         * returned.
+         *
+         * Note that the results of this routine are not cached.
+         * Thus the results will be reevaluated every time this routine is
+         * called.
+         *
+         * @return a pair containing the triangle(s) linked by a positive
+         * rational multiple of this hypersurface, as described above.
+         */
+        std::pair<const Triangle<4>*, const Triangle<4>*> isThinTriangleLink()
+            const;
+        /**
+         * Determines whether or not a positive rational multiple of this
+         * hypersurface is the normalised link of a single triangle.
+         *
+         * Here the phrase \e normalised link of a triangle \a t means the
+         * frontier of a regular neighbourhood of \a t, converted into a normal
+         * hypersurface by expanding away from the triangle using some basic
+         * normalisation moves.  It could be that there is no normalisation
+         * required at all (in which case it is also a \e thin triangle link).
+         * However, it could be that the normalisation process expands
+         * the hypersurface far away from the triangle itself, or changes its
+         * topology, or disconnects the hypersurface, or even normalises it
+         * away to an empty hypersurface.
+         *
+         * In particular, this test behaves differently from
+         * isThinTriangleLink(), which tests for thin triangle links only
+         * (where no additional normalisation is required).  Unlike the
+         * tests for edge links, the routines isThinTriangleLink() and
+         * isNormalTriangleLink() use essentially the same implementation (so
+         * testing for only thin links may be a little faster, but not by much).
+         *
+         * A hypersurface (or its positive rational multiple) could be the
+         * normalised link of many triangles.  The return value will be a pair
+         * (\a v, \a thin), where:
+         *
+         * - \a v is a vector containing all such triangles.  This will begin
+         *   with the triangles for which this hypersurface is a thin link,
+         *   followed by the triangles where normalisation was required;
+         *   within each category the triangles will be ordered by their
+         *   index within the triangulation.
+         *
+         * - \a thin is either 0, 1 or 2, indicating how many triangles this
+         *   hypersurface is a thin link for.  This uses an unsigned type,
+         *   since it will often be compared to <tt>v.size()</tt>.
+         *
+         * If no positive rational multiple of this hypersurface is the
+         * normalised link of any triangle, then \a link will be 0 and \a v
+         * will be the empty vector.
+         *
+         * Note that the results of this routine are not cached.
+         * Thus the results will be reevaluated every time this routine is
+         * called.
+         *
+         * @return a vector containing the triangle(s) linked by a positive
+         * rational multiple of this hypersurface and an integer indicating
+         * how many of these links are thin, as described above.
+         */
+        std::pair<std::vector<const Triangle<4>*>, unsigned>
+            isNormalTriangleLink() const;
+        /**
+         * Determines whether or not a positive rational multiple of this
+         * hypersurface is the thin link of a single tetrahedron.
+         *
+         * Here a \e thin tetrahedron link is a normal hypersurface which
+         * appears naturally as the frontier of a regular neighbourhood of a
+         * tetrahedron, with no need for any further normalisation.
+         *
+         * This behaves differently from isNormalTetrahedronLink(), which tests
+         * for a \e normalised tetrahedron link (which could end up far away
+         * from the tetrahedron, or could be normalised into a hypersurface with
+         * different topology, or could even be normalised away to nothing).
+         * Unlike the tests for edge links, the routines isThinTetrahedronLink()
+         * and isNormalTetrahedronLink() use essentially the same implementation
+         * (so testing for only thin links may be a little faster, but not by
+         * much).
+         *
+         * A hypersurface (or its positive rational multiple) can be the thin
+         * link of at most two tetrahedra.  If there are indeed two different
+         * tetrahedra \a t1 and \a t2 for which a multiple of this
+         * hypersurface can be expressed as the thin tetrahedron link, then
+         * the pair (\a t1, \a t2) will be returned.  If there is only one such
+         * tetrahedron \a t, then the pair (\a t, \c null) will be returned.
+         * If no positive rational multiple of this hypersurface is the thin
+         * link of any tetrahedron, then the pair (\c null, \c null) will be
+         * returned.
+         *
+         * Note that the results of this routine are not cached.
+         * Thus the results will be reevaluated every time this routine is
+         * called.
+         *
+         * @return a pair containing the tetrahedra linked by a positive
+         * rational multiple of this hypersurface, as described above.
+         */
+        std::pair<const Tetrahedron<4>*, const Tetrahedron<4>*>
+            isThinTetrahedronLink() const;
+        /**
+         * Determines whether or not a positive rational multiple of this
+         * hypersurface is the normalised link of a single tetrahedron.
+         *
+         * Here the phrase \e normalised link of a tetrahedron \a t means the
+         * frontier of a regular neighbourhood of \a t, converted into a normal
+         * hypersurface by expanding away from the tetrahedron using some basic
+         * normalisation moves.  It could be that there is no normalisation
+         * required at all (in which case it is also a \e thin tetrahedron
+         * link).  However, it could be that the normalisation process expands
+         * the hypersurface far away from the tetrahedron itself, or changes its
+         * topology, or disconnects the hypersurface, or even normalises it
+         * away to an empty hypersurface.
+         *
+         * In particular, this test behaves differently from
+         * isThinTetrahedronLink(), which tests for thin tetrahedron links only
+         * (where no additional normalisation is required).  Unlike the
+         * tests for edge links, the routines isThinTetrahedronLink() and
+         * isNormalTetrahedronLink() use essentially the same implementation (so
+         * testing for only thin links may be a little faster, but not by much).
+         *
+         * A hypersurface (or its positive rational multiple) could be the
+         * normalised link of many tetrahedra.  The return value will be a
+         * pair (\a v, \a thin), where:
+         *
+         * - \a v is a vector containing all such tetrahedra.  This will begin
+         *   with the tetrahedra for which this hypersurface is a thin link,
+         *   followed by the tetrahedra where normalisation was required;
+         *   within each category the tetrahedra will be ordered by their
+         *   index within the triangulation.
+         *
+         * - \a thin is either 0, 1 or 2, indicating how many tetrahedra this
+         *   hypersurface is a thin link for.  This uses an unsigned type,
+         *   since it will often be compared to <tt>v.size()</tt>.
+         *
+         * If no positive rational multiple of this hypersurface is the
+         * normalised link of any tetrahedron, then \a link will be 0 and \a v
+         * will be the empty vector.
+         *
+         * Note that the results of this routine are not cached.
+         * Thus the results will be reevaluated every time this routine is
+         * called.
+         *
+         * @return a vector containing the tetrahedra linked by a positive
+         * rational multiple of this hypersurface and an integer indicating
+         * how many of these links are thin, as described above.
+         */
+        std::pair<std::vector<const Tetrahedron<4>*>, unsigned>
+            isNormalTetrahedronLink() const;
 
         /**
          * Returns the first homology group of this hypersurface.
@@ -931,19 +1201,6 @@ class NormalHypersurface : public ShortOutput<NormalHypersurface> {
          */
         bool operator < (const NormalHypersurface& other) const;
 
-        /**
-         * Deprecated routine that determines whether this and the given
-         * hypersurface in fact represent the same normal hypersurface.
-         *
-         * \deprecated This routine has been renamed to the comparison
-         * operator (==).
-         *
-         * @param other the hypersurface to be compared with this hypersurface.
-         * @return \c true if both hypersurfaces represent the same normal
-         * hypersurface, or \c false if not.
-         */
-        [[deprecated]] bool sameSurface(const NormalHypersurface& other) const;
-
         /**
          * Determines whether this hypersurface is embedded.  This is true if
          * and only if the surface contains no conflicting prism types.
@@ -1007,15 +1264,6 @@ class NormalHypersurface : public ShortOutput<NormalHypersurface> {
          */
         const Vector<LargeInteger>& vector() const;
 
-        /**
-         * A deprecated alias for vector().
-         *
-         * \deprecated This routine has been renamed to vector().
-         *
-         * @return the underlying integer vector.
-         */
-        [[deprecated]] const Vector<LargeInteger>& rawVector() const;
-
         /**
          * Returns the specific integer vector encoding that this hypersurface
          * uses internally.  This is the encoding that should be used
@@ -1062,6 +1310,27 @@ class NormalHypersurface : public ShortOutput<NormalHypersurface> {
          */
         void calculateFromTriangulation() const;
 
+        /**
+         * Determines whether or not a positive rational multiple of this
+         * hypersurface \e could be the normalised link of a face of positive
+         * dimension.
+         *
+         * A non-null return value is \e not a guarantee that this hypersurface
+         * \e is such a link; however, if this routine returns no value then
+         * this \e is a guarantee that the hypersurface is not such a link.
+         *
+         * The precise tests that this routine carries out involve a trade-off
+         * between speed and mathematical power, and so are subject to change
+         * in future versions of Regina.
+         *
+         * \pre This hypersurface is non-empty.
+         *
+         * @return the precise multiple of this hypersurface that \e could be a
+         * normalised non-vertex face link, or no value if we can prove
+         * that this hypersurface is not such a link.
+         */
+        std::optional<NormalHypersurface> couldLinkFace() const;
+
     friend class XMLNormalHypersurfaceReader;
 };
 
@@ -1164,10 +1433,6 @@ inline NormalHypersurface::NormalHypersurface(const NormalHypersurface& src,
     triangulation_ = triangulation;
 }
 
-inline NormalHypersurface* NormalHypersurface::clone() const {
-    return new NormalHypersurface(*this);
-}
-
 inline void NormalHypersurface::swap(NormalHypersurface& other) noexcept {
     std::swap(enc_, other.enc_);
     vector_.swap(other.vector_);
@@ -1180,6 +1445,7 @@ inline void NormalHypersurface::swap(NormalHypersurface& other) noexcept {
     realBoundary_.swap(other.realBoundary_);
     compact_.swap(other.compact_);
     H1_.swap(other.H1_);
+    std::swap(linkOf_, other.linkOf_);
 }
 
 inline LargeInteger NormalHypersurface::tetrahedra(size_t pentIndex, int vertex)
@@ -1202,10 +1468,6 @@ inline void NormalHypersurface::setName(const std::string& name) {
     name_ = name;
 }
 
-inline void NormalHypersurface::writeRawVector(std::ostream& out) const {
-    out << vector_;
-}
-
 inline bool NormalHypersurface::isEmpty() const {
     // All vector encodings store the empty hypersurface as the zero vector.
     return vector_.isZero();
@@ -1245,10 +1507,6 @@ inline const Vector<LargeInteger>& NormalHypersurface::vector() const {
     return vector_;
 }
 
-inline const Vector<LargeInteger>& NormalHypersurface::rawVector() const {
-    return vector_;
-}
-
 inline HyperEncoding NormalHypersurface::encoding() const {
     return enc_;
 }
@@ -1258,11 +1516,6 @@ inline bool NormalHypersurface::operator != (const NormalHypersurface& other)
     return ! ((*this) == other);
 }
 
-inline bool NormalHypersurface::sameSurface(const NormalHypersurface& other)
-        const {
-    return (*this) == other;
-}
-
 inline NormalHypersurface NormalHypersurface::operator + (
         const NormalHypersurface& rhs) const {
     // Given our current conditions on vector storage, both underlying
@@ -1272,6 +1525,10 @@ inline NormalHypersurface NormalHypersurface::operator + (
             vector_ + rhs.vector_);
 }
 
+inline NormalHypersurface NormalHypersurface::doubleHypersurface() const {
+    return (*this) * 2;
+}
+
 inline void swap(NormalHypersurface& a, NormalHypersurface& b) noexcept {
     a.swap(b);
 }
