@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -47,6 +47,7 @@
 #include <cassert>
 #include <cstdlib>
 #include <iostream>
+#include <limits>
 
 namespace regina {
 
@@ -64,25 +65,39 @@ namespace regina {
 
 template <typename Iterator>
 Link Link::fromDT(Iterator begin, Iterator end) {
+    using InputInt = std::remove_cv_t<std::remove_reference_t<decltype(*begin)>>;
+    static_assert(std::is_integral_v<InputInt> &&
+        ! std::is_unsigned_v<InputInt>, "fromDT(): the iterator type "
+        "needs to dereference to give a native signed C++ integer type.");
+
     // Extract the number of crossings.
-    int aNumCrossings = end - begin;
+    size_t aNumCrossings = end - begin;
     if (aNumCrossings == 0)
         return Link(1);
 
     // Some basic sanity checking.
     // We ensure that the integers are in range, but we do not yet check
     // that their absolute values are distinct (that will come later).
+
+    if constexpr (sizeof(InputInt) <= sizeof(size_t)) {
+        if (2 * aNumCrossings >
+                static_cast<size_t>(std::numeric_limits<InputInt>::max()))
+            throw InvalidArgument("fromDT(): too many crossings for "
+                "the given integer type");
+    }
+    const auto maxEntry = static_cast<InputInt>(2 * aNumCrossings);
+
     Iterator it;
     for (it = begin; it != end; ++it) {
         if (*it % 2 != 0)
             throw InvalidArgument("fromDT(): code contains odd integer");
-        if (*it == 0 || *it > 2*aNumCrossings || *it < -2*aNumCrossings)
+        if (*it == 0 || *it > maxEntry || *it < -maxEntry)
             throw InvalidArgument("fromDT(): integer out of range in code");
     }
 
     Link ans;
 
-    int i;
+    size_t i;
     for (i = 0; i < aNumCrossings; ++i)
         ans.crossings_.push_back(new Crossing);
 
@@ -90,7 +105,7 @@ Link Link::fromDT(Iterator begin, Iterator end) {
 
     // Here starts the SnapPea code!
 
-    int             *theAlternatingDT,
+    size_t          *theAlternatingDT,
                     *theInvolution;
     bool            *theRealization;
 
@@ -101,7 +116,7 @@ Link Link::fromDT(Iterator begin, Iterator end) {
      *  eight knot example, theAlternatingDT and the DT code are the same,
      *  because the figure eight knot is already alternating.
      */
-    theAlternatingDT = new int[aNumCrossings];
+    theAlternatingDT = new size_t[aNumCrossings];
     for (it = begin, i = 0; it != end; ++it, ++i)
         theAlternatingDT[i] = abs(*it);
 
@@ -125,7 +140,7 @@ Link Link::fromDT(Iterator begin, Iterator end) {
      *
      *  As an array, theInvolution = {3, 6, 5, 0, 7, 2, 1, 4}.
      */
-    theInvolution = new int[2 * aNumCrossings];
+    theInvolution = new size_t[2 * aNumCrossings];
     std::fill(theInvolution, theInvolution + 2 * aNumCrossings,
         1 /* a value that does not appear in theAlternatingDT[] */);
     for (i = 0; i < aNumCrossings; i++)
@@ -173,11 +188,11 @@ Link Link::fromDT(Iterator begin, Iterator end) {
      *  For each position in the involution, we also identify which
      *  crossing it represents.
      */
-    int* oddPos = new int[aNumCrossings];
-    int* evenPos = new int[aNumCrossings];
-    int* crossingForPos = new int[2 * aNumCrossings];
+    auto* oddPos = new size_t[aNumCrossings];
+    auto* evenPos = new size_t[aNumCrossings];
+    auto* crossingForPos = new size_t[2 * aNumCrossings];
 
-    int nextUnused = 0;
+    size_t nextUnused = 0;
     for (i = 0; i < 2 * aNumCrossings; ++i) {
         // Examine position i from the involution.
         if (theInvolution[i] > i) {
