@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -32,6 +32,7 @@
 
 #include "link/tangle.h"
 #include "utilities/stringutils.h"
+#include <climits>
 #include <iterator>
 
 // We wish to use std::to_chars(), but GCC only implements it in gcc-8.1.
@@ -62,16 +63,18 @@
 namespace regina {
 
 Link Link::fromGauss(const std::string& s) {
+    // Work with the largest integer type that we could possibly need.
+    using Int = std::make_signed_t<size_t>;
     std::istringstream in(s);
-    std::vector<int> terms;
+    std::vector<Int> terms;
 
-    int i;
+    Int i;
     while (true) {
         in >> i;
         if (! in) {
             if (in.eof())
                 break;
-            throw InvalidArgument("fromGauss(): invalid character");
+            throw InvalidArgument("fromGauss(): invalid integer in sequence");
         }
         terms.push_back(i);
     }
@@ -164,7 +167,10 @@ std::vector<int> Link::gaussData() const {
         throw NotImplemented(
             "Gauss codes are only implemented for single-component links");
     if (crossings_.empty())
-        return std::vector<int>();
+        return {};
+    if (crossings_.size() > INT_MAX)
+        throw NotImplemented("This Gauss code has entries that cannot "
+            "fit into a C++ int");
 
     std::vector<int> ans;
     ans.reserve(crossings_.size() * 2);
@@ -173,9 +179,9 @@ std::vector<int> Link::gaussData() const {
     StrandRef s = start;
     do {
         if (s.strand() == 0)
-            ans.push_back(-static_cast<int>(s.crossing()->index() + 1));
+            ans.push_back(- static_cast<int>(s.crossing()->index() + 1));
         else
-            ans.push_back(s.crossing()->index() + 1);
+            ans.push_back(static_cast<int>(s.crossing()->index() + 1));
         ++s;
     } while (s != start);
 
