@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -52,6 +52,7 @@ class ModelLinkGraph;
 class ModelLinkGraphArc;
 class ModelLinkGraphCells;
 class ModelLinkGraphNode;
+class StrandRef;
 
 /**
  * A reference to an outgoing edge from a node of a model graph for a
@@ -322,9 +323,6 @@ class ModelLinkGraphArc {
         /**
          * Tests whether this is a non-null arc.
          *
-         * \ifacespython Not available; instead you can simply test whether
-         * <tt>node() == None</tt>.
-         *
          * @return \c true if this is not a null arc (i.e., node()
          * does not return a null pointer), or \c false if this is a null
          * arc.
@@ -381,7 +379,7 @@ class ModelLinkGraphNode : public MarkedElement,
          *
          * @return the index of this node.
          */
-        int index() const;
+        size_t index() const;
         /**
          * Returns a reference to one of the four arcs of the graph that
          * exit this node.  This is equivalent to directly constructing
@@ -475,6 +473,20 @@ class ModelLinkGraph : public Output<ModelLinkGraph> {
          * Constructs an empty graph.
          */
         ModelLinkGraph();
+        /**
+         * Constructs the graph that models the given link.
+         *
+         * Any zero-component unknot components of the link will be ignored.
+         *
+         * The nodes of this graph will be numbered in the same way as
+         * the crossings of \a link.  For each node, arc 0 will represent
+         * the outgoing lower strand of the corresponding crossing.
+         *
+         * Using this constructor is identical to calling Link::graph().
+         *
+         * @param link the link that this new graph will model.
+         */
+        ModelLinkGraph(const Link& link);
         /**
          * Constructs a new copy of the given graph.
          *
@@ -591,16 +603,6 @@ class ModelLinkGraph : public Output<ModelLinkGraph> {
          */
         void swap(ModelLinkGraph& other) noexcept;
 
-        /**
-         * Deprecated routine that swaps the contents of this and the
-         * given graph.
-         *
-         * \deprecated Use swap() instead.
-         *
-         * @param other the graph whose contents should be swapped with this.
-         */
-        [[deprecated]] void swapContents(ModelLinkGraph& other) noexcept;
-
         /**
          * Determines if this graph is combinatorially identical to the
          * given graph.
@@ -954,6 +956,21 @@ class ModelLinkGraph : public Output<ModelLinkGraph> {
             { -1, -1, -1, 0, -1, -1, 1, -1, -1, 3, -1, -1, 2 },
             { -1, -1, -1, 1, -1, -1, 2, -1, -1, 0, -1, -1, 3 }
         };
+
+        /**
+         * Internal to the constructor that takes a link to model.
+         *
+         * This function returns the outgoing arc of this graph that
+         * corresponds to the outgoing upper/lower strand at the given crossing.
+         */
+        ModelLinkGraphArc outgoingArc(const StrandRef& s);
+        /**
+         * Internal to the constructor that takes a link to model.
+         *
+         * This function returns the outgoing arc of this graph that
+         * corresponds to the incoming upper/lower strand at the given crossing.
+         */
+        ModelLinkGraphArc incomingArc(const StrandRef& s);
 };
 
 /**
@@ -1459,7 +1476,7 @@ inline std::ostream& operator << (std::ostream& out,
 
 // Inline functions for ModelLinkGraphNode
 
-inline int ModelLinkGraphNode::index() const {
+inline size_t ModelLinkGraphNode::index() const {
     return markedIndex();
 }
 
@@ -1524,10 +1541,6 @@ inline void ModelLinkGraph::swap(ModelLinkGraph& other) noexcept {
     }
 }
 
-inline void ModelLinkGraph::swapContents(ModelLinkGraph& other) noexcept {
-    swap(other);
-}
-
 inline bool ModelLinkGraph::operator != (const ModelLinkGraph& other) const {
     return ! ((*this) == other);
 }
