@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -342,8 +342,12 @@ class Tangle : public Output<Tangle> {
         bool operator != (const Tangle& other) const;
 
         /**
-         * Translates a strand reference for some other tangle into the
-         * corresponding strand reference for this tangle.
+         * Translates a strand reference from some other tangle into the
+         * corresponding strand reference from this tangle.
+         *
+         * Typically this routine would be used when the given strand comes
+         * from a tangle that is combinatorially identical to this, and you wish
+         * to obtain the corresponding strand in this tangle.
          *
          * Specifically: if \a other refers to some strand (upper or lower)
          * of crossing number \a k of some other tangle, then the return
@@ -352,6 +356,10 @@ class Tangle : public Output<Tangle> {
          *
          * This routine behaves correctly even if \a other is a null reference.
          *
+         * \pre This tangle contains at least as many crossings as the tangle
+         * containing \a other (though, as noted above, in typical scenarios
+         * both tangles would actually be combinatorially identical).
+         *
          * @param other the strand reference to translate.
          * @return the corresponding strand reference for this tangle.
          */
@@ -403,16 +411,6 @@ class Tangle : public Output<Tangle> {
          */
         void swap(Tangle& other) noexcept;
 
-        /**
-         * Deprecated routine that swaps the contents of this and the
-         * given tangle.
-         *
-         * \deprecated Use swap() instead.
-         *
-         * @param other the tangle whose contents should be swapped with this.
-         */
-        [[deprecated]] void swapContents(Tangle& other) noexcept;
-
         /**
          * Adds a twist to the right-hand end of this tangle.
          *
@@ -1025,10 +1023,6 @@ inline Tangle::~Tangle() {
         delete c;
 }
 
-inline void Tangle::swapContents(Tangle& other) noexcept {
-    swap(other);
-}
-
 inline char Tangle::type() const {
     return type_;
 }
