@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -47,8 +47,7 @@ namespace regina {
 /**
  * Represents an orientable handlebody.
  *
- * All optional Manifold routines except for Manifold::construct() are
- * implemented for this class.
+ * All optional Manifold routines are implemented for this class.
  *
  * This class supports copying but does not implement separate move operations,
  * since its internal data is so small that copying is just as efficient.
@@ -79,26 +78,6 @@ class Handlebody : public Manifold {
          * @return the genus; that is, number of handles.
          */
         size_t genus() const;
-        /**
-         * Deprecated alias for genus().
-         *
-         * \deprecated This routine has been renamed to genus().
-         *
-         * @return the genus of this handlebody; that is, number of handles.
-         */
-        [[deprecated]] size_t handles() const;
-        /**
-         * Deprecated routine that returns whether this handlebody is
-         * orientable.
-         *
-         * \deprecated Since Regina 7.0, this class only supports
-         * orientable handlebodies, and so this routine will always
-         * return \c true.
-         *
-         * @return \c true always, since only orientable handlebodies
-         * are supported by this class.
-         */
-        [[deprecated]] bool isOrientable() const;
         /**
          * Determines whether this and the given handlebody have the same
          * genus.
@@ -137,6 +116,7 @@ class Handlebody : public Manifold {
          */
         void swap(Handlebody& other) noexcept;
 
+        Triangulation<3> construct() const override;
         AbelianGroup homology() const override;
         bool isHyperbolic() const override;
         std::ostream& writeName(std::ostream& out) const override;
@@ -163,12 +143,6 @@ inline Handlebody::Handlebody(size_t genus) : genus_(genus) {
 inline size_t Handlebody::genus() const {
     return genus_;
 }
-inline size_t Handlebody::handles() const {
-    return genus_;
-}
-inline bool Handlebody::isOrientable() const {
-    return true;
-}
 inline bool Handlebody::operator == (const Handlebody& compare) const {
     return (genus_ == compare.genus_);
 }
