@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -1030,8 +1030,8 @@ inline Cyclotomic& Cyclotomic::operator /= (const Cyclotomic& other) {
 
 inline std::string Cyclotomic::str(const char* variable) const {
     // Make sure that python will be able to find the inherited str().
-    static_assert(std::is_same<typename OutputBase<Cyclotomic>::type,
-        Output<Cyclotomic, true>>::value,
+    static_assert(std::is_same_v<typename OutputBase<Cyclotomic>::type,
+        Output<Cyclotomic, true>>,
         "Cyclotomic is not identified as being inherited from Output<...>");
 
     std::ostringstream out;
