@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -32,6 +32,7 @@
 
 #include <algorithm>
 #include <functional>
+#include <numeric> // for std::gcd()
 #include "maths/numbertheory.h"
 
 namespace regina {
@@ -106,16 +107,6 @@ namespace {
     }
 }
 
-long gcdWithCoeffs(long a, long b, long& u, long& v) {
-    long signA = (a > 0 ? 1 : a == 0 ? 0 : -1);
-    long signB = (b > 0 ? 1 : b == 0 ? 0 : -1);
-    long ans = gcdWithCoeffsInternal(a >= 0 ? a : -a,
-        b >= 0 ? b : -b, u, v);
-    u *= signA;
-    v *= signB;
-    return ans;
-}
-
 std::tuple<long, long, long> gcdWithCoeffs(long a, long b) {
     long signA = (a > 0 ? 1 : a == 0 ? 0 : -1);
     long signB = (b > 0 ? 1 : b == 0 ? 0 : -1);
@@ -132,7 +123,7 @@ long lcm(long a, long b) {
     if (a == 0 || b == 0)
         return 0;
 
-    long tmp = regina::gcd(a, b);
+    long tmp = std::gcd(a, b);
     tmp = (a / tmp) * b;
     return (tmp >= 0 ? tmp : -tmp);
 }
