@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -67,19 +67,21 @@ namespace regina {
 long reducedMod(long k, long modBase);
 
 /**
- * Calculates the greatest common divisor of two signed integers.
- * This routine is not recursive.
+ * Deprecated routine that calculates the greatest common divisor of
+ * two signed integers.  This routine is not recursive.
  *
  * Although the arguments may be negative, the result is guaranteed to
  * be non-negative.  As a special case, gcd(0,0) is considered to be zero.
  *
+ * \deprecated Simply use std::gcd(), which was introduced with C++17.
+ *
  * @param a one of the two integers to work with.
  * @param b the other integer with which to work.
  * @return the greatest common divisor of \a a and \a b.
  *
  * \ingroup maths
  */
-long gcd(long a, long b);
+[[deprecated]] long gcd(long a, long b);
 
 /**
  * Calculates the greatest common divisor of two given integers and finds the
@@ -112,47 +114,24 @@ long gcd(long a, long b);
 std::tuple<long, long, long> gcdWithCoeffs(long a, long b);
 
 /**
- * Deprecated function that calculates the greatest common divisor of two
- * given integers and finds the smallest coefficients with which these
- * integers combine to give their gcd.  This routine is not recursive.
- *
- * \deprecated This function has been redesigned to take just \a a and \a b
- * as argument, and to return (\a gcd, \a u, \a v) as a tuple.
- * See that version of gcdWithCoeffs() for further details on how this
- * function operates.
- *
- * \ifacespython Not present; instead you can use the two-argument variant
- * that returns the tuple (\a gcd, \a u, \a v).
- *
- * @param a the first integer to compute the gcd of.
- * @param b the second integer to compute the gcd of.
- * @param u a variable into which the final coefficient of \a a will be
- * placed.  Any existing contents of \a u will be overwritten.
- * @param v a variable into which the final coefficient of \a b will be
- * placed.  Any existing contents of \a v will be overwritten.
- * @return the greatest common divisor of \a a and \a b.
- *
- * \ingroup maths
- */
-[[deprecated]] long gcdWithCoeffs(long a, long b, long& u, long& v);
-
-/**
- * Calculates the lowest common multiple of two signed integers.
- * Although the arguments may be negative, the result is guaranteed to
- * be non-negative.
+ * Deprecated routine that calculates the lowest common multiple of
+ * two signed integers.  Although the arguments may be negative, the result
+ * is guaranteed to be non-negative.
  *
  * If either of the arguments is zero, the return value will also be zero.
  *
  * Regarding possible overflow:  This routine does not create any temporary
  * integers that are larger than the final LCM.
  *
+ * \deprecated Simply use std::lcm(), which was introduced with C++17.
+ *
  * @param a one of the two integers to work with.
  * @param b the other integer with which to work.
  * @return the lowest common multiple of \a a and \a b.
  *
  * \ingroup maths
  */
-long lcm(long a, long b);
+[[deprecated]] long lcm(long a, long b);
 
 /**
  * Calculates the multiplicative inverse of one integer modulo another.
