@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -413,9 +413,6 @@ class Rational {
          * \exception UnsolvedCase This rational lies outside double's
          * allowable range.
          *
-         * \ifacespython The old python-only routine doubleApproxCheck() is
-         * now deprecated.  Just call this routine inside a try/catch block.
-         *
          * @return the double approximation to this rational.
          *
          * @author Ryan Budney, B.B.
@@ -432,16 +429,6 @@ class Rational {
          */
         std::string tex() const;
 
-        /**
-         * Deprecated routine that returns this rational as written using TeX
-         * formatting.
-         *
-         * \deprecated This routine has been renamed to tex().
-         *
-         * @return this rational as written using TeX formatting.
-         */
-        [[deprecated]] std::string TeX() const;
-
         /**
          * Writes this rational in TeX format to the given output stream.
          * No leading or trailing dollar signs will be included.
@@ -615,10 +602,6 @@ inline bool Rational::operator != (const Rational& compare) const {
     return ! (*this == compare);
 }
 
-inline std::string Rational::TeX() const {
-    return tex();
-}
-
 inline void swap(Rational& a, Rational& b) noexcept {
     a.swap(b);
 }
