@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -48,10 +48,6 @@
 #define __REGINA_PERM3_H
 #endif
 
-#include <cstdlib>
-#include <string>
-#include "regina-core.h"
-
 namespace regina {
 
 /**
@@ -342,38 +338,6 @@ class Perm<3> {
          */
         constexpr Perm(const std::array<int, 3>& image);
 
-        /**
-         * Deprecated constructor that creates a permutation mapping
-         * \a i to \a image[i] for each \a i = 0,1,2.
-         *
-         * \deprecated Use the three-integer constructor or the
-         * std::array constructor instead.
-         *
-         * \pre The array \a image contains three elements, which are
-         * 0, 1 and 2 in some order.
-         *
-         * @param image the array of images.
-         */
-        [[deprecated]] constexpr Perm(const int* image);
-
-        /**
-         * Deprecated constructor that creates a permutation mapping
-         * (\a a[0], ..., \a a[2]) to (\a b[0], ..., \a b[2]) respectively.
-         *
-         * \deprecated Use the six-integer constructor or the
-         * std::array constructor instead.
-         *
-         * \pre Both arrays \a a and \a b contain 3 elements, which
-         * are 0,...,2 in some order.
-         *
-         * \ifacespython Not present; use the single-array constructor instead.
-         *
-         * @param a the array of preimages; this must have length 3.
-         * @param b the corresponding array of images; this must also have
-         * length 3.
-         */
-        [[deprecated]] constexpr Perm(const int* a, const int* b);
-
         /**
          * Creates a permutation mapping (<i>a0</i>,<i>b0</i>,<i>c0</i>) to
          * (<i>a1</i>,<i>b1</i>,<i>c1</i>) respectively.
@@ -528,18 +492,6 @@ class Perm<3> {
          */
         constexpr int pre(int image) const;
 
-        /**
-         * Deprecated routine that determines the preimage of the given
-         * integer under this permutation.
-         *
-         * \deprecated This routine has been renamed to pre().
-         *
-         * @param image the integer whose preimage we wish to find.  This
-         * should be between 0 and 2 inclusive.
-         * @return the preimage of \a image.
-         */
-        [[deprecated]] constexpr int preImageOf(int image) const;
-
         /**
          * Determines if this is equal to the given permutation.
          * This is true if and only if both permutations have the same
@@ -698,7 +650,7 @@ class Perm<3> {
          * @return the corresponding prefix of the string representation
          * of this permutation.
          */
-        std::string trunc(unsigned len) const;
+        std::string trunc(int len) const;
 
         /**
          * Returns a string representation of this permutation with only
@@ -709,6 +661,89 @@ class Perm<3> {
          */
         std::string trunc2() const;
 
+        /**
+         * Writes the tight encoding of this permutation to the given output
+         * stream.  See the page on \ref tight "tight encodings" for details.
+         *
+         * For all permutation classes Perm<n>, the tight encoding is based on
+         * the index into the full permutation group \a S_n.  For smaller
+         * permutation classes (\a n &le; 7), such encodings are very fast to
+         * work with since the \a S_n index is used as the internal permutation
+         * code.  For larger permutation classes however (8 &le; \a n &le; 16),
+         * the \a S_n index requires some non-trivial work to compute.
+         *
+         * \ifacespython Not present; use tightEncoding() instead, which
+         * returns a string.
+         *
+         * @param out the output stream to which the encoded string will
+         * be written.
+         */
+        void tightEncode(std::ostream& out) const;
+
+        /**
+         * Returns the tight encoding of this permutation.
+         * See the page on \ref tight "tight encodings" for details.
+         *
+         * For all permutation classes Perm<n>, the tight encoding is based on
+         * the index into the full permutation group \a S_n.  For smaller
+         * permutation classes (\a n &le; 7), such encodings are very fast to
+         * work with since the \a S_n index is used as the internal permutation
+         * code.  For larger permutation classes however (8 &le; \a n &le; 16),
+         * the \a S_n index requires some non-trivial work to compute.
+         *
+         * @return the resulting encoded string.
+         */
+        std::string tightEncoding() const;
+
+        /**
+         * Reconstructs a permutation from its given tight encoding.
+         * See the page on \ref tight "tight encodings" for details.
+         *
+         * The tight encoding will be given as a string.  If this string
+         * contains leading whitespace or any trailing characters at all
+         * (including trailing whitespace), then it will be treated as
+         * an invalid encoding (i.e., this routine will throw an exception).
+         *
+         * Tight encodings are fast to work with for small permutation classes
+         * (\a n &le; 7), but slower for larger permutation classes
+         * (8 &le; \a n &le; 16).  See tightEncoding() for further details.
+         *
+         * \exception InvalidArgument the given string is not a tight encoding
+         * of a 3-element permutation.
+         *
+         * @param enc the tight encoding for a 3-element permutation.
+         * @return the permutation represented by the given tight encoding.
+         */
+        static Perm tightDecoding(const std::string& enc);
+
+        /**
+         * Reconstructs a permutation from its given tight encoding.
+         * See the page on \ref tight "tight encodings" for details.
+         *
+         * The tight encoding will be read from the given input stream.
+         * If the input stream contains leading whitespace then it will be
+         * treated as an invalid encoding (i.e., this routine will throw an
+         * exception).  The input routine \e may contain further data: if this
+         * routine is successful then the input stream will be left positioned
+         * immediately after the encoding, without skipping any trailing
+         * whitespace.
+         *
+         * Tight encodings are fast to work with for small permutation classes
+         * (\a n &le; 7), but slower for larger permutation classes
+         * (8 &le; \a n &le; 16).  See tightEncoding() for further details.
+         *
+         * \exception InvalidInput the given input stream does not begin with
+         * a tight encoding of a 3-element permutation.
+         *
+         * \ifacespython Not present; use tightDecoding() instead, which takes
+         * a string as its argument.
+         *
+         * @param input an input stream that begins with the tight encoding
+         * for a 3-element permutation.
+         * @return the permutation represented by the given tight encoding.
+         */
+        static Perm tightDecode(std::istream& input);
+
         /**
          * Resets the images of all integers from \a from onwards to the
          * identity map.
@@ -778,28 +813,6 @@ class Perm<3> {
          */
         constexpr Index orderedS3Index() const;
 
-        /**
-         * Deprecated routine that returns the lexicographical index of this
-         * permutation.
-         *
-         * \deprecated Use the equivalent routine orderedSnIndex() instead.
-         *
-         * @return the lexicographical index of this permutation.
-         */
-        [[deprecated]] constexpr Index index() const;
-
-        /**
-         * Deprecated routine that returns the <i>i</i>th permutation on
-         * three elements, where permutations are numbered lexicographically.
-         *
-         * \deprecated Use orderedSn[\a i] instead.
-         *
-         * @param i the lexicographical index of the permutation; this
-         * must be between 0 and 5 inclusive.
-         * @return the <i>i</i>th permutation.
-         */
-        [[deprecated]] static constexpr Perm atIndex(Index i);
-
         /**
          * Extends a <i>k</i>-element permutation to an 3-element permutation.
          * where 2 &le; \a k &lt; 3.  The only possible value of \a k is 2, but
@@ -927,6 +940,38 @@ class Perm<3> {
          */
         template <typename Int>
         static constexpr Int convOrderedUnordered(Int index);
+
+        /**
+         * Reconstructs a permutation from its given tight encoding.
+         *
+         * The tight encoding will be extracted one character at a time
+         * beginning with the iterator \a start, in a single pass, without
+         * skipping any leading whitespace.  If the iterator ever reaches
+         * \a limit before the encoding is complete then the encoding is
+         * treated as invalid (i.e., this routine will throw an exception).
+         *
+         * If \a noTrailingData is \c true then the iterator is required to
+         * \e finish at \a limit, or else the encoding will be considered
+         * invalid also; if \a noTrailingData is \c false then there is no
+         * constraint on the final state of the iterator.
+         *
+         * \exception InvalidInput the given iterator does not point to
+         * a tight encoding of a 3-element permutation.
+         *
+         * \tparam iterator an input iterator type.
+         *
+         * @param start an iterator that points to the beginning of a
+         * tight encoding.
+         * @param limit an iterator that, if reached, indicates that no more
+         * characters are available.
+         * @param noTrailingData \c true if iteration should reach \a limit
+         * immediately after the encoding is read, or \c false if there is
+         * allowed to be additional unread data.
+         * @return the permutation represented by the given tight encoding.
+         */
+        template <typename iterator>
+        static Perm tightDecode(iterator start, iterator limit,
+            bool noTrailingData);
 };
 
 // Inline functions for Perm<3>
@@ -976,29 +1021,6 @@ inline constexpr Perm<3>::Perm(const std::array<int, 3>& image) :
                               (image[1] == 0 ? 4 : 5)) {
 }
 
-inline constexpr Perm<3>::Perm(const int* image) :
-        code_(image[0] == 0 ? (image[1] == 1 ? 0 : 1) :
-              image[0] == 1 ? (image[1] == 2 ? 2 : 3) :
-                              (image[1] == 0 ? 4 : 5)) {
-}
-
-inline constexpr Perm<3>::Perm(const int* a, const int* b) : code_(0) {
-    // TODO: When we move to C++20, we can get rid of the zero initialisers.
-    int image[3] = { 0, 0, 0 };
-    image[a[0]] = b[0];
-    image[a[1]] = b[1];
-    image[a[2]] = b[2];
-
-    switch (image[0]) {
-        case 0:
-            code_ = static_cast<Code>(image[1] == 1 ? 0 : 1); break;
-        case 1:
-            code_ = static_cast<Code>(image[1] == 2 ? 2 : 3); break;
-        case 2:
-            code_ = static_cast<Code>(image[1] == 0 ? 4 : 5); break;
-    }
-}
-
 inline constexpr Perm<3>::Perm(int a0, int a1, int b0, int b1, int c0, int c1) :
         code_(0) {
     // TODO: When we move to C++20, we can get rid of the zero initialisers.
@@ -1083,10 +1105,6 @@ inline constexpr int Perm<3>::pre(int image) const {
     return imageTable[invS3[code_]][image];
 }
 
-inline constexpr int Perm<3>::preImageOf(int image) const {
-    return imageTable[invS3[code_]][image];
-}
-
 inline constexpr bool Perm<3>::operator == (const Perm<3>& other) const {
     return (code_ == other.code_);
 }
@@ -1148,6 +1166,45 @@ inline Perm<3> Perm<3>::rand(URBG&& gen, bool even) {
     }
 }
 
+inline void Perm<3>::tightEncode(std::ostream& out) const {
+    out << static_cast<char>(code_ + 33);
+}
+
+inline std::string Perm<3>::tightEncoding() const {
+    char ans[2] { static_cast<char>(code_ + 33), 0 };
+    return ans;
+}
+
+inline Perm<3> Perm<3>::tightDecoding(const std::string& enc) {
+    try {
+        return tightDecode(enc.begin(), enc.end(), true);
+    } catch (const InvalidInput& exc) {
+        // For strings we use a different exception type.
+        throw InvalidArgument(exc.what());
+    }
+}
+
+inline Perm<3> Perm<3>::tightDecode(std::istream& input) {
+    return tightDecode(std::istreambuf_iterator<char>(input),
+        std::istreambuf_iterator<char>(), false);
+}
+
+template <typename iterator>
+Perm<3> Perm<3>::tightDecode(iterator start, iterator limit,
+        bool noTrailingData) {
+    if (start == limit)
+        throw InvalidInput("The tight encoding is incomplete");
+
+    Code code = (*start++) - 33;
+    // code >= 0 because we are using an unsigned data type.
+    if (code >= 6)
+        throw InvalidInput("The tight encoding is invalid");
+    if (noTrailingData && (start != limit))
+        throw InvalidInput("The tight encoding has trailing characters");
+
+    return Perm<3>(code);
+}
+
 inline constexpr Perm<3>::Index Perm<3>::S3Index() const {
     return code_;
 }
@@ -1164,14 +1221,6 @@ inline constexpr Perm<3>::Index Perm<3>::orderedSnIndex() const {
     return convOrderedUnordered(code_);
 }
 
-inline constexpr Perm<3>::Index Perm<3>::index() const {
-    return convOrderedUnordered(code_);
-}
-
-inline constexpr Perm<3> Perm<3>::atIndex(Index i) {
-    return orderedS3[i];
-}
-
 inline constexpr bool Perm<3>::isConjugacyMinimal() const {
     return (code_ < 3);
 }
