@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -48,10 +48,6 @@
 #define __REGINA_PERM4_H
 #endif
 
-#include <cstdlib>
-#include <string>
-#include "regina-core.h"
-
 namespace regina {
 
 /**
@@ -253,14 +249,6 @@ class Perm<4> {
          */
         using Code2 = uint8_t;
 
-        /**
-         * An alias for the first-generation code type Code1.
-         *
-         * Instead of Code, you should use either Code1 or Code2 to more
-         * clearly express which kind of permutation code you are using.
-         */
-        using Code [[deprecated]] = Code1;
-
         /**
          * Gives array-like access to all possible permutations of
          * four elements.
@@ -470,38 +458,6 @@ class Perm<4> {
          */
         constexpr Perm(const std::array<int, 4>& image);
 
-        /**
-         * Deprecated constructor that creates a permutation mapping
-         * \a i to \a image[i] for each \a i = 0,1,2,3.
-         *
-         * \deprecated Use the four-integer constructor or the
-         * std::array constructor instead.
-         *
-         * \pre The array \a image contains four elements, which are
-         * 0, 1, 2 and 3 in some order.
-         *
-         * @param image the array of images.
-         */
-        [[deprecated]] constexpr Perm(const int* image);
-
-        /**
-         * Deprecated constructor that creates a permutation mapping
-         * (\a a[0], ..., \a a[3]) to (\a b[0], ..., \a b[3]) respectively.
-         *
-         * \deprecated Use the eight-integer constructor or the
-         * std::array constructor instead.
-         *
-         * \pre Both arrays \a a and \a b contain 4 elements, which
-         * are 0,...,3 in some order.
-         *
-         * \ifacespython Not present; use the single-array constructor instead.
-         *
-         * @param a the array of preimages; this must have length 4.
-         * @param b the corresponding array of images; this must also have
-         * length 4.
-         */
-        [[deprecated]] constexpr Perm(const int* a, const int* b);
-
         /**
          * Creates a permutation mapping
          * (<i>a0</i>,<i>b0</i>,<i>c0</i>,<i>d0</i>) to
@@ -560,26 +516,6 @@ class Perm<4> {
          */
         constexpr Code2 permCode2() const;
 
-        /**
-         * Deprecated routine that returns the first-generation code
-         * representing this permutation.
-         *
-         * The code returned will be a valid first-generation permutation
-         * code as determined by isPermCode1().
-         *
-         * \deprecated Use permCode1() to reproduce this behaviour.
-         * However, unless you need backward compatibility, it is
-         * strongly recommended to switch to the much faster
-         * second-generation codes instead.
-         *
-         * \warning This routine will incur additional overhead, since
-         * Perm<4> now uses second-generation codes internally.
-         * See the class notes and the routine permCode2() for details.
-         *
-         * @return the first-generation permutation code.
-         */
-        [[deprecated]] constexpr Code1 permCode() const;
-
         /**
          * Sets this permutation to that represented by the given
          * first-generation permutation code.
@@ -611,27 +547,6 @@ class Perm<4> {
          */
         void setPermCode2(Code2 code);
 
-        /**
-         * Deprecated routine that sets this permutation to that represented
-         * by the given first-generation permutation code.
-         *
-         * \deprecated Use setPermCode1() to reproduce this behaviour.
-         * However, unless you need backward compatibility, it is
-         * strongly recommended to switch to the much faster
-         * second-generation codes instead.
-         *
-         * \pre the given code is a valid first-generation permutation code;
-         * see isPermCode1() for details.
-         *
-         * \warning This routine will incur additional overhead, since
-         * Perm<4> now uses second-generation codes internally.
-         * See the class notes and the routine setPermCode2() for details.
-         *
-         * @param code the first-generation code that will determine the
-         * new value of this permutation.
-         */
-        [[deprecated]] void setPermCode(Code1 code);
-
         /**
          * Creates a permutation from the given first-generation
          * permutation code.
@@ -663,27 +578,6 @@ class Perm<4> {
          */
         static constexpr Perm<4> fromPermCode2(Code2 code);
 
-        /**
-         * Deprecated routine that creates a permutation from the given
-         * first-generation permutation code.
-         *
-         * \pre the given code is a valid first-generation permutation code;
-         * see isPermCode1() for details.
-         *
-         * \deprecated Use fromPermCode1() to reproduce this behaviour.
-         * However, unless you need backward compatibility, it is
-         * strongly recommended to switch to the much faster
-         * second-generation codes instead.
-         *
-         * \warning This routine will incur additional overhead, since
-         * Perm<4> now uses second-generation codes internally.
-         * See the class notes and the routine fromPermCode2() for details.
-         *
-         * @param code the first-generation code for the new permutation.
-         * @return the permutation represented by the given code.
-         */
-        [[deprecated]] static constexpr Perm<4> fromPermCode(Code1 code);
-
         /**
          * Determines whether the given character is a valid first-generation
          * permutation code.  Valid first-generation codes can be passed to
@@ -713,25 +607,6 @@ class Perm<4> {
          */
         static constexpr bool isPermCode2(Code2 code);
 
-        /**
-         * Deprecated routine that determines whether the given character
-         * is a valid first-generation permutation code.
-         *
-         * \deprecated Use isPermCode1() to reproduce this behaviour.
-         * However, unless you need backward compatibility, it is
-         * strongly recommended to switch to the much faster
-         * second-generation codes instead.
-         *
-         * \warning This routine will incur additional overhead, since
-         * Perm<4> now uses second-generation codes internally.
-         * See the class notes and the routine isPermCode2() for details.
-         *
-         * @param code the permutation code to test.
-         * @return \c true if and only if the given code is a valid
-         * first-generation permutation code.
-         */
-        [[deprecated]] static constexpr bool isPermCode(Code1 code);
-
         /**
          * Returns the image pack that represents this permutation.
          *
@@ -859,18 +734,6 @@ class Perm<4> {
          */
         constexpr int pre(int image) const;
 
-        /**
-         * Deprecated routine that determines the preimage of the given
-         * integer under this permutation.
-         *
-         * \deprecated This routine has been renamed to pre().
-         *
-         * @param image the integer whose preimage we wish to find.  This
-         * should be between 0 and 3 inclusive.
-         * @return the preimage of \a image.
-         */
-        [[deprecated]] constexpr int preImageOf(int image) const;
-
         /**
          * Determines if this is equal to the given permutation.
          * This is true if and only if both permutations have the same
@@ -1031,7 +894,7 @@ class Perm<4> {
          * @return the corresponding prefix of the string representation
          * of this permutation.
          */
-        std::string trunc(unsigned len) const;
+        std::string trunc(int len) const;
 
         /**
          * Returns a string representation of this permutation with only
@@ -1051,6 +914,89 @@ class Perm<4> {
          */
         std::string trunc3() const;
 
+        /**
+         * Writes the tight encoding of this permutation to the given output
+         * stream.  See the page on \ref tight "tight encodings" for details.
+         *
+         * For all permutation classes Perm<n>, the tight encoding is based on
+         * the index into the full permutation group \a S_n.  For smaller
+         * permutation classes (\a n &le; 7), such encodings are very fast to
+         * work with since the \a S_n index is used as the internal permutation
+         * code.  For larger permutation classes however (8 &le; \a n &le; 16),
+         * the \a S_n index requires some non-trivial work to compute.
+         *
+         * \ifacespython Not present; use tightEncoding() instead, which
+         * returns a string.
+         *
+         * @param out the output stream to which the encoded string will
+         * be written.
+         */
+        void tightEncode(std::ostream& out) const;
+
+        /**
+         * Returns the tight encoding of this permutation.
+         * See the page on \ref tight "tight encodings" for details.
+         *
+         * For all permutation classes Perm<n>, the tight encoding is based on
+         * the index into the full permutation group \a S_n.  For smaller
+         * permutation classes (\a n &le; 7), such encodings are very fast to
+         * work with since the \a S_n index is used as the internal permutation
+         * code.  For larger permutation classes however (8 &le; \a n &le; 16),
+         * the \a S_n index requires some non-trivial work to compute.
+         *
+         * @return the resulting encoded string.
+         */
+        std::string tightEncoding() const;
+
+        /**
+         * Reconstructs a permutation from its given tight encoding.
+         * See the page on \ref tight "tight encodings" for details.
+         *
+         * The tight encoding will be given as a string.  If this string
+         * contains leading whitespace or any trailing characters at all
+         * (including trailing whitespace), then it will be treated as
+         * an invalid encoding (i.e., this routine will throw an exception).
+         *
+         * Tight encodings are fast to work with for small permutation classes
+         * (\a n &le; 7), but slower for larger permutation classes
+         * (8 &le; \a n &le; 16).  See tightEncoding() for further details.
+         *
+         * \exception InvalidArgument the given string is not a tight encoding
+         * of a 4-element permutation.
+         *
+         * @param enc the tight encoding for a 4-element permutation.
+         * @return the permutation represented by the given tight encoding.
+         */
+        static Perm tightDecoding(const std::string& enc);
+
+        /**
+         * Reconstructs a permutation from its given tight encoding.
+         * See the page on \ref tight "tight encodings" for details.
+         *
+         * The tight encoding will be read from the given input stream.
+         * If the input stream contains leading whitespace then it will be
+         * treated as an invalid encoding (i.e., this routine will throw an
+         * exception).  The input routine \e may contain further data: if this
+         * routine is successful then the input stream will be left positioned
+         * immediately after the encoding, without skipping any trailing
+         * whitespace.
+         *
+         * Tight encodings are fast to work with for small permutation classes
+         * (\a n &le; 7), but slower for larger permutation classes
+         * (8 &le; \a n &le; 16).  See tightEncoding() for further details.
+         *
+         * \exception InvalidInput the given input stream does not begin with
+         * a tight encoding of a 4-element permutation.
+         *
+         * \ifacespython Not present; use tightDecoding() instead, which takes
+         * a string as its argument.
+         *
+         * @param input an input stream that begins with the tight encoding
+         * for a 4-element permutation.
+         * @return the permutation represented by the given tight encoding.
+         */
+        static Perm tightDecode(std::istream& input);
+
         /**
          * Resets the images of all integers from \a from onwards to the
          * identity map.
@@ -1120,28 +1066,6 @@ class Perm<4> {
          */
         constexpr Index orderedS4Index() const;
 
-        /**
-         * Deprecated routine that returns the lexicographical index of this
-         * permutation.
-         *
-         * \deprecated Use the equivalent routine orderedSnIndex() instead.
-         *
-         * @return the lexicographical index of this permutation.
-         */
-        [[deprecated]] constexpr Index index() const;
-
-        /**
-         * Deprecated routine that returns the <i>i</i>th permutation on
-         * four elements, where permutations are numbered lexicographically.
-         *
-         * \deprecated Use orderedSn[\a i] instead.
-         *
-         * @param i the lexicographical index of the permutation; this
-         * must be between 0 and 23 inclusive.
-         * @return the <i>i</i>th permutation.
-         */
-        [[deprecated]] static constexpr Perm atIndex(Index i);
-
         /**
          * Extends a <i>k</i>-element permutation to a 4-element permutation,
          * where 2 &le; \a k &lt; 4.
@@ -1354,6 +1278,38 @@ class Perm<4> {
          */
         template <typename Int>
         static constexpr Int convOrderedUnordered(Int index);
+
+        /**
+         * Reconstructs a permutation from its given tight encoding.
+         *
+         * The tight encoding will be extracted one character at a time
+         * beginning with the iterator \a start, in a single pass, without
+         * skipping any leading whitespace.  If the iterator ever reaches
+         * \a limit before the encoding is complete then the encoding is
+         * treated as invalid (i.e., this routine will throw an exception).
+         *
+         * If \a noTrailingData is \c true then the iterator is required to
+         * \e finish at \a limit, or else the encoding will be considered
+         * invalid also; if \a noTrailingData is \c false then there is no
+         * constraint on the final state of the iterator.
+         *
+         * \exception InvalidInput the given iterator does not point to
+         * a tight encoding of a 4-element permutation.
+         *
+         * \tparam iterator an input iterator type.
+         *
+         * @param start an iterator that points to the beginning of a
+         * tight encoding.
+         * @param limit an iterator that, if reached, indicates that no more
+         * characters are available.
+         * @param noTrailingData \c true if iteration should reach \a limit
+         * immediately after the encoding is read, or \c false if there is
+         * allowed to be additional unread data.
+         * @return the permutation represented by the given tight encoding.
+         */
+        template <typename iterator>
+        static Perm tightDecode(iterator start, iterator limit,
+            bool noTrailingData);
 };
 
 // Inline functions for Perm<4>
@@ -1409,11 +1365,6 @@ inline constexpr Perm<4>::Perm(const std::array<int, 4>& image) :
         image[0], image[1], image[2], image[3]))) {
 }
 
-inline constexpr Perm<4>::Perm(const int* image) :
-        code_(static_cast<Code2>(S4Index(
-        image[0], image[1], image[2], image[3]))) {
-}
-
 inline constexpr Perm<4>::Perm(int a0, int a1, int b0, int b1, int c0, int c1,
         int d0, int d1) : code_(0) {
     // TODO: When we move to C++20, we can get rid of the zero initialisers.
@@ -1425,16 +1376,6 @@ inline constexpr Perm<4>::Perm(int a0, int a1, int b0, int b1, int c0, int c1,
     code_ = static_cast<Code2>(S4Index(image[0], image[1], image[2], image[3]));
 }
 
-inline constexpr Perm<4>::Perm(const int* a, const int* b) : code_(0) {
-    // TODO: When we move to C++20, we can get rid of the zero initialisers.
-    int image[4] = { 0, 0, 0, 0 };
-    image[a[0]] = b[0];
-    image[a[1]] = b[1];
-    image[a[2]] = b[2];
-    image[a[3]] = b[3];
-    code_ = static_cast<Code2>(S4Index(image[0], image[1], image[2], image[3]));
-}
-
 inline constexpr Perm<4>::Code1 Perm<4>::permCode1() const {
     return static_cast<Code1>(
         imageTable[code_][0] |
@@ -1447,10 +1388,6 @@ inline constexpr Perm<4>::Code2 Perm<4>::permCode2() const {
     return code_;
 }
 
-inline constexpr Perm<4>::Code1 Perm<4>::permCode() const {
-    return permCode1();
-}
-
 inline void Perm<4>::setPermCode1(Code1 code) {
     code_ = static_cast<Code2>(S4Index(
         code & 0x03,
@@ -1463,10 +1400,6 @@ inline void Perm<4>::setPermCode2(Code2 code) {
     code_ = code;
 }
 
-inline void Perm<4>::setPermCode(Code1 code) {
-    setPermCode1(code);
-}
-
 inline constexpr Perm<4> Perm<4>::fromPermCode1(Code1 code) {
     return Perm<4>(static_cast<Code2>(S4Index(
         code & 0x03,
@@ -1479,10 +1412,6 @@ inline constexpr Perm<4> Perm<4>::fromPermCode2(Code2 code) {
     return Perm<4>(code);
 }
 
-inline constexpr Perm<4> Perm<4>::fromPermCode(Code1 code) {
-    return fromPermCode1(code);
-}
-
 inline constexpr bool Perm<4>::isPermCode1(Code1 code) {
     // There are no "spare" bits since Code1 is uint8_t, which means there is
     // no need to check that only the first 8 bits are in use.
@@ -1498,10 +1427,6 @@ inline constexpr bool Perm<4>::isPermCode2(Code2 code) {
     return (code < 24);
 }
 
-inline constexpr bool Perm<4>::isPermCode(Code1 code) {
-    return isPermCode1(code);
-}
-
 inline constexpr Perm<4>::ImagePack Perm<4>::imagePack() const {
     return permCode1();
 }
@@ -1566,10 +1491,6 @@ inline constexpr int Perm<4>::pre(int image) const {
     return imageTable[invS4[code_]][image];
 }
 
-inline constexpr int Perm<4>::preImageOf(int image) const {
-    return imageTable[invS4[code_]][image];
-}
-
 inline constexpr int Perm<4>::compareWith(const Perm<4>& other) const {
     // Computing orderedS4Index() is very fast, now that we use S4 indices
     // for internal permutation codes.  Use this instead of comparing images
@@ -1633,6 +1554,45 @@ inline constexpr bool Perm<4>::operator != (const Perm<4>& other) const {
     return (code_ != other.code_);
 }
 
+inline void Perm<4>::tightEncode(std::ostream& out) const {
+    out << static_cast<char>(code_ + 33);
+}
+
+inline std::string Perm<4>::tightEncoding() const {
+    char ans[2] { static_cast<char>(code_ + 33), 0 };
+    return ans;
+}
+
+inline Perm<4> Perm<4>::tightDecoding(const std::string& enc) {
+    try {
+        return tightDecode(enc.begin(), enc.end(), true);
+    } catch (const InvalidInput& exc) {
+        // For strings we use a different exception type.
+        throw InvalidArgument(exc.what());
+    }
+}
+
+inline Perm<4> Perm<4>::tightDecode(std::istream& input) {
+    return tightDecode(std::istreambuf_iterator<char>(input),
+        std::istreambuf_iterator<char>(), false);
+}
+
+template <typename iterator>
+Perm<4> Perm<4>::tightDecode(iterator start, iterator limit,
+        bool noTrailingData) {
+    if (start == limit)
+        throw InvalidInput("The tight encoding is incomplete");
+
+    Code2 code = (*start++) - 33;
+    // code >= 0 because we are using an unsigned data type.
+    if (code >= 24)
+        throw InvalidInput("The tight encoding is invalid");
+    if (noTrailingData && (start != limit))
+        throw InvalidInput("The tight encoding has trailing characters");
+
+    return Perm<4>(code);
+}
+
 inline constexpr int Perm<4>::S4Index(int a, int b, int c, int d) {
     // First compute the ordered S4 index.
     int ans = 6 * a +
@@ -1659,14 +1619,6 @@ inline constexpr Perm<4>::Index Perm<4>::orderedSnIndex() const {
     return convOrderedUnordered(code_);
 }
 
-inline constexpr Perm<4>::Index Perm<4>::index() const {
-    return convOrderedUnordered(code_);
-}
-
-inline constexpr Perm<4> Perm<4>::atIndex(Index i) {
-    return orderedS4[i];
-}
-
 inline constexpr bool Perm<4>::isConjugacyMinimal() const {
     return (code_ < 3 || code_ == 6 || code_ == 9);
 }
