@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -172,11 +172,10 @@ class Attachment : public Packet {
          * which case the new packet will have no attachment stored
          * (so isNull() will return \c true).
          *
-         * \ifacespython The \a size and \a alloc arguments are not present,
-         * since a Python \c bytes object knows its size, and in Python the
+         * \ifacespython The argument \a data should be a Python \c bytes
+         * object.  The \a size and \a alloc arguments are not present,
+         * since a \c bytes object knows its size, and in Python the
          * only available ownership policy is to make a deep copy.
-         * In Python 3, the binary data should be passed as a Python
-         * \c bytes object; in Python 2 it should be passed as a byte string.
          *
          * @param data the block of binary data that forms the attachment,
          * or \c null if no attachment is to be stored.
@@ -262,11 +261,10 @@ class Attachment : public Packet {
          * (i.e., if isNull() returns \c true), then this routine will return
          * \c null.
          *
-         * \ifacespython In Python 3, this routine returns a Python \c bytes
-         * object; in Python 2 it returns a Python byte string.  Unlike the
-         * C++ version of this routine, the Python version returns a deep copy
-         * of the raw data (so this routine becomes slower, but the resulting
-         * byte sequence can be freely modified).
+         * \ifacespython This routine returns a Python \c bytes object.
+         * Unlike the C++ version of this routine, the Python version returns
+         * a deep copy of the raw data (so this routine becomes slower, but
+         * the resulting byte sequence can be freely modified).
          *
          * @return the raw attachment data.
          */
@@ -340,11 +338,10 @@ class Attachment : public Packet {
          * \warning This cannot be used for self-assignment.  If the argument
          * \a data is in fact data() then this routine will most likely crash.
          *
-         * \ifacespython The \a size and \a alloc arguments are not present,
-         * since a Python \c bytes object knows its size, and in Python the
+         * \ifacespython The argument \a data should be a Python \c bytes
+         * object.  The \a size and \a alloc arguments are not present,
+         * since a \c bytes object knows its size, and in Python the
          * only available ownership policy is to make a deep copy.
-         * In Python 3, the binary data should be passed as a Python
-         * \c bytes object; in Python 2 it should be passed as a byte string.
          *
          * @param data the block of binary data that forms the new attachment,
          * or \c null if no attachment is to be stored.
@@ -378,19 +375,6 @@ class Attachment : public Packet {
          */
         bool save(const char* pathname) const;
 
-        /**
-         * A deprecated function that saves the contents of this
-         * attachment to the given file.
-         *
-         * \deprecated This routine has been renamed to save(), now that this
-         * class can store arbitrary attachments and not just PDF documents.
-         *
-         * @param pathname the full pathname of the file to write.
-         * @return \c true if the file was successfully written, or
-         * \c false otherwise.
-         */
-        [[deprecated]] bool savePDF(const char* pathname) const;
-
         /**
          * Determines if this and the given attachment hold identical data.
          *
@@ -442,17 +426,6 @@ class Attachment : public Packet {
  */
 void swap(Attachment& a, Attachment& b);
 
-/**
- * Deprecated alias for a packet that can hold an arbitrary file attachment.
- *
- * \deprecated The old PDF class has now been renamed to Attachment.  Instead
- * of exclusively storing PDF documents, it can now store any type of
- * file attachment.
- *
- * \ingroup packet
- */
-using PDF [[deprecated]] = Attachment;
-
 // Inline functions for Attachment
 
 inline Attachment::Attachment() : data_(nullptr), size_(0), alloc_(OWN_NEW) {
@@ -550,10 +523,6 @@ inline void swap(Attachment& a, Attachment& b) {
     a.swap(b);
 }
 
-inline bool Attachment::savePDF(const char* pathname) const {
-    return save(pathname);
-}
-
 } // namespace regina
 
 #endif
