@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -127,7 +127,11 @@ std::shared_ptr<Packet> Packet::root() const {
         return const_cast<Packet*>(this)->shared_from_this();
 }
 
-void Packet::insertChildFirst(std::shared_ptr<Packet> child) {
+void Packet::prepend(std::shared_ptr<Packet> child) {
+    if (child->hasParent())
+        throw InvalidArgument("The new child packet being prepended "
+            "already has a parent packet");
+
     fireEvent(&PacketListener::childToBeAdded, *child);
 
     child->treeParent_ = weak_from_this();
@@ -143,7 +147,11 @@ void Packet::insertChildFirst(std::shared_ptr<Packet> child) {
     fireEvent(&PacketListener::childWasAdded, *firstTreeChild_);
 }
 
-void Packet::insertChildLast(std::shared_ptr<Packet> child) {
+void Packet::append(std::shared_ptr<Packet> child) {
+    if (child->hasParent())
+        throw InvalidArgument("The new child packet being appended "
+            "already has a parent packet");
+
     fireEvent(&PacketListener::childToBeAdded, *child);
 
     child->treeParent_ = weak_from_this();
@@ -159,13 +167,20 @@ void Packet::insertChildLast(std::shared_ptr<Packet> child) {
     fireEvent(&PacketListener::childWasAdded, *lastTreeChild_);
 }
 
-void Packet::insertChildAfter(std::shared_ptr<Packet> newChild,
+void Packet::insert(std::shared_ptr<Packet> newChild,
         std::shared_ptr<Packet> prevChild) {
     if (! prevChild) {
-        insertChildFirst(newChild);
+        prepend(newChild);
         return;
     }
 
+    if (newChild->hasParent())
+        throw InvalidArgument("The new child packet being inserted "
+            "already has a parent packet");
+    if (prevChild->treeParent_.lock().get() != this)
+        throw InvalidArgument("The insertion location is neither null "
+            "nor a child of this parent packet");
+
     fireEvent(&PacketListener::childToBeAdded, *newChild);
 
     // We could be fancy and reorder things so that we std::move() the
@@ -226,9 +241,9 @@ void Packet::reparent(const std::shared_ptr<Packet>& newParent, bool first) {
         makeOrphan();
 
     if (first)
-        newParent->insertChildFirst(me);
+        newParent->prepend(me);
     else
-        newParent->insertChildLast(me);
+        newParent->append(me);
 }
 
 void Packet::transferChildren(const std::shared_ptr<Packet>& newParent) {
@@ -607,10 +622,9 @@ std::shared_ptr<Packet> Packet::cloneAsSibling(bool cloneDescendants,
     auto ans = internalClonePacket();
     ans->setLabel(adornedLabel("Clone"));
     if (end)
-        parent->insertChildLast(ans);
+        parent->append(ans);
     else
-        parent->insertChildAfter(ans,
-            const_cast<Packet*>(this)->shared_from_this());
+        parent->insert(ans, const_cast<Packet*>(this)->shared_from_this());
     if (cloneDescendants)
         internalCloneDescendants(*ans);
     return ans;
@@ -650,7 +664,7 @@ void Packet::internalCloneDescendants(Packet& parent) const {
     for (auto child = firstTreeChild_; child; child = child->nextTreeSibling_) {
         auto clone = child->internalClonePacket();
         clone->setLabel(child->label_);
-        parent.insertChildLast(clone);
+        parent.append(clone);
         child->internalCloneDescendants(*clone);
     }
 }
