@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -89,13 +89,6 @@ enum PacketType {
      * Represents an arbitrary file attachment, of class Attachment.
      */
     PACKET_ATTACHMENT = 10,
-    /**
-     * A legacy constant that represents a PDF document.  This is stored
-     * using the Attachment class, which can now represent any kind of
-     * file attachment (not just a PDF document).  This legacy constant
-     * PACKET_PDF takes the same value as the new constant PACKET_ATTACHMENT.
-     */
-    PACKET_PDF [[deprecated]] = PACKET_ATTACHMENT,
     /**
      * Represents a normal hypersurface list, of class NormalHypersurfaces.
      */
