@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -247,20 +247,6 @@ class Signature : public ShortOutput<Signature> {
          */
         bool operator != (const Signature& other) const;
 
-        /**
-         * Deprecated routine that parses the given signature string.
-         *
-         * \deprecated This routine has been replaced with a string-based
-         * constructor.  See the constructor for further details.
-         *
-         * \exception InvalidArgument the given string was not a valid
-         * signature with a positive number of letters.
-         *
-         * @param str a string representation of a splitting surface signature.
-         * @return the corresponding signature.
-         */
-        [[deprecated]] static Signature parse(const std::string& str);
-
         /**
          * Returns the 3-manifold triangulation corresponding to
          * this splitting surface signature.
@@ -464,10 +450,6 @@ inline Signature::~Signature() {
     delete[] cycleGroupStart;
 }
 
-inline Signature Signature::parse(const std::string& str) {
-    return Signature(str);
-}
-
 inline unsigned Signature::order() const {
     return order_;
 }
