@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -119,7 +119,7 @@ class AugTriSolidTorus : public StandardTriangulation {
                  <tt>i</tt>.  For permutation <tt>p</tt>, group <tt>p[0]</tt>
                  is glued to an axis edge, group <tt>p[1]</tt> is glued to a
                  major edge and group <tt>p[2]</tt> is glued to a minor edge. */
-        unsigned long chainIndex_;
+        size_t chainIndex_;
             /**< The number of tetrahedra in the layered chain if
                  present, or 0 if there is no layered chain. */
         int chainType_;
@@ -143,15 +143,6 @@ class AugTriSolidTorus : public StandardTriangulation {
          */
         AugTriSolidTorus& operator = (const AugTriSolidTorus&) = default;
 
-        /**
-         * Deprecated routine that returns a new copy of this structure.
-         *
-         * \deprecated Just use the copy constructor instead.
-         *
-         * @return a newly created clone.
-         */
-        [[deprecated]] AugTriSolidTorus* clone() const;
-
         /**
          * Swaps the contents of this and the given structure.
          *
@@ -213,7 +204,7 @@ class AugTriSolidTorus : public StandardTriangulation {
          * @return the number of tetrahedra in the layered chain, or 0
          * if there is no layered chain linking two boundary annuli.
          */
-        unsigned long chainLength() const;
+        size_t chainLength() const;
 
         /**
          * Returns the way in which a layered chain links
@@ -325,15 +316,6 @@ class AugTriSolidTorus : public StandardTriangulation {
          */
         static std::unique_ptr<AugTriSolidTorus> recognise(
             const Component<3>* comp);
-        /**
-         * A deprecated alias to recognise if a component forms an
-         * augmented triangular solid torus.
-         *
-         * \deprecated This function has been renamed to recognise().
-         * See recognise() for details on the parameters and return value.
-         */
-        [[deprecated]] static std::unique_ptr<AugTriSolidTorus>
-            isAugTriSolidTorus(const Component<3>* comp);
 
         std::unique_ptr<Manifold> manifold() const override;
         std::ostream& writeName(std::ostream& out) const override;
@@ -378,10 +360,6 @@ inline AugTriSolidTorus::AugTriSolidTorus(const TriSolidTorus& core) :
         core_(core), chainType_(CHAIN_NONE) {
 }
 
-inline AugTriSolidTorus* AugTriSolidTorus::clone() const {
-    return new AugTriSolidTorus(*this);
-}
-
 inline void AugTriSolidTorus::swap(AugTriSolidTorus& other) noexcept {
     core_.swap(other.core_);
     augTorus_[0].swap(other.augTorus_[0]);
@@ -404,7 +382,7 @@ inline const std::optional<LayeredSolidTorus>& AugTriSolidTorus::augTorus(
 inline Perm<3> AugTriSolidTorus::edgeGroupRoles(int annulus) const {
     return edgeGroupRoles_[annulus];
 }
-inline unsigned long AugTriSolidTorus::chainLength() const {
+inline size_t AugTriSolidTorus::chainLength() const {
     return chainIndex_;
 }
 inline int AugTriSolidTorus::chainType() const {
@@ -446,11 +424,6 @@ inline void swap(AugTriSolidTorus& a, AugTriSolidTorus& b) noexcept {
     a.swap(b);
 }
 
-inline std::unique_ptr<AugTriSolidTorus> AugTriSolidTorus::isAugTriSolidTorus(
-        const Component<3>* comp) {
-    return recognise(comp);
-}
-
 } // namespace regina
 
 #endif
