@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -310,15 +310,6 @@ class BlockedSFSTriple : public StandardTriangulation {
          */
         static std::unique_ptr<BlockedSFSTriple> recognise(
             const Triangulation<3>& tri);
-        /**
-         * A deprecated alias to recognise if a triangulation forms a
-         * saturated region joined to a think I-bundle via optional layerings.
-         *
-         * \deprecated This function has been renamed to recognise().
-         * See recognise() for details on the parameters and return value.
-         */
-        [[deprecated]] static std::unique_ptr<BlockedSFSTriple>
-            isBlockedSFSTriple(const Triangulation<3>& tri);
 
     private:
         /**
@@ -398,11 +389,6 @@ inline bool BlockedSFSTriple::operator != (const BlockedSFSTriple& other)
     return ! ((*this) == other);
 }
 
-inline std::unique_ptr<BlockedSFSTriple> BlockedSFSTriple::isBlockedSFSTriple(
-        const Triangulation<3>& tri) {
-    return recognise(tri);
-}
-
 inline void swap(BlockedSFSTriple& a, BlockedSFSTriple& b) noexcept {
     a.swap(b);
 }
