@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -99,15 +99,6 @@ class LayeredChainPair : public StandardTriangulation {
          */
         LayeredChainPair& operator = (const LayeredChainPair&) = default;
 
-        /**
-         * Deprecated routine that returns a new copy of this structure.
-         *
-         * \deprecated Just use the copy constructor instead.
-         *
-         * @return a newly created clone.
-         */
-        [[deprecated]] LayeredChainPair* clone() const;
-
         /**
          * Swaps the contents of this and the given structure.
          *
@@ -183,16 +174,6 @@ class LayeredChainPair : public StandardTriangulation {
          */
         static std::unique_ptr<LayeredChainPair> recognise(
             const Component<3>* comp);
-        /**
-         * A deprecated alias to recognise if a component is a layered chain
-         * pair.
-         *
-         * \deprecated This function has been renamed to recognise().
-         * See recognise() for details on the parameters and return value.
-         */
-        [[deprecated]] static std::unique_ptr<LayeredChainPair>
-            isLayeredChainPair(const Component<3>* comp);
-
         std::unique_ptr<Manifold> manifold() const override;
         AbelianGroup homology() const override;
         std::ostream& writeName(std::ostream& out) const override;
@@ -230,10 +211,6 @@ inline void LayeredChainPair::swap(LayeredChainPair& other) noexcept {
     chain_[1].swap(other.chain_[1]);
 }
 
-inline LayeredChainPair* LayeredChainPair::clone() const {
-    return new LayeredChainPair(*this);
-}
-
 inline const LayeredChain& LayeredChainPair::chain(int which) const {
     return chain_[which];
 }
@@ -258,11 +235,6 @@ inline void LayeredChainPair::writeTextLong(std::ostream& out) const {
         << chain_[0].index() << ", " << chain_[1].index() << ')';
 }
 
-inline std::unique_ptr<LayeredChainPair> LayeredChainPair::isLayeredChainPair(
-        const Component<3>* comp) {
-    return recognise(comp);
-}
-
 inline void swap(LayeredChainPair& a, LayeredChainPair& b) noexcept {
     a.swap(b);
 }
