@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -91,15 +91,6 @@ class LayeredLensSpace : public StandardTriangulation {
          */
         LayeredLensSpace& operator = (const LayeredLensSpace&) = default;
 
-        /**
-         * Deprecated routine that returns a new copy of this structure.
-         *
-         * \deprecated Just use the copy constructor instead.
-         *
-         * @return a newly created clone.
-         */
-        [[deprecated]] LayeredLensSpace* clone() const;
-
         /**
          * Swaps the contents of this and the given structure.
          *
@@ -214,15 +205,6 @@ class LayeredLensSpace : public StandardTriangulation {
          */
         static std::unique_ptr<LayeredLensSpace> recognise(
             const Component<3>* comp);
-        /**
-         * A deprecated alias to recognise if a component forms a
-         * layered lens space.
-         *
-         * \deprecated This function has been renamed to recognise().
-         * See recognise() for details on the parameters and return value.
-         */
-        [[deprecated]] static std::unique_ptr<LayeredLensSpace>
-            isLayeredLensSpace(const Component<3>* comp);
 
         std::unique_ptr<Manifold> manifold() const override;
         AbelianGroup homology() const override;
@@ -257,10 +239,6 @@ inline LayeredLensSpace::LayeredLensSpace(const LayeredSolidTorus& torus) :
         torus_(torus) {
 }
 
-inline LayeredLensSpace* LayeredLensSpace::clone() const {
-    return new LayeredLensSpace(*this);
-}
-
 inline void LayeredLensSpace::swap(LayeredLensSpace& other) noexcept {
     torus_.swap(other.torus_);
     std::swap(mobiusBoundaryGroup_, other.mobiusBoundaryGroup_);
@@ -303,11 +281,6 @@ inline void LayeredLensSpace::writeTextLong(std::ostream& out) const {
     writeName(out);
 }
 
-inline std::unique_ptr<LayeredLensSpace> LayeredLensSpace::isLayeredLensSpace(
-        const Component<3>* comp) {
-    return recognise(comp);
-}
-
 inline void swap(LayeredLensSpace& a, LayeredLensSpace& b) noexcept {
     a.swap(b);
 }
