@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -114,15 +114,6 @@ class LayeredSolidTorus : public StandardTriangulation {
          */
         LayeredSolidTorus& operator = (const LayeredSolidTorus&) = default;
 
-        /**
-         * Deprecated routine that returns a new copy of this structure.
-         *
-         * \deprecated Just use the copy constructor instead.
-         *
-         * @return a newly created clone.
-         */
-        [[deprecated]] LayeredSolidTorus* clone() const;
-
         /**
          * Swaps the contents of this and the given structure.
          *
@@ -389,15 +380,6 @@ class LayeredSolidTorus : public StandardTriangulation {
          */
         static std::unique_ptr<LayeredSolidTorus> recogniseFromBase(
             const Tetrahedron<3>* tet);
-        /**
-         * A deprecated alias to recognise if a tetrahedron forms the
-         * base of a layered solid torus.
-         *
-         * \deprecated This function has been renamed to recogniseFromBase().
-         * See recognise() for details on the parameters and return value.
-         */
-        [[deprecated]] static std::unique_ptr<LayeredSolidTorus>
-            formsLayeredSolidTorusBase(const Tetrahedron<3>* tet);
 
         /**
          * Determines if the given tetrahedron forms the top level
@@ -432,22 +414,12 @@ class LayeredSolidTorus : public StandardTriangulation {
          */
         static std::unique_ptr<LayeredSolidTorus> recogniseFromTop(
             const Tetrahedron<3>* tet, unsigned topFace1, unsigned topFace2);
-        /**
-         * A deprecated alias to recognise if a tetrahedron forms the
-         * top level of a layered solid torus.
-         *
-         * \deprecated This function has been renamed to recogniseFromTop().
-         * See recognise() for details on the parameters and return value.
-         */
-        [[deprecated]] static std::unique_ptr<LayeredSolidTorus>
-            formsLayeredSolidTorusTop(const Tetrahedron<3>* tet,
-            unsigned topFace1, unsigned topFace2);
 
         /**
          * Determines if the given triangulation component forms a
          * layered solid torus in its entirity.
          *
-         * Note that, unlike formsLayeredSolidTorusBase(), this routine
+         * Note that, unlike recogniseFromBase(), this routine
          * tests for a component that is a layered solid torus with no
          * additional tetrahedra or gluings.  That is, the two boundary
          * triangles of the layered solid torus must in fact be boundary
@@ -463,15 +435,6 @@ class LayeredSolidTorus : public StandardTriangulation {
          */
         static std::unique_ptr<LayeredSolidTorus> recognise(
             Component<3>* comp);
-        /**
-         * A deprecated alias to recognise if a component forms a
-         * layered solid torus.
-         *
-         * \deprecated This function has been renamed to recognise().
-         * See recognise() for details on the parameters and return value.
-         */
-        [[deprecated]] static std::unique_ptr<LayeredSolidTorus>
-            isLayeredSolidTorus(Component<3>* comp);
 
         std::unique_ptr<Manifold> manifold() const override;
         AbelianGroup homology() const override;
@@ -501,10 +464,6 @@ void swap(LayeredSolidTorus& a, LayeredSolidTorus& b) noexcept;
 
 // Inline functions for LayeredSolidTorus
 
-inline LayeredSolidTorus* LayeredSolidTorus::clone() const {
-    return new LayeredSolidTorus(*this);
-}
-
 inline void LayeredSolidTorus::swap(LayeredSolidTorus& other) noexcept {
     std::swap(size_, other.size_);
     std::swap(base_, other.base_);
@@ -573,23 +532,6 @@ inline std::ostream& LayeredSolidTorus::writeTeXName(std::ostream& out) const {
         << meridinalCuts_[1] << ',' << meridinalCuts_[2] << ')';
 }
 
-inline std::unique_ptr<LayeredSolidTorus>
-        LayeredSolidTorus::formsLayeredSolidTorusBase(
-        const Tetrahedron<3>* tet) {
-    return recogniseFromBase(tet);
-}
-
-inline std::unique_ptr<LayeredSolidTorus>
-        LayeredSolidTorus::formsLayeredSolidTorusTop(
-        const Tetrahedron<3>* tet, unsigned topFace1, unsigned topFace2) {
-    return recogniseFromTop(tet, topFace1, topFace2);
-}
-
-inline std::unique_ptr<LayeredSolidTorus>
-        LayeredSolidTorus::isLayeredSolidTorus(Component<3>* comp) {
-    return recognise(comp);
-}
-
 inline void swap(LayeredSolidTorus& a, LayeredSolidTorus& b) noexcept {
     a.swap(b);
 }
