@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -92,15 +92,6 @@ class PillowTwoSphere : public ShortOutput<PillowTwoSphere> {
          */
         PillowTwoSphere& operator = (const PillowTwoSphere&) = default;
 
-        /**
-         * Deprecated routine that returns a new copy of this structure.
-         *
-         * \deprecated Just use the copy constructor instead.
-         *
-         * @return a newly created clone.
-         */
-        [[deprecated]] PillowTwoSphere* clone() const;
-
         /**
          * Returns one of the two triangles whose boundaries are joined.
          *
@@ -179,16 +170,6 @@ class PillowTwoSphere : public ShortOutput<PillowTwoSphere> {
         static std::unique_ptr<PillowTwoSphere> recognise(
             Triangle<3>* tri1, Triangle<3>* tri2);
 
-        /**
-         * A deprecated alias to recognise if two triangles together
-         * form a pillow 2-sphere.
-         *
-         * \deprecated This function has been renamed to recognise().
-         * See recognise() for details on the parameters and return value.
-         */
-        [[deprecated]] static std::unique_ptr<PillowTwoSphere>
-            formsPillowTwoSphere(Triangle<3>* tri1, Triangle<3>* tri2);
-
         /**
          * Writes a short text representation of this object to the
          * given output stream.
@@ -213,9 +194,6 @@ inline PillowTwoSphere::PillowTwoSphere(Triangle<3>* tri0, Triangle<3>* tri1,
         Perm<4> triMapping) :
         triangle_ { tri0, tri1 }, triMapping_(triMapping) {
 }
-inline PillowTwoSphere* PillowTwoSphere::clone() const {
-    return new PillowTwoSphere(*this);
-}
 inline Triangle<3>* PillowTwoSphere::triangle(int index) const {
     return triangle_[index];
 }
@@ -236,11 +214,6 @@ inline bool PillowTwoSphere::operator != (const PillowTwoSphere& other) const {
         triMapping_ != other.triMapping_;
 }
 
-inline std::unique_ptr<PillowTwoSphere> PillowTwoSphere::formsPillowTwoSphere(
-        Triangle<3>* tri1, Triangle<3>* tri2) {
-    return recognise(tri1, tri2);
-}
-
 } // namespace regina
 
 #endif
