@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -149,15 +149,6 @@ class PlugTriSolidTorus : public StandardTriangulation {
          */
         PlugTriSolidTorus& operator = (const PlugTriSolidTorus&) = default;
 
-        /**
-         * Deprecated routine that returns a new copy of this structure.
-         *
-         * \deprecated Just use the copy constructor instead.
-         *
-         * @return a newly created clone.
-         */
-        [[deprecated]] PlugTriSolidTorus* clone() const;
-
         /**
          * Swaps the contents of this and the given structure.
          *
@@ -278,15 +269,6 @@ class PlugTriSolidTorus : public StandardTriangulation {
          * triangular solid torus.
          */
         static std::unique_ptr<PlugTriSolidTorus> recognise(Component<3>* comp);
-        /**
-         * A deprecated alias to recognise if a component forms a
-         * plugged triangular solid torus.
-         *
-         * \deprecated This function has been renamed to recognise().
-         * See recognise() for details on the parameters and return value.
-         */
-        [[deprecated]] static std::unique_ptr<PlugTriSolidTorus>
-            isPlugTriSolidTorus(Component<3>* comp);
 
         std::unique_ptr<Manifold> manifold() const override;
         std::ostream& writeName(std::ostream& out) const override;
@@ -322,10 +304,6 @@ inline PlugTriSolidTorus::PlugTriSolidTorus(const TriSolidTorus& core) :
         core_(core), chainType_ { CHAIN_NONE, CHAIN_NONE, CHAIN_NONE } {
 }
 
-inline PlugTriSolidTorus* PlugTriSolidTorus::clone() const {
-    return new PlugTriSolidTorus(*this);
-}
-
 inline void PlugTriSolidTorus::swap(PlugTriSolidTorus& other) noexcept {
     core_.swap(other.core_);
     chain_[0].swap(other.chain_[0]);
@@ -365,11 +343,6 @@ inline void swap(PlugTriSolidTorus& a, PlugTriSolidTorus& b) noexcept {
     a.swap(b);
 }
 
-inline std::unique_ptr<PlugTriSolidTorus>
-        PlugTriSolidTorus::isPlugTriSolidTorus(Component<3>* comp) {
-    return recognise(comp);
-}
-
 } // namespace regina
 
 #endif
