@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -205,7 +205,7 @@ struct SatAnnulus {
      * @return the number of triangles of this annulus that lie on the boundary
      * of the triangulation; this will be 0, 1 or 2.
      */
-    unsigned meetsBoundary() const;
+    int meetsBoundary() const;
 
     /**
      * Converts this into a representation of the same annulus from the
