@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -383,15 +383,6 @@ class SatRegion : public Output<SatRegion> {
          * @return the total number of blocks.
          */
         size_t countBlocks() const;
-        /**
-         * Deprecated routine that returns the number of saturated blocks
-         * in this saturated region.
-         *
-         * \deprecated This routine has been renamed countBlocks().
-         *
-         * @return the total number of blocks.
-         */
-        [[deprecated]] size_t numberOfBlocks() const;
         /**
          * Returns details of the requested saturated block within this
          * region.  The information will returned will include
@@ -427,15 +418,6 @@ class SatRegion : public Output<SatRegion> {
          * @return the number of boundary annuli.
          */
         size_t countBoundaryAnnuli() const;
-        /**
-         * Deprecated routine that returns the number of saturated annuli
-         * in the boundary of this region.
-         *
-         * \deprecated This routine has been renamed countBoundaryAnnuli().
-         *
-         * @return the total number of boundary annuli.
-         */
-        [[deprecated]] size_t numberOfBoundaryAnnuli() const;
         /**
          * Returns details of the requested saturated annulus on
          * the boundary of this region.
@@ -489,7 +471,7 @@ class SatRegion : public Output<SatRegion> {
          * requested annulus is horizontally reflected within this region.
          * See SatBlockSpec for further details on these reflections.
          */
-        std::tuple<const SatBlock*, unsigned, bool, bool> boundaryAnnulus(
+        std::tuple<const SatBlock*, size_t, bool, bool> boundaryAnnulus(
             size_t which) const;
 
         /**
@@ -817,7 +799,7 @@ class SatRegion : public Output<SatRegion> {
          * @param twisted returns the number of twisted (Klein bottle)
          * boundary components.
          */
-        void countBoundaries(unsigned& untwisted, unsigned& twisted) const;
+        void countBoundaries(size_t& untwisted, size_t& twisted) const;
 
         /**
          * Expands this region as far as possible within the overall
@@ -998,10 +980,6 @@ inline size_t SatRegion::countBlocks() const {
     return blocks_.size();
 }
 
-inline size_t SatRegion::numberOfBlocks() const {
-    return blocks_.size();
-}
-
 inline const SatBlockSpec& SatRegion::block(size_t which) const {
     return blocks_[which];
 }
@@ -1010,10 +988,6 @@ inline size_t SatRegion::countBoundaryAnnuli() const {
     return nBdryAnnuli_;
 }
 
-inline size_t SatRegion::numberOfBoundaryAnnuli() const {
-    return nBdryAnnuli_;
-}
-
 inline bool SatRegion::operator != (const SatRegion& other) const {
     return ! ((*this) == other);
 }
