@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -82,15 +82,6 @@ class SnappedBall : public StandardTriangulation {
          */
         SnappedBall& operator = (const SnappedBall&) = default;
 
-        /**
-         * Deprecated routine that returns a new copy of this structure.
-         *
-         * \deprecated Just use the copy constructor instead.
-         *
-         * @return a newly created clone.
-         */
-        [[deprecated]] SnappedBall* clone() const;
-
         /**
          * Swaps the contents of this and the given structure.
          *
@@ -210,16 +201,6 @@ class SnappedBall : public StandardTriangulation {
          */
         static std::unique_ptr<SnappedBall> recognise(Tetrahedron<3>* tet);
 
-        /**
-         * A deprecated alias to recognise if a tetrahedron forms a
-         * snapped 3-ball.
-         *
-         * \deprecated This function has been renamed to recognise().
-         * See recognise() for details on the parameters and return value.
-         */
-        [[deprecated]] static std::unique_ptr<SnappedBall> formsSnappedBall(
-            Tetrahedron<3>* tet);
-
         std::unique_ptr<Manifold> manifold() const override;
         AbelianGroup homology() const override;
         std::ostream& writeName(std::ostream& out) const override;
@@ -252,10 +233,6 @@ inline SnappedBall::SnappedBall(Tetrahedron<3>* tet, int equator) :
         tet_(tet), equator_(equator) {
 }
 
-inline SnappedBall* SnappedBall::clone() const {
-    return new SnappedBall(*this);
-}
-
 inline void SnappedBall::swap(SnappedBall& other) noexcept {
     std::swap(tet_, other.tet_);
     std::swap(equator_, other.equator_);
@@ -301,11 +278,6 @@ inline void SnappedBall::writeTextShort(std::ostream& out) const {
         << Edge<3>::edgeVertex[5 - equator_][1] << ')';
 }
 
-inline std::unique_ptr<SnappedBall> SnappedBall::formsSnappedBall(
-        Tetrahedron<3>* tet) {
-    return recognise(tet);
-}
-
 inline void swap(SnappedBall& a, SnappedBall& b) noexcept {
     a.swap(b);
 }
