@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -84,14 +84,14 @@ class TxICore : public Output<TxICore> {
         Triangulation<3> core_;
             /**< A full copy of the <tt>T x I</tt> triangulation that is
                  described. */
-        unsigned bdryTet_[2][2];
+        std::array<std::array<size_t, 2>, 2> bdryTet_;
             /**< The tetrahedra that provide the upper and lower
                  boundary triangles.  See bdryTet() for details. */
-        Perm<4> bdryRoles_[2][2];
+        std::array<std::array<Perm<4>, 2>, 2> bdryRoles_;
             /**< Describes which tetrahedron vertices play which roles
                  in the upper and lower boundary triangles.  See bdryRoles()
                  for details. */
-        Matrix2 bdryReln_[2];
+        std::array<Matrix2, 2> bdryReln_;
             /**< Expresses the \a alpha and \a beta curves for each
                  torus boundary in terms of specific tetrahedron edges and
                  vertices.  The elements \a bdryReln_[0] and \a bdryReln_[1]
@@ -140,7 +140,7 @@ class TxICore : public Output<TxICore> {
          * @param whichTri 0 if the first boundary triangle should be
          * examined, or 1 if the second boundary triangle should be examined.
          */
-        unsigned bdryTet(unsigned whichBdry, unsigned whichTri) const;
+        size_t bdryTet(int whichBdry, int whichTri) const;
         /**
          * Describes which tetrahedron vertices play which roles in the
          * upper and lower boundary triangles.
@@ -183,7 +183,7 @@ class TxICore : public Output<TxICore> {
          * @return the permutation mapping roles 0, 1 and 2 in the
          * diagram above to real tetrahedron vertex numbers.
          */
-        Perm<4> bdryRoles(unsigned whichBdry, unsigned whichTri) const;
+        Perm<4> bdryRoles(int whichBdry, int whichTri) const;
         /**
          * Returns a 2-by-2 matrix describing the \a alpha and \a beta curves
          * on a torus boundary in terms of specific tetrahedron edges.
@@ -212,7 +212,7 @@ class TxICore : public Output<TxICore> {
          * @return the relationship between the boundary curves and
          * tetrahedron edges.
          */
-        const Matrix2& bdryReln(unsigned whichBdry) const;
+        const Matrix2& bdryReln(int whichBdry) const;
         /**
          * Returns a 2-by-2 matrix describing the parallel relationship
          * between the upper and lower boundary curves.
@@ -257,15 +257,6 @@ class TxICore : public Output<TxICore> {
          * @return the name of this triangulation in TeX format.
          */
         std::string texName() const;
-        /**
-         * Deprecated routine that returns the name of this specific
-         * triangulation of <tt>T x I</tt> in TeX format.
-         *
-         * \deprecated This routine has been renamed to texName().
-         *
-         * @return the name of this triangulation in TeX format.
-         */
-        [[deprecated]] std::string TeXName() const;
 
         /**
          * Determines if this and the given <tt>T x I</tt> triangulation
@@ -352,7 +343,7 @@ class TxICore : public Output<TxICore> {
         /**
          * Move constructor.
          */
-        TxICore(TxICore&& src) = default;
+        TxICore(TxICore&& src) noexcept = default;
         /**
          * Copy assignment operator.
          *
@@ -372,7 +363,7 @@ class TxICore : public Output<TxICore> {
          * packet event listeners, and the internal triangulation here does
          * not belong to a packet.
          */
-        TxICore& operator = (TxICore&&) = default;
+        TxICore& operator = (TxICore&&) noexcept;
         /**
          * Swaps all data that is managed by this base class with the
          * given triangulation.
@@ -456,10 +447,10 @@ class TxICore : public Output<TxICore> {
  */
 class TxIDiagonalCore : public TxICore {
     private:
-        unsigned long size_;
+        size_t size_;
             /**< The number of tetrahedra in this <tt>T x I</tt>
                  triangulation. */
-        unsigned long k_;
+        size_t k_;
             /**< The additional parameter \a k as described in the
                  class notes. */
 
@@ -468,13 +459,13 @@ class TxIDiagonalCore : public TxICore {
          * Creates a new <tt>T x I</tt> triangulation with the given
          * parameters.
          *
-         * @param newSize the number of tetrahedra in this
+         * @param size the number of tetrahedra in this
          * triangulation.  This must be at least 6.
-         * @param newK the additional parameter \a k as described in the
+         * @param k the additional parameter \a k as described in the
          * class notes.  This must be between 1 and (\a newSize - 5)
          * inclusive.
          */
-        TxIDiagonalCore(unsigned long newSize, unsigned long newK);
+        TxIDiagonalCore(size_t size, size_t k);
 
         /**
          * Creates a new copy of the given <tt>T x I</tt> triangulation.
@@ -513,7 +504,7 @@ class TxIDiagonalCore : public TxICore {
          *
          * @return the total number of tetrahedra.
          */
-        unsigned long size() const;
+        size_t size() const;
 
         /**
          * Returns the additional parameter \a k as described in the
@@ -521,7 +512,7 @@ class TxIDiagonalCore : public TxICore {
          *
          * @return the additional parameter \a k.
          */
-        unsigned long k() const;
+        size_t k() const;
 
         /**
          * Swaps the contents of this and the given
@@ -663,17 +654,15 @@ inline const Triangulation<3>& TxICore::core() const {
     return core_;
 }
 
-inline unsigned TxICore::bdryTet(unsigned whichBdry, unsigned whichTri)
-        const {
+inline size_t TxICore::bdryTet(int whichBdry, int whichTri) const {
     return bdryTet_[whichBdry][whichTri];
 }
 
-inline Perm<4> TxICore::bdryRoles(unsigned whichBdry, unsigned whichTri)
-        const {
+inline Perm<4> TxICore::bdryRoles(int whichBdry, int whichTri) const {
     return bdryRoles_[whichBdry][whichTri];
 }
 
-inline const Matrix2& TxICore::bdryReln(unsigned whichBdry) const {
+inline const Matrix2& TxICore::bdryReln(int whichBdry) const {
     return bdryReln_[whichBdry];
 }
 
@@ -685,10 +674,6 @@ inline bool TxICore::operator != (const TxICore& other) const {
     return ! ((*this) == other);
 }
 
-inline std::string TxICore::TeXName() const {
-    return texName();
-}
-
 inline void TxICore::writeTextShort(std::ostream& out) const {
     writeName(out);
 }
@@ -699,6 +684,29 @@ inline void TxICore::writeTextLong(std::ostream& out) const {
     out << std::endl;
 }
 
+inline TxICore& TxICore::operator = (TxICore&& src) noexcept {
+    // This is essentially what the default implementation would do; however,
+    // we need to reimplement it ourselves to work around the fact that this
+    // assignment operator is noexcept but the Triangulation<3> assignment
+    // operator is not.  Just using the default operator causes compile errors
+    // on old versions of clang (e.g., clang 7), and causes move assignment to
+    // fall back to a deep copy on old versions of gcc (e.g., gcc 7-9).
+    // (Both newer clang and newer gcc behave fine.)
+
+    // The core triangulation can be moved.
+    // Although Triangulation's move assignment can throw in general,
+    // in this setting it never will (since there is no surrounding packet).
+    core_ = std::move(src.core_);
+
+    // Everything else is so small that it can only be copied.
+    bdryTet_ = src.bdryTet_;
+    bdryRoles_ = src.bdryRoles_;
+    bdryReln_ = src.bdryReln_;
+    parallelReln_ = src.parallelReln_;
+
+    return *this;
+}
+
 // Inline functions for TxIDiagonalCore
 
 inline void TxIDiagonalCore::swap(TxIDiagonalCore& other) noexcept {
@@ -707,11 +715,11 @@ inline void TxIDiagonalCore::swap(TxIDiagonalCore& other) noexcept {
     std::swap(k_, other.k_);
 }
 
-inline unsigned long TxIDiagonalCore::size() const {
+inline size_t TxIDiagonalCore::size() const {
     return size_;
 }
 
-inline unsigned long TxIDiagonalCore::k() const {
+inline size_t TxIDiagonalCore::k() const {
     return k_;
 }
 
