@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -408,12 +408,6 @@ class DiscSetTetData : public DiscSetTet {
          * The type of data being stored alongside each normal disc.
          */
         using Data = T;
-        /**
-         * A pointer to the type of data stored alongside each normal disc.
-         *
-         * \deprecated Simply use <tt>Data*</tt> instead.
-         */
-        using DataPtr [[deprecated]] = T*;
 
     protected:
         T* data_[10];
