@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -37,6 +37,7 @@
 #include "enumerate/hilbertdual.h"
 #include "enumerate/hilbertprimal.h"
 #include "enumerate/treetraversal.h"
+#include "libnormaliz/cone.h"
 #include "maths/matrixops.h"
 #include "maths/matrix.h"
 #include "progress/progresstracker.h"
@@ -108,45 +109,6 @@ namespace {
     }
 }
 
-std::shared_ptr<PacketOf<NormalSurfaces>> NormalSurfaces::enumerate(
-        Triangulation<3>& owner, NormalCoords coords, NormalList which,
-        NormalAlg algHints, ProgressTracker* tracker) {
-    // Like the constructor, but (1) we have tree insertion; and (2) we
-    // need to convert exceptions to null returns.
-    MatrixInt eqns;
-    try {
-        eqns = makeMatchingEquations(owner, coords);
-    } catch (const ReginaException&) {
-        if (tracker)
-            tracker->setFinished();
-        return nullptr;
-    }
-
-    auto ans = make_packet<NormalSurfaces>(std::in_place, coords, which,
-        algHints, owner);
-    auto treeParent = owner.inAnyPacket();
-
-    if (tracker) {
-        // We pass the matching equations as an argument to the thread
-        // function so we can be sure that the equations are moved into
-        // the thread before they are destroyed.
-        //
-        // Likewise, passing the shared pointers ans and treeParent into the
-        // thread ensures that they survive for the lifetime of the thread.
-        std::thread([tracker](
-                // NOLINTNEXTLINE(performance-unnecessary-value-param)
-                MatrixInt e,
-                // NOLINTNEXTLINE(performance-unnecessary-value-param)
-                std::shared_ptr<NormalSurfaces> s,
-                // NOLINTNEXTLINE(performance-unnecessary-value-param)
-                std::shared_ptr<Packet> p) {
-            Enumerator(s.get(), e, tracker, p.get()).enumerate();
-        }, std::move(eqns), ans, std::move(treeParent)).detach();
-    } else
-        Enumerator(ans.get(), eqns, tracker, treeParent.get()).enumerate();
-    return ans;
-}
-
 void NormalSurfaces::Enumerator::enumerate() {
     // Clean up the "type of list" flag.
     list_->which_ &= (
@@ -163,8 +125,7 @@ void NormalSurfaces::Enumerator::enumerate() {
 
     // Insert the results into the packet tree, but only once they are ready.
     if (treeParent_ && ! (tracker_ && tracker_->isCancelled()))
-        treeParent_->insertChildLast(
-            static_cast<PacketOf<NormalSurfaces>*>(list_)->
+        treeParent_->append(static_cast<PacketOf<NormalSurfaces>*>(list_)->
             shared_from_this());
 
     if (tracker_)
@@ -347,7 +308,7 @@ void NormalSurfaces::Enumerator::fillVertexTree() {
     // The maximum number of columns in the tableaux that could be added
     // to form the right hand side, as a consequence of either
     // LPData::constrainPositive() or LPData::constrainOct():
-    unsigned long maxColsRHS;
+    size_t maxColsRHS;
 
     switch (list_->coords_) {
         case NS_STANDARD:
@@ -381,7 +342,7 @@ void NormalSurfaces::Enumerator::fillVertexTree() {
     Integer tmp;
 
     // The rank of the matching equation matrix:
-    unsigned long rank = rowBasis(eqns);
+    size_t rank = rowBasis(eqns);
 
     // The maximum entry in the matching equation matrix:
     Integer maxEqnEntry = 0;
@@ -653,15 +614,15 @@ void NormalSurfaces::Enumerator::fillFundamentalFullCone() {
     // This is okay, since fillFundamentalFullCone() is only used as a
     // top-level enumeration routine (and is never used at all unless
     // the user explicitly chooses this algorithm).
-    unsigned rank = rowBasis(const_cast<MatrixInt&>(eqns_));
+    size_t rank = rowBasis(const_cast<MatrixInt&>(eqns_));
     size_t dim = eqns_.columns();
 
     std::vector<std::vector<mpz_class>> input;
     input.reserve(rank);
-    for (unsigned r = 0; r < rank; ++r) {
+    for (size_t r = 0; r < rank; ++r) {
         std::vector<mpz_class> v;
         v.reserve(eqns_.columns());
-        for (unsigned c = 0; c < eqns_.columns(); ++c) {
+        for (size_t c = 0; c < eqns_.columns(); ++c) {
             const Integer& entry(eqns_.entry(r, c));
             if (entry.isNative())
                 v.emplace_back(entry.longValue());
