@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -318,6 +318,14 @@ class NormalSurface : public ShortOutput<NormalSurface> {
             /**< Is this surface compact (i.e. does it only contain
                  finitely many discs)?
                  This is std::nullopt if it has not yet been computed. */
+        mutable uint8_t linkOf_ { 0 };
+            /**< Indicates which dimensions of face a positive rational multiple
+                 of this surface is a thin or normalised link of.  This is
+                 treated as a bitmask: for each i=0,1,2, the (2i+1)th bit
+                 indicates whether this surface scales to the link of an
+                 i-face, and the (2i)th bit indicates whether this information
+                 has actually been computed yet; if it has not been computed,
+                 then the (2i+1)th bit will be zero. */
 
     public:
         /**
@@ -635,19 +643,6 @@ class NormalSurface : public ShortOutput<NormalSurface> {
         NormalSurface(const SnapshotRef<Triangulation<3>>& triang,
             NormalCoords coords, Vector<LargeInteger>&& vector);
 
-        /**
-         * Deprecated routine that creates a newly allocated clone of this
-         * normal surface.
-         *
-         * The name of the normal surface will \e not be copied to the
-         * clone; instead the clone will have an empty name.
-         *
-         * \deprecated Simply use the copy constructor instead.
-         *
-         * @return a clone of this normal surface.
-         */
-        [[deprecated]] NormalSurface* clone() const;
-
         /**
          * Sets this to be a copy of the given normal surface.
          *
@@ -695,11 +690,15 @@ class NormalSurface : public ShortOutput<NormalSurface> {
         void swap(NormalSurface& other) noexcept;
 
         /**
-         * Returns the double of this surface.
+         * Deprecated routine that returns the double of this surface.
+         *
+         * \deprecated Normal surfaces can now be multiplied by integer
+         * constants.  In particular, this routine has exactly the same
+         * effect as multiplying the surface by 2.
          *
          * @return the double of this normal surface.
          */
-        NormalSurface doubleSurface() const;
+        [[deprecated]] NormalSurface doubleSurface() const;
 
         /**
          * Returns the sum of this and the given surface.  This will combine
@@ -719,6 +718,46 @@ class NormalSurface : public ShortOutput<NormalSurface> {
          */
         NormalSurface operator + (const NormalSurface& rhs) const;
 
+        /**
+         * Returns the given integer multiple of this surface.
+         *
+         * The resulting surface will use the same internal vector encoding
+         * as this surface.
+         *
+         * @param coeff the coefficient to multiply this surface by;
+         * this must be non-negative.
+         * @return the resulting multiple of this surface.
+         */
+        NormalSurface operator * (const LargeInteger& coeff) const;
+
+        /**
+         * Converts this surface into the given integer multiple of itself.
+         *
+         * The internal vector encoding used by this surface will not change.
+         *
+         * @param coeff the coefficient to multiply this surface by;
+         * this must be non-negative.
+         * @return a reference to this surface.
+         */
+        NormalSurface& operator *= (const LargeInteger& coeff);
+
+        /**
+         * Converts this surface into its smallest positive rational multiple
+         * with integer coordinates.
+         *
+         * Note that the scaling factor will be independent of which
+         * internal vector encoding is used.  This is essentially because
+         * integer quad coordinates (which are stored in every encoding)
+         * and integer octagon coordinates (which are stored in every
+         * almost normal encoding) are enough to guarantee integer triangle
+         * coordinates (which might or might not be stored).
+         *
+         * @return the integer by which the original surface was divided
+         * (i.e., the gcd of all normal coordinates in the original surface).
+         * This will always be strictly positive.
+         */
+        LargeInteger scaleDown();
+
         /**
          * Returns the number of triangular discs of the given type in
          * this normal surface.
@@ -882,21 +921,6 @@ class NormalSurface : public ShortOutput<NormalSurface> {
          * @param out the output stream to which to write.
          */
         void writeTextShort(std::ostream& out) const;
-        /**
-         * Deprecated routine that writes the underlying coordinate vector
-         * to the given output stream in text format.
-         * No indication will be given as to which coordinate
-         * system is being used or what each coordinate means.
-         * No newline will be written.
-         *
-         * \deprecated Just write vector() directly to the output stream.
-         *
-         * \ifacespython Not present; instead just write vector() to the
-         * appropriate output stream.
-         *
-         * @param out the output stream to which to write.
-         */
-        [[deprecated]] void writeRawVector(std::ostream& out) const;
 
         /**
          * Writes a chunk of XML containing this normal surface and all
@@ -1050,52 +1074,198 @@ class NormalSurface : public ShortOutput<NormalSurface> {
          * Determines whether or not this surface is vertex linking.
          * A <i>vertex linking</i> surface contains only triangles.
          *
+         * This behaves differently from isVertexLink(), which only detects
+         * the link of a single vertex (or a multiple of such a link).
+         * In contrast, this routine will also detect the union of
+         * several \e different vertex links.
+         *
          * Note that the results of this routine are not cached.
          * Thus the results will be reevaluated every time this routine is
          * called.
          *
-         * \todo \opt Cache results.
-         *
          * @return \c true if and only if this surface is vertex linking.
          */
         bool isVertexLinking() const;
         /**
-         * Determines whether or not a rational multiple of this surface
-         * is the link of a single vertex.
+         * Determines whether or not a positive rational multiple of this
+         * surface is the link of a single vertex.
+         *
+         * This behaves differently from isVertexLinking(), which will also
+         * detect a union of several different vertex links.  In contrast,
+         * this routine will only identify the link of a \e single vertex
+         * (or a multiple of such a link).
          *
          * Note that the results of this routine are not cached.
          * Thus the results will be reevaluated every time this routine is
          * called.
          *
-         * \todo \opt Cache results.
-         *
-         * @return the vertex linked by this surface, or \c null if this
-         * surface is not the link of a single vertex.
+         * @return the vertex linked by a positive rational multiple of this
+         * surface,
+         * or \c null if this surface is not a multiple of a single vertex link.
          */
         const Vertex<3>* isVertexLink() const;
         /**
-         * Determines whether or not a rational multiple of this surface
-         * is the thin link of a single edge.
+         * Determines whether or not a positive rational multiple of this
+         * surface is the thin link of a single edge.
          *
-         * If there are two different edges <i>e1</i> and <i>e2</i> for
-         * which this surface could be expressed as the thin link of
-         * either <i>e1</i> or <i>e2</i>, the pair
-         * (<i>e1</i>, <i>e2</i>) will be returned.
-         * If this surface is the thin link of only one edge <i>e</i>,
-         * the pair (<i>e</i>, \c null) will be returned.
-         * If this surface is not the thin link of any edges, the pair
-         * (\c null, \c null) will be returned.
+         * Here a \e thin edge link is a normal surface which appears naturally
+         * as the frontier of a regular neighbourhood of an edge, with no need
+         * for any further normalisation.
+         *
+         * This behaves differently from isNormalEdgeLink(), which tests for a
+         * \e normalised edge link (which could end up far away from the
+         * edge, or could be normalised into a surface with different
+         * topology, or could even be normalised away to nothing).
+         * Although isNormalEdgeLink() will also indicate thin edge links,
+         * this test has significantly less overhead (and so should be faster).
+         *
+         * A surface (or its positive rational multiple) can be the \e thin edge
+         * link of at most two edges.  If there are indeed two different edges
+         * \a e1 and \a e2 for which a multiple of this surface can be expressed
+         * as the thin edge link, then the pair (\a e1, \a e2) will be returned.
+         * If there is only one such edge \a e, then the pair (\a e, \c null)
+         * will be returned.  If no positive rational multiple of this surface
+         * is the thin link of any edge, then the pair (\c null, \c null) will
+         * be returned.
          *
          * Note that the results of this routine are not cached.
          * Thus the results will be reevaluated every time this routine is
          * called.
          *
-         * \todo \opt Cache results.
-         *
-         * @return a pair containing the edge(s) linked by this surface,
-         * as described above.
+         * @return a pair containing the edge(s) linked by a positive rational
+         * multiple of this surface, as described above.
          */
         std::pair<const Edge<3>*, const Edge<3>*> isThinEdgeLink() const;
+        /**
+         * Determines whether or not a positive rational multiple of this
+         * surface is the normalised link of a single edge.
+         *
+         * Here the phrase \e normalised link of an edge \a e means the
+         * frontier of a regular neighbourhood of \a e, converted into a
+         * normal surface by expanding away from the edge using the
+         * normalisation process.  It could be that there is no normalisation
+         * required at all (in which case it is also a \e thin edge link).
+         * However, it could be that the normalisation process expands
+         * the surface far away from the edge itself, or changes its
+         * topology, or disconnects the surface, or even normalises it
+         * away to an empty surface.
+         *
+         * In particular, this test behaves differently from isThinEdgeLink(),
+         * which tests for thin edge links only (where no additional
+         * normalisation is required).  If you are only interested in thin
+         * edge links, then you should call isThinEdgeLink(), which has much
+         * less overhead.
+         *
+         * A surface (or its positive rational multiple) could be the
+         * normalised link of many edges.  The return value will be a pair
+         * (\a v, \a thin), where:
+         *
+         * - \a v is a vector containing all such edges.  This will begin
+         *   with the edges for which this surface is a thin link, followed by
+         *   the edges where normalisation was required; within each category
+         *   the edges will be ordered by their index within the triangulation.
+         *
+         * - \a thin is either 0, 1 or 2, indicating how many edges this
+         *   surface is a thin link for.  This uses an unsigned type, since
+         *   it will often be compared to <tt>v.size()</tt>.
+         *
+         * If no positive rational multiple of this surface is the normalised
+         * link of any edge, then \a link will be 0 and \a v will be the
+         * empty vector.
+         *
+         * Note that the results of this routine are not cached.
+         * Thus the results will be reevaluated every time this routine is
+         * called.
+         *
+         * @return a vector containing the edge(s) linked by a positive rational
+         * multiple of this surface and an integer indicating how many
+         * of these links are thin, as described above.
+         */
+        std::pair<std::vector<const Edge<3>*>, unsigned> isNormalEdgeLink()
+            const;
+        /**
+         * Determines whether or not a positive rational multiple of this
+         * surface is the thin link of a single triangle.
+         *
+         * Here a \e thin triangle link is a normal surface which appears
+         * naturally as the frontier of a regular neighbourhood of a
+         * triangle, with no need for any further normalisation.
+         *
+         * This behaves differently from isNormalTriangleLink(), which tests
+         * for a \e normalised triangle link (which could end up far away from
+         * the triangle, or could be normalised into a surface with different
+         * topology, or could even be normalised away to nothing).  Unlike the
+         * tests for edge links, the routines isThinTriangleLink() and
+         * isNormalTriangleLink() use essentially the same implementation (so
+         * testing for only thin links may be a little faster, but not by much).
+         *
+         * A surface (or its positive rational multiple) can be the \e thin link
+         * of at most two triangles.  If there are indeed two different
+         * triangles \a t1 and \a t2 for which a multiple of this surface can
+         * be expressed as the thin triangle link, then the pair (\a t1, \a t2)
+         * will be returned.  If there is only one such triangle \a t, then the
+         * pair (\a t, \c null) will be returned.  If no positive rational
+         * multiple of this surface is the thin link of any triangle, then the
+         * pair (\c null, \c null) will be returned.
+         *
+         * Note that the results of this routine are not cached.
+         * Thus the results will be reevaluated every time this routine is
+         * called.
+         *
+         * @return a pair containing the triangle(s) linked by a positive
+         * rational multiple of this surface, as described above.
+         */
+        std::pair<const Triangle<3>*, const Triangle<3>*> isThinTriangleLink()
+            const;
+        /**
+         * Determines whether or not a positive rational multiple of this
+         * surface is the normalised link of a single triangle.
+         *
+         * Here the phrase \e normalised link of a triangle \a t means the
+         * frontier of a regular neighbourhood of \a t, converted into a
+         * normal surface by expanding away from the triangle using the
+         * normalisation process.  It could be that there is no normalisation
+         * required at all (in which case it is also a \e thin triangle link).
+         * However, it could be that the normalisation process expands
+         * the surface far away from the triangle itself, or changes its
+         * topology, or disconnects the surface, or even normalises it
+         * away to an empty surface.
+         *
+         * In particular, this test behaves differently from
+         * isThinTriangleLink(), which tests for thin triangle links only
+         * (where no additional normalisation is required).  Unlike the
+         * tests for edge links, the routines isThinTriangleLink() and
+         * isNormalTriangleLink() use essentially the same implementation (so
+         * testing for only thin links may be a little faster, but not by much).
+         *
+         * A surface (or its positive rational multiple) could be the
+         * normalised link of many triangles.  The return value will be a pair
+         * (\a v, \a thin), where:
+         *
+         * - \a v is a vector containing all such triangles.  This will begin
+         *   with the triangles for which this surface is a thin link, followed
+         *   by the triangles where normalisation was required; within each
+         *   category the triangles will be ordered by their index within the
+         *   triangulation.
+         *
+         * - \a thin is either 0, 1 or 2, indicating how many triangles this
+         *   surface is a thin link for.  This uses an unsigned type, since
+         *   it will often be compared to <tt>v.size()</tt>.
+         *
+         * If no positive rational multiple of this surface is the normalised
+         * link of any triangle, then \a link will be 0 and \a v will be the
+         * empty vector.
+         *
+         * Note that the results of this routine are not cached.
+         * Thus the results will be reevaluated every time this routine is
+         * called.
+         *
+         * @return a vector containing the triangle(s) linked by a positive
+         * rational multiple of this surface and an integer indicating how many
+         * of these links are thin, as described above.
+         */
+        std::pair<std::vector<const Triangle<3>*>, unsigned>
+            isNormalTriangleLink() const;
         /**
          * Determines whether or not this surface is a splitting surface.
          * A \a splitting surface is a compact surface containing
@@ -1106,8 +1276,6 @@ class NormalSurface : public ShortOutput<NormalSurface> {
          * Thus the results will be reevaluated every time this routine is
          * called.
          *
-         * \todo \opt Cache results.
-         *
          * @return \c true if and only if this is a splitting surface.
          */
         bool isSplitting() const;
@@ -1122,8 +1290,6 @@ class NormalSurface : public ShortOutput<NormalSurface> {
          * Thus the results will be reevaluated every time this routine is
          * called.
          *
-         * \todo \opt Cache results.
-         *
          * @return the number of tetrahedra that this surface meets if it
          * is a central surface, or 0 if it is not a central surface.
          */
@@ -1231,11 +1397,15 @@ class NormalSurface : public ShortOutput<NormalSurface> {
          * operation will not change the topology of the underlying
          * 3-manifold beyond simply slicing along this surface.
          *
+         * As of Regina 7.1, this routine can happily cut along \e almost
+         * normal surfaces as well as normal surfaces.  That is, it can
+         * now handle octagons, including cases with multiple octagons in the
+         * same tetrahedron and/or octagons in multiple tetrahedra.
+         *
          * \warning The number of tetrahedra in the new triangulation
          * can be <i>very</i> large.
          *
          * \pre This normal surface is compact and embedded.
-         * \pre This normal surface contains no octagonal discs.
          *
          * @return the resulting cut-open triangulation.
          */
@@ -1279,6 +1449,42 @@ class NormalSurface : public ShortOutput<NormalSurface> {
          */
         Triangulation<3> crush() const;
 
+        /**
+         * Returns an isotopic normal surface that contains no octagons,
+         * placed within a homeomorphic (but possibly different) triangulation.
+         *
+         * Specifically: this routine returns a surface \a s within a
+         * triangulation \a t, where \a t is homeomorphic to the triangulation
+         * containing this surface, and where \a s is a normal (not
+         * almost normal) surface isotopic to this.  Only the surface \a s is
+         * returned; you can access \a t by calling <tt>s.triangulation()</tt>.
+         *
+         * If this surface is already normal (i.e., it does not contain
+         * any octagons), then the surface returned will simply be a copy
+         * of this surface (but possibly using a different vector encoding),
+         * living within the same triangulation.
+         *
+         * If this surface does contain octagons, then the triangulation \a t
+         * will be obtained from the original by replacing each tetrahedron
+         * containing octagons with \e three tetrahedra (essentially
+         * performing a 0-2 move).  Each octagon can then be subdivided
+         * into a quadrilateral and four triangles.  If the original
+         * triangulation is oriented, then the new triangulation \a t will
+         * preserve this orientation.
+         *
+         * In all cases, the surface that is returned will use a vector
+         * encoding that does not store octagons.
+         *
+         * Note: this routine can happily cope with multiple octagons in the
+         * same tetrahedron, and/or multiple tetrahedra containing octagons.
+         *
+         * \pre This surface is embedded.
+         *
+         * @return an isotopic normal (not almost normal) surface \a s,
+         * as described above.
+         */
+        NormalSurface removeOcts() const;
+
         /**
          * Determines whether this and the given surface in fact
          * represent the same normal (or almost normal) surface.
@@ -1364,19 +1570,6 @@ class NormalSurface : public ShortOutput<NormalSurface> {
          */
         bool operator < (const NormalSurface& other) const;
 
-        /**
-         * Deprecated routine that determines whether this and the given
-         * surface in fact represent the same normal (or almost normal) surface.
-         *
-         * \deprecated This routine has been renamed to the comparison
-         * operator (==).
-         *
-         * @param other the surface to be compared with this surface.
-         * @return \c true if both surfaces represent the same normal or
-         * almost normal surface, or \c false if not.
-         */
-        [[deprecated]] bool sameSurface(const NormalSurface& other) const;
-
         /**
          * Determines whether this surface contains only triangle and/or
          * quadrilateral discs.  This is to distinguish normal surfaces
@@ -1558,15 +1751,6 @@ class NormalSurface : public ShortOutput<NormalSurface> {
          */
         const Vector<LargeInteger>& vector() const;
 
-        /**
-         * A deprecated alias for vector().
-         *
-         * \deprecated This routine has been renamed to vector().
-         *
-         * @return the underlying integer vector.
-         */
-        [[deprecated]] const Vector<LargeInteger>& rawVector() const;
-
         /**
          * Returns the specific integer vector encoding that this surface
          * uses internally.  This is the encoding that should be used
@@ -1674,6 +1858,27 @@ class NormalSurface : public ShortOutput<NormalSurface> {
          */
         void calculateBoundaries() const;
 
+        /**
+         * Determines whether or not a positive rational multiple of this
+         * surface \e could be the normalised link of a face of positive
+         * dimension.
+         *
+         * A non-null return value is \e not a guarantee that this surface
+         * \e is such a link; however, if this routine returns no value then
+         * this \e is a guarantee that the surface is not such a link.
+         *
+         * The precise tests that this routine carries out involve a trade-off
+         * between speed and mathematical power, and so are subject to change
+         * in future versions of Regina.
+         *
+         * \pre This surface is non-empty.
+         *
+         * @return the precise multiple of this surface that \e could be a
+         * normalised non-vertex face link, or no value if we can prove
+         * that this surface is not such a link.
+         */
+        std::optional<NormalSurface> couldLinkFace() const;
+
     friend class XMLNormalSurfaceReader;
 };
 
@@ -1776,10 +1981,6 @@ inline NormalSurface::NormalSurface(const NormalSurface& src,
     triangulation_ = triangulation;
 }
 
-inline NormalSurface* NormalSurface::clone() const {
-    return new NormalSurface(*this);
-}
-
 inline void NormalSurface::swap(NormalSurface& other) noexcept {
     std::swap(enc_, other.enc_);
     vector_.swap(other.vector_);
@@ -1794,6 +1995,7 @@ inline void NormalSurface::swap(NormalSurface& other) noexcept {
     connected_.swap(other.connected_);
     realBoundary_.swap(other.realBoundary_);
     compact_.swap(other.compact_);
+    std::swap(linkOf_, other.linkOf_);
 }
 
 inline LargeInteger NormalSurface::triangles(size_t tetIndex, int vertex)
@@ -1827,10 +2029,6 @@ inline void NormalSurface::setName(const std::string& name) {
     name_ = name;
 }
 
-inline void NormalSurface::writeRawVector(std::ostream& out) const {
-    out << vector_;
-}
-
 inline LargeInteger NormalSurface::eulerChar() const {
     if (! eulerChar_.has_value())
         calculateEulerChar();
@@ -1876,10 +2074,6 @@ inline bool NormalSurface::operator != (const NormalSurface& other) const {
     return ! ((*this) == other);
 }
 
-inline bool NormalSurface::sameSurface(const NormalSurface& other) const {
-    return (*this) == other;
-}
-
 inline bool NormalSurface::normal() const {
     return ! octPosition();
 }
@@ -1888,10 +2082,6 @@ inline const Vector<LargeInteger>& NormalSurface::vector() const {
     return vector_;
 }
 
-inline const Vector<LargeInteger>& NormalSurface::rawVector() const {
-    return vector_;
-}
-
 inline NormalEncoding NormalSurface::encoding() const {
     return enc_;
 }
@@ -1904,6 +2094,10 @@ inline bool NormalSurface::couldBeNonCompact() const {
     return enc_.couldBeNonCompact();
 }
 
+inline NormalSurface NormalSurface::doubleSurface() const {
+    return (*this) * 2;
+}
+
 inline void swap(NormalSurface& a, NormalSurface& b) noexcept {
     a.swap(b);
 }
