@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -1178,7 +1178,7 @@ Perm<dim + 1> FaceBase<dim, subdim>::faceMapping(int f) const {
         front().simplex()->template faceMapping<lowerdim>(inSimp);
 
     // Ensure the images of lowerdim+1,...,dim are correct also.
-    for (unsigned i = subdim + 1; i <= dim; ++i)
+    for (int i = subdim + 1; i <= dim; ++i)
         if (p[i] != i) {
             // Suppose p maps x -> i -> y.
             //
