@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -339,14 +339,18 @@ bool Triangulation<3>::knowsSphere() const {
 }
 
 bool Triangulation<3>::isBall() const {
-    if (prop_.threeBall_.has_value())
-        return *prop_.threeBall_;
+    if (prop_.handlebody_.has_value())
+        return (*prop_.handlebody_ == 0);
 
     // Basic property checks.
-    if (! (isValid() && hasBoundaryTriangles() && isOrientable() && isConnected()
-            && countBoundaryComponents() == 1
-            && boundaryComponents().front()->eulerChar() == 2)) {
-        prop_.threeBall_ = false;
+    if (! (isValid() && isOrientable() && isConnected() &&
+            countBoundaryComponents() == 1)) {
+        // This cannot be a handlebody at all.
+        prop_.handlebody_ = -1;
+        return false;
+    }
+    if (boundaryComponents().front()->eulerChar() != 2) {
+        // It might be a handlebody, but it's not a 3-ball.
         return false;
     }
 
@@ -363,19 +367,32 @@ bool Triangulation<3>::isBall() const {
     // Simplify again in case our coning was inefficient.
     working.intelligentSimplify();
 
-    prop_.threeBall_ = working.isSphere();
-    return *prop_.threeBall_;
+    if (working.isSphere()) {
+        prop_.handlebody_ = 0;
+        return true;
+    } else {
+        // From what we know above, if it's not a 3-ball then it's not a
+        // handlebody of any genus.
+        prop_.handlebody_ = -1;
+        return false;
+    }
 }
 
 bool Triangulation<3>::knowsBall() const {
-    if (prop_.threeBall_.has_value())
+    if (prop_.handlebody_.has_value())
         return true;
 
     // Run some very fast prelimiary tests before we give up and say no.
-    if (! (isValid() && hasBoundaryTriangles() && isOrientable() && isConnected()
-            && countBoundaryComponents() == 1
-            && boundaryComponents().front()->eulerChar() == 2)) {
-        prop_.threeBall_ = false;
+    if (! (isValid() && isOrientable() && isConnected() &&
+            countBoundaryComponents() == 1)) {
+        // This cannot be a handlebody at all.
+        prop_.handlebody_ = -1;
+        return true;
+    }
+    if (boundaryComponents().front()->eulerChar() != 2) {
+        // It might be a handlebody, but it's not a 3-ball.
+        // Although we are not caching the result, we still return true
+        // because isBall() will nevertheless give a very fast "no" answer.
         return true;
     }
 
@@ -384,15 +401,23 @@ bool Triangulation<3>::knowsBall() const {
 }
 
 bool Triangulation<3>::isSolidTorus() const {
-    if (prop_.solidTorus_.has_value())
-        return *prop_.solidTorus_;
+    if (prop_.handlebody_.has_value())
+        return (*prop_.handlebody_ == 1);
 
     // Basic property checks.
     if (! (isValid() && isOrientable() && isConnected() &&
-            countBoundaryComponents() == 1 &&
-            boundaryComponents().front()->eulerChar() == 0 &&
-            boundaryComponents().front()->isOrientable()))
-        return *(prop_.solidTorus_ = false);
+            countBoundaryComponents() == 1)) {
+        // This cannot be a handlebody at all.
+        prop_.handlebody_ = -1;
+        return false;
+    }
+    if (boundaryComponents().front()->eulerChar() != 0) {
+        // It might be a handlebody, but it's not a solid torus.
+        return false;
+    }
+
+    // From here on, if we ever deduce this is not a solid torus, then
+    // we know it cannot be an orientable handlebody of *any* genus at all.
 
     // If it's ideal, make it a triangulation with real boundary.
     // If it's not ideal, clone it anyway so we can modify it.
@@ -404,8 +429,10 @@ bool Triangulation<3>::isSolidTorus() const {
     }
 
     // Check homology.
-    if (! (working.homology().isZ()))
-        return *(prop_.solidTorus_ = false);
+    if (! (working.homology().isZ())) {
+        prop_.handlebody_ = -1;
+        return false;
+    }
 
     // So:
     // We are valid, orientable, compact and connected, with H1 = Z.
@@ -435,7 +462,8 @@ bool Triangulation<3>::isSolidTorus() const {
         std::optional<NormalSurface> s = working.nonTrivialSphereOrDisc();
         if (! s) {
             // No non-trivial normal disc.  This cannot be a solid torus.
-            return *(prop_.solidTorus_ = false);
+            prop_.handlebody_ = -1;
+            return false;
         }
 
         // Crush it and see what happens.
@@ -457,7 +485,8 @@ bool Triangulation<3>::isSolidTorus() const {
                 // A closed piece.
                 // Must be a 3-sphere, or else we didn't have a solid torus.
                 if (! comp.isSphere()) {
-                    return *(prop_.solidTorus_ = false);
+                    prop_.handlebody_ = -1;
+                    return false;
                 }
             } else if (comp.countBoundaryComponents() > 1) {
                 // Multiple boundaries on the same component.
@@ -467,12 +496,14 @@ bool Triangulation<3>::isSolidTorus() const {
                     "isSolidTorus() that should not exist." << std::endl;
 
                 // At any rate, it means we did not have a solid torus.
-                return *(prop_.solidTorus_ = false);
+                prop_.handlebody_ = -1;
+                return false;
             } else if (comp.boundaryComponent(0)->eulerChar() == 2) {
                 // A component with sphere boundary.
                 // Must be a 3-ball, or else we didn't have a solid torus.
                 if (! comp.isBall()) {
-                    return *(prop_.solidTorus_ = false);
+                    prop_.handlebody_ = -1;
+                    return false;
                 }
             } else {
                 // The only other possibility is a component with torus
@@ -497,7 +528,8 @@ bool Triangulation<3>::isSolidTorus() const {
             // The only way this can happen is if we had a solid torus
             // (and we crushed and/or cut along a compressing disc
             // during the crushing operation).
-            return *(prop_.solidTorus_ = true);
+            prop_.handlebody_ = 1;
+            return true;
         }
 
         // We have the original manifold in working, but this time with
@@ -506,23 +538,21 @@ bool Triangulation<3>::isSolidTorus() const {
 }
 
 bool Triangulation<3>::knowsSolidTorus() const {
-    if (prop_.solidTorus_.has_value())
+    if (prop_.handlebody_.has_value())
         return true;
 
-    // Run some very fast prelimiary tests before we give up and say no.
-    if (! (isValid() && isOrientable() && isConnected())) {
-        prop_.solidTorus_ = false;
+    // Run some very fast preliminary tests before we give up and say no.
+    if (! (isValid() && isOrientable() && isConnected() &&
+            countBoundaryComponents() == 1)) {
+        // This cannot be a handlebody at all.
+        prop_.handlebody_ = -1;
         return true;
     }
-
-    if (countBoundaryComponents() != 1) {
-        prop_.solidTorus_ = false;
-        return true;
-    }
-
-    if (boundaryComponents().front()->eulerChar() != 0 ||
-            (! boundaryComponents().front()->isOrientable())) {
-        prop_.solidTorus_ = false;
+    if (boundaryComponents().front()->eulerChar() != 0) {
+        // It might be a handlebody, but it's not a solid torus.
+        // Although we are not caching the result, we still return true
+        // because isSolidTorus() will nevertheless give a very fast "no"
+        // answer.
         return true;
     }
 
@@ -530,6 +560,247 @@ bool Triangulation<3>::knowsSolidTorus() const {
     return false;
 }
 
+ssize_t Triangulation<3>::isHandlebody() const {
+    if ( prop_.handlebody_.has_value() ) {
+        return *prop_.handlebody_;
+    }
+
+    // Basic property checks.
+    // Note: an orientable 3-manifold implies that the boundary is also
+    // orientable (so we do not need to test that separately).
+    if (! (isValid() && isOrientable() && isConnected() &&
+            countBoundaryComponents() == 1)) {
+        // This cannot be a handlebody at all.
+        return *(prop_.handlebody_ = -1);
+    }
+
+    // Determine the genus.
+    // We can immediately check whether this is a 3-ball or a solid torus.
+    size_t genus = ( 2 - boundaryComponents().front()->eulerChar() ) / 2;
+    if ( genus == 0 ) {
+        if ( isBall() ) {
+            // A 3-ball never has a compressing disc.
+            prop_.compressingDisc_ = false;
+            return *(prop_.handlebody_ = 0);
+        } else {
+            return *(prop_.handlebody_ = -1);
+        }
+    }
+    else if ( genus == 1 ) {
+        if ( isSolidTorus() ) {
+            // A solid torus always has a compressing disc.
+            prop_.compressingDisc_ = true;
+            return *(prop_.handlebody_ = 1);
+        } else {
+            return *(prop_.handlebody_ = -1);
+        }
+    }
+
+    // We now know that if this is indeed an orientable handlebody, then it
+    // must have genus at least 2.
+    // Check that homology matches the genus.
+    if ( not ( homology().isFree(genus) ) ) {
+        return *(prop_.handlebody_ = -1);
+    }
+
+    // If it's ideal, make it a triangulation with real boundary.
+    // If it's not ideal, clone it anyway so we can modify it.
+    std::stack<Triangulation<3>> toProcess;
+    Triangulation<3>& start = toProcess.emplace( *this, false );
+    start.intelligentSimplify();
+    if ( start.isIdeal() ) {
+        start.idealToFinite();
+        start.intelligentSimplify();
+    }
+
+    // So:
+    // We are valid, orientable, compact and connected. There is exactly one
+    // boundary component, and this is a genus-g torus, where g >= 2. We also
+    // have H1 = gZ.
+    //
+    // Since "half of the boundary homology lives and the other half dies"
+    // (see Lemma 3.5 of Hatcher's "Notes on Basic 3-Manifold Topology"), we
+    // know that our manifold has g Z terms from the genus-g torus boundary.
+    // Therefore, our manifold cannot be a connected sum of something with
+    // S2xS1, as this would require additional Z terms in the homology.
+    // This observation simplifies the crushing cases later on.
+    //
+    // Hatcher's notes are available at:
+    //      https://pi.math.cornell.edu/~hatcher/3M/3Mdownloads.html
+    //
+    // Anyway, it's time to pull out normal surfaces.
+    while ( not toProcess.empty() ) {
+        // INVARIANT: Our original triangulation is an orientable handlebody
+        // if and only if every triangulation in toProcess is an orientable
+        // handlebody with positive genus.
+        Triangulation<3>& top = toProcess.top();
+        if ( top.countVertices() > 1 ) {
+            // Try *really* hard to get to a 1-vertex triangulation, since
+            // this will make nonTrivialSphereOrDisc() much faster (it will
+            // be able to use linear programming).
+            top.intelligentSimplify();
+            if ( top.countVertices() > 1 ) {
+                top.barycentricSubdivision();
+                top.intelligentSimplify();
+                top.intelligentSimplify();
+            }
+        }
+
+        // Find a non-trivial normal disc or sphere.
+        std::optional<NormalSurface> s = top.nonTrivialSphereOrDisc();
+        if ( not s ) {
+            // No non-trivial normal disc. This cannot be an orientable
+            // handlebody.
+            return *(prop_.handlebody_ = -1);
+        }
+
+        // Crush it and see what happens.
+        // Given what we know about the manifold so far, the only things that
+        // can happen during crushing are:
+        // - undoing connected sums;
+        // - cutting along properly embedded discs;
+        // - gaining and/or losing 3-balls and/or 3-spheres.
+        //
+        // Thus, if we started with a handlebody, then crushing can only
+        // result in pieces of the following types:
+        // - handlebodies;
+        // - 3-spheres;
+        // - 3-balls.
+        Triangulation<3> crushed = s->crush();
+        s.reset(); // to avoid a deep copy of top when we pop
+        toProcess.pop();
+
+        crushed.intelligentSimplify();
+
+        for ( Triangulation<3>& comp : crushed.triangulateComponents() ) {
+            // Examine each connected component after crushing.
+            if ( comp.isClosed() ) {
+                // A closed piece.
+                // Must be a 3-sphere, or else we didn't have an orientable
+                // handlebody.
+                if ( not comp.isSphere() ) {
+                    return *(prop_.handlebody_ = -1);
+                }
+            } else if ( comp.countBoundaryComponents() > 1 ) {
+                // Multiple boundaries on the same component.
+                // CLAIM. Given what we know about the homology, this can
+                //      never happen.
+                // A very rough sketch of a proof for this claim is given at
+                // the end of this for loop
+                // TODO Throw a proper exception.
+                std::cerr << "ERROR: S2xS1 summand detected in "
+                    "isHandlebody() that should not exist." << std::endl;
+
+                // At any rate, it means we did not have an orientable
+                // handlebody.
+                return *(prop_.handlebody_ = -1);
+            } else if ( comp.boundaryComponent(0)->eulerChar() == 2 ) {
+                // A component with sphere boundary.
+                // Must be a 3-ball, or else we didn't have a solid torus.
+                if ( not comp.isBall() ) {
+                    return *(prop_.handlebody_ = -1);
+                }
+            } else {
+                // The only other possibility is a component whose boundary
+                // has positive genus. We need to check that this component
+                // is an orientable handlebody, so insert it back into the
+                // list to process.
+                toProcess.push( std::move(comp) );
+            }
+        }
+        // PROOF OF CLAIM.
+        // ==================================================================
+        // Suppose crushing produced a component with multiple boundaries.
+        // We must have cut some piece P along a properly embedded disc D
+        // such that:
+        // ---> the boundary of D separates the boundary of P into two pieces
+        //      A and B; and
+        // ---> D itself is non-separating, so there exists a closed curve c
+        //      in P that meets D exactly once.
+        // Our goal is to show that this, in combination with what we know
+        // about the homology of P, leads to a contradiction.
+        //
+        // We first consider the case where P is the original triangulation.
+        // Let g and h denote the genus of A and B, respectively, so that the
+        // boundary of P has genus g+h. At the beginning of the algorithm, we
+        // verified that the homology of P is (g+h)Z. With this in mind, take
+        // the union of A and D, and push this slightly off the boundary to
+        // obtain a closed genus-g surface S in P that meets the closed curve
+        // c exactly once. Let X' be a small neighbourhood of the union of S
+        // and c, and let Y' be the space given by removing the interior of X
+        // from P. The intersection of X and Y is a surface R of genus 2g.
+        // Let N be a thickening of R, let X be the union of X' and N, and
+        // let Y be the union of Y' and N; this gives subspaces X and Y whose
+        // interiors cover P, and the intersection of these two subspaces is
+        // exactly N. Thus, letting G = H1(X)+H1(Y), we have the following
+        // exact sequence (the Mayer-Vietoris sequence for the pair of
+        // subspaces X and Y):
+        //                        p       q
+        //      ... ----> H1(N) ----> G ----> H1(P) ----> ...
+        // We make the following homological observations:
+        // ---> H1(N) = (4g)Z, because N retracts to a surface of genus 2g.
+        // ---> H1(X) = (2g+1)Z, because X retracts to the union of S and c.
+        // ---> H1(Y) has rank at least 2g+(g+h) because it has two boundary
+        //      components, one of which has genus 2g, while the other has
+        //      genus g+h.
+        // ---> Thus, G has rank at least (4g+1)+(g+h)
+        // ---> H1(P) = (g+h)Z, by assumption.
+        // Notice that im(q) must be a free abelian group (of rank at most
+        // g+h), which implies that G is isomorphic to ker(q)+im(q). Thus,
+        // ker(q) must have rank at least 4g+1. By exactness, im(p) must also
+        // have rank at least 4g+1, which is impossible since H1(N) only has
+        // rank 4g.
+        //
+        // It remains to consider the case where P was obtained by some
+        // sequence of crushing operations. Topologically, we know that P
+        // was obtained from the original triangulation by cutting along
+        // properly embedded discs. This implies that the homology of P is
+        // kZ, where k is the genus of the boundary of P (one way to check
+        // that cutting along discs does indeed preserve this property of the
+        // homology is to again use Mayer-Vietoris sequences). Thus, by the
+        // same argument as above, it is impossible for P to contain an
+        // embedded disc with the properties of D.
+        // ==================================================================
+
+        // If we survived to this point, then we still haven't conclusively
+        // determined whether we started with an orientable handlebody.
+        // However, we have made progress: by crushing a non-trivial normal
+        // surface, we guarantee that we have reduced the total number of
+        // tetrahedra in the list to process, so we won't keep reaching this
+        // point forever. This means that we must eventually terminate via
+        // one of the following pathways:
+        // ---> We detect a component that cannot occur if we started with an
+        //      orientable handlebody.
+        // ---> The list to process becomes empty. This can only happen if
+        //      everything fell apart into 3-spheres and 3-balls, which
+        //      certifies that we started with an orientable handlebody.
+    }
+
+    // The list to process became empty, so we must have started with an
+    // orientable handlebody (of genus at least 2).
+    // By the way, we know that this always has a compressing disc.
+    prop_.compressingDisc_ = true;
+    return *(prop_.handlebody_ = genus);
+}
+
+bool Triangulation<3>::knowsHandlebody() const {
+    if (prop_.handlebody_.has_value()) {
+        return true;
+    }
+
+    // Run some very fast preliminary tests before we give up and say no.
+    // Note: an orientable 3-manifold implies that the boundary is also
+    // orientable (so we do not need to test that separately).
+    if (! (isValid() && isOrientable() && isConnected() &&
+            countBoundaryComponents() == 1)) {
+        // This cannot be a handlebody at all.
+        prop_.handlebody_ = -1;
+        return true;
+    }
+
+    // More work is required.
+    return false;
+}
 
 bool Triangulation<3>::isTxI() const {
     // This call to knowsTxI checks basic things including validity and also
@@ -1059,13 +1330,13 @@ bool Triangulation<3>::isHaken() const {
     unsigned i;
     for (i = 0; i < list.size(); ++i) {
         id[i].index = i;
-        id[i].euler = list.surface(i).eulerChar().longValue();
+        id[i].euler = list[i].eulerChar().longValue();
     }
     std::sort(id, id + list.size());
 
     for (i = 0; i < list.size(); ++i) {
         // std::cout << "Testing surface " << i << "..." << std::endl;
-        if (list.surface(id[i].index).isIncompressible()) {
+        if (list[id[i].index].isIncompressible()) {
             delete[] id;
             prop_.threeSphere_ = false; // Implied by Hakenness.
             return *(prop_.haken_ = true);
