@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -46,6 +46,7 @@
 #include <memory>
 #include <vector>
 #include "regina-core.h"
+#include "hypersurface/normalhypersurface.h"
 #include "progress/progresstracker.h"
 #include "triangulation/detail/retriangulate.h"
 #include "triangulation/generic/triangulation.h"
@@ -56,6 +57,7 @@
 
 namespace regina {
 
+class IntersectionForm;
 class ProgressTracker;
 class ProgressTrackerOpen;
 
@@ -90,17 +92,20 @@ template <int> class XMLTriangulationReader;
 template <>
 class Triangulation<4> : public detail::TriangulationBase<4> {
     private:
-        bool knownSimpleLinks_ { false };
-            /**< Is it known that all vertex links are 3-spheres or 3-balls?
-                 This may be \c true even if the skeleton has not yet been
-                 calculated (thereby allowing us to avoid costly 3-sphere or
-                 3-ball recognition when the skeleton is eventually computed).
-                 A value of \c false may mean that there are other vertex
-                 links, or it may mean that the vertex links have not yet
-                 been calculated. */
-
-        bool ideal_;
-            /**< Is the triangulation ideal? */
+        long vertexLinkSummary_ { -1 };
+            /**< This is a computed property that caches information about the
+                 vertex links, and is optimised for valid triangulations.
+                 If \a vertexLinkSummary_ is non-negative, this means that
+                 every vertex is valid (i.e., each vertex link is either
+                 a 3-ball or a closed 3-manifold); moreover, the numeric value
+                 of \a vertexLinkSummary_ indicates how many vertices are ideal
+                 (i.e., have links that are closed but not 3-spheres).
+                 If \a vertexLinkSummary_ is negative, this means that either
+                 one or more vertices are invalid, and/or the skeleton has
+                 not yet been computed.
+                 Crucially, this property may be known \e before the skeleton
+                 is computed (thereby allowing us to avoid costly 3-sphere or
+                 3-ball recognition when the skeleton is computed later on). */
 
         /**
          * A struct that holds all of our calculated properties.
@@ -318,16 +323,6 @@ class Triangulation<4> : public detail::TriangulationBase<4> {
          * swapped with this.
          */
         void swap(Triangulation<4>& other);
-        /**
-         * Deprecated routine that swaps the contents of this and the
-         * given triangulation.
-         *
-         * \deprecated Use swap() instead.
-         *
-         * @param other the triangulation whose contents should be
-         * swapped with this.
-         */
-        [[deprecated]] void swapContents(Triangulation<4>& other);
 
         /*@}*/
         /**
@@ -416,18 +411,35 @@ class Triangulation<4> : public detail::TriangulationBase<4> {
         /*@{*/
 
         /**
-         * Deprecated routine that returns the second homology group for
-         * this triangulation.
+         * Returns the intersection form of this 4-manifold.
          *
-         * \deprecated This is identical to calling homology<2>().
+         * The intersection form is stored as a square matrix with respect to
+         * an arbitrary basis of the second homology group; you can access this
+         * matrix via IntersectionForm::matrix().  This matrix is guaranteed
+         * to be symmetric and unimodular, but will not be normalised in any
+         * way.  You can, however, query invariants of the intersection form
+         * via routines such as IntersectionForm::signature() and
+         * IntersectionForm::even(), which in the simply connected case are
+         * enough to determine the topology of the underlying 4-manifold.
          *
-         * \pre This triangulation is valid.
+         * (As an implementation detail, the basis is currently chosen to be
+         * the one produced by constructing a MarkedAbelianGroup using the
+         * boundary maps on the dual faces.  This specific choice of basis is
+         * subject to change in future releases of Regina.)
          *
-         * \exception FailedPrecondition This triangulation is invalid.
+         * The sign convention for counting intersections is as follows: in a
+         * pentachoron with positive orientation and vertices (0,1,2,3,4),
+         * the triangles with ordered vertices (0,1,2) and ordered vertices
+         * (2,3,4) have positive intersection.
          *
-         * @return the second homology group.
+         * \pre This triangulation is valid, non-empty, orientable and closed.
+         *
+         * \exception FailedPrecondition This triangulation is invalid,
+         * empty, non-orientable, or not closed.
+         *
+         * @return the intersection form of this 4-manifold.
          */
-        [[deprecated]] AbelianGroup homologyH2() const;
+        IntersectionForm intersectionForm() const;
 
         /*@}*/
         /**
@@ -1102,6 +1114,42 @@ class Triangulation<4> : public detail::TriangulationBase<4> {
         bool idealToFinite();
 
         /*@}*/
+        /**
+         * \name Normal Hypersurfaces
+         */
+        /*@{*/
+
+        /**
+         * Returns the link of the given face as a normal hypersurface.
+         *
+         * Constructing the link of a face begins with building the frontier
+         * of a regular neighbourhood of the face.  If this is already a
+         * normal hypersurface, then then link is called \e thin.  Otherwise
+         * some basic normalisation steps are performed until the hypersurface
+         * becomes normal; note that these normalisation steps could
+         * change the topology of the hypersurface, and in some pathological
+         * cases could even reduce it to the empty hypersurface.
+         *
+         * Although normalisation of arbitrary embedded 3-manifolds is messy,
+         * for face links the process is thankfully simpler.  Essentially,
+         * any changes will be limited to operations analagous to compressions
+         * and boundary compressions along discs and 3-balls, as well as
+         * removing trivial 4-sphere components.
+         *
+         * \tparam subdim the dimension of the face to link; this must be
+         * between 0 and 3 inclusive.
+         *
+         * \pre The given face is a face of this triangulation.
+         *
+         * @return a pair (\a s, \a thin), where \a s is the face linking
+         * normal hypersurface, and \a thin is \c true if and only if this link
+         * is thin (i.e., no additional normalisation steps were required).
+         */
+        template <int subdim>
+        std::pair<NormalHypersurface, bool> linkingSurface(
+            const Face<4, subdim>& face) const;
+
+        /*@}*/
 
     private:
         /**
@@ -1137,6 +1185,8 @@ class Triangulation<4> : public detail::TriangulationBase<4> {
 } // namespace regina
 // Some more headers that are required for inline functions:
 #include "triangulation/dim4/pentachoron4.h"
+#include "triangulation/dim4/tetrahedron4.h"
+#include "triangulation/dim4/triangle4.h"
 #include "triangulation/dim4/edge4.h"
 #include "triangulation/dim4/vertex4.h"
 #include "triangulation/dim4/component4.h"
@@ -1190,8 +1240,7 @@ inline Triangulation<4>& Triangulation<4>::operator = (
 
     TriangulationBase<4>::operator = (src);
 
-    knownSimpleLinks_ = src.knownSimpleLinks_;
-    ideal_ = src.ideal_;
+    vertexLinkSummary_ = src.vertexLinkSummary_;
     prop_ = src.prop_;
 
     return *this;
@@ -1203,12 +1252,13 @@ inline Triangulation<4>& Triangulation<4>::operator = (Triangulation&& src) {
 
     ChangeEventSpan span(*this);
 
-    TriangulationBase<4>::operator = (std::move(src));
+    // The parent class assignment goes last, since its move invalidates src.
 
-    knownSimpleLinks_ = src.knownSimpleLinks_;
-    ideal_ = src.ideal_;
+    vertexLinkSummary_ = src.vertexLinkSummary_;
     prop_ = std::move(src.prop_);
 
+    TriangulationBase<4>::operator = (std::move(src));
+
     return *this;
 }
 
@@ -1222,7 +1272,15 @@ inline size_t Triangulation<4>::countBoundaryTetrahedra() const {
 
 inline bool Triangulation<4>::isIdeal() const {
     ensureSkeleton();
-    return ideal_;
+
+    // Recall that for 4-manifolds we restrict "ideal" to only include
+    // valid triangulations.
+    if (! valid_)
+        return false;
+
+    // Since the triangulation is valid, we can deduce the answer from
+    // vertexLinkSummary_.
+    return (vertexLinkSummary_ > 0);
 }
 
 inline bool Triangulation<4>::isClosed() const {
@@ -1230,10 +1288,6 @@ inline bool Triangulation<4>::isClosed() const {
     return boundaryComponents().empty();
 }
 
-inline AbelianGroup Triangulation<4>::homologyH2() const {
-    return homology<2>();
-}
-
 template <typename Action, typename... Args>
 inline bool Triangulation<4>::retriangulate(int height, unsigned nThreads,
         ProgressTrackerOpen* tracker, Action&& action, Args&&... args) const {
@@ -1280,10 +1334,6 @@ inline bool Triangulation<4>::simplifyExhaustive(int height, unsigned nThreads,
         }, *this, size());
 }
 
-inline void Triangulation<4>::swapContents(Triangulation<4>& other) {
-    swap(other);
-}
-
 } // namespace regina
 
 #endif
