@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -83,17 +83,48 @@ Triangulation<2> Example<2>::nonOrientable(unsigned genus, unsigned punctures) {
 
     Triangulation<2> ans;
 
-    unsigned n = 2 * genus + 3 * punctures - 2;
-    unsigned i;
-    ans.newTriangles(n);
-    for (i = 0; i < n - 1; ++i)
-        ans.triangle(i)->join(1, ans.triangle(i + 1), Perm<3>(1, 2));
-    ans.triangle(0)->join(2, ans.triangle(n - 1), Perm<3>(2, 0, 1));
-    for (i = 1; i < genus; ++i)
-        ans.triangle(2 * i - 2)->join(0, ans.triangle(2 * i - 1), Perm<3>());
-    for (i = 0; i < punctures; ++i)
-        ans.triangle(2 * genus + 3 * i - 2)->join(
-            0, ans.triangle(2 * genus + 3 * i), Perm<3>(1, 2));
+    // The generic code below will create one internal vertex, and one for
+    // each puncture.  This is minimal for zero punctures, but non-minimal
+    // otherwise.  For now, we use a different triangulation for the
+    // once-punctured case so at least that gets to be minimal also;
+    // ideally these should be minimal for all values of punctures.
+
+    if (punctures == 1) {
+        // Thanks to Alex He for this code.
+
+        // Let g denote the given genus. We use g-1 "inner" triangles and g
+        // "outer" triangles, for a total of 2*g-1 triangles. We start by using
+        // the g-1 "inner" triangles to build a (g+1)-sided polygon P. We then
+        // form each of the g "outer" triangles into a one-triangle Mobius band,
+        // and attach the boundary of each of these Mobius bands to one of the
+        // sides of P. It is clear that the resulting surface is once-punctured
+        // and one-vertex, and has non-orientable genus g.
+        unsigned n = 2*genus - 1;
+        unsigned i;
+        ans.newTriangles(n);
+        // Form "outer" triangles into Mobius bands.
+        for ( i = genus - 1; i < n; ++i ) {
+            ans.triangle(i)->join(
+                    0, ans.triangle(i), Perm<3>(1, 2, 0) );
+        }
+        // Glue everything together.
+        for ( i = 1; i < n; ++i ) {
+            ans.triangle(i)->join(
+                    2, ans.triangle( (i-1)/2 ), Perm<3>( 2, i%2 ) );
+        }
+    } else {
+        unsigned n = 2 * genus + 3 * punctures - 2;
+        unsigned i;
+        ans.newTriangles(n);
+        for (i = 0; i < n - 1; ++i)
+            ans.triangle(i)->join(1, ans.triangle(i + 1), Perm<3>(1, 2));
+        ans.triangle(0)->join(2, ans.triangle(n - 1), Perm<3>(2, 0, 1));
+        for (i = 1; i < genus; ++i)
+            ans.triangle(2 * i - 2)->join(0, ans.triangle(2 * i - 1), Perm<3>());
+        for (i = 0; i < punctures; ++i)
+            ans.triangle(2 * genus + 3 * i - 2)->join(
+                0, ans.triangle(2 * genus + 3 * i), Perm<3>(1, 2));
+    }
 
     return ans;
 }
