@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -152,16 +152,6 @@ class Example<3> : public detail::ExampleBase<3> {
          */
         static Triangulation<3> poincare();
 
-        /**
-         * Deprecated routine that returns the five-tetrahedron triangulation
-         * of the Poincare homology sphere.
-         *
-         * \deprecated This routine has been renamed to poincare().
-         *
-         * @return the Poincare homology sphere.
-         */
-        [[deprecated]] static Triangulation<3> poincareHomologySphere();
-
         /**
          * Returns an augmented triangular solid torus with the given
          * parameters.  Almost all augmented triangular solid tori represent
@@ -321,6 +311,28 @@ class Example<3> : public detail::ExampleBase<3> {
          */
         static Triangulation<3> lst(size_t a, size_t b);
 
+        /**
+         * Returns a triangulation of the orientable handlebody with the
+         * given genus.
+         *
+         * For positive genus, this routine uses a minimal layered
+         * triangulation of the orientable handlebody. This is constructed by
+         * starting with a one-vertex triangulation of a once-punctured
+         * non-orientable surface with the given genus, and layering a
+         * tetrahedron onto each internal edge of this surface, yielding a
+         * (3*genus-2)-tetrahedron triangulation. For genus greater than one,
+         * there are many choices for how to do this; this routine makes an
+         * arbitrary choice.
+         *
+         * For genus 0, this routine uses the one-tetrahedron 3-ball.
+         *
+         * @param genus the genus of the handlebody.
+         * @return the orientable handlebody with the given genus.
+         *
+         * @author Alex He
+         */
+        static Triangulation<3> handlebody(size_t genus);
+
         /**
          * Returns a triangulation of the solid Klein bottle.
          * This is isomorphic to the triangulation returned by the
@@ -388,10 +400,6 @@ inline Triangulation<3> Example<3>::s2xs1() {
     return sphereBundle();
 }
 
-inline Triangulation<3> Example<3>::poincareHomologySphere() {
-    return poincare();
-}
-
 inline Triangulation<3> Example<3>::solidKleinBottle() {
     return twistedBallBundle();
 }
