@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -70,6 +70,27 @@ template MatrixInt TriangulationBase<11>::boundaryMap<8>() const;
 template MatrixInt TriangulationBase<11>::boundaryMap<9>() const;
 template MatrixInt TriangulationBase<11>::boundaryMap<10>() const;
 template MatrixInt TriangulationBase<11>::boundaryMap<11>() const;
+template MatrixInt TriangulationBase<11>::dualBoundaryMap<1>() const;
+template MatrixInt TriangulationBase<11>::dualBoundaryMap<2>() const;
+template MatrixInt TriangulationBase<11>::dualBoundaryMap<3>() const;
+template MatrixInt TriangulationBase<11>::dualBoundaryMap<4>() const;
+template MatrixInt TriangulationBase<11>::dualBoundaryMap<5>() const;
+template MatrixInt TriangulationBase<11>::dualBoundaryMap<6>() const;
+template MatrixInt TriangulationBase<11>::dualBoundaryMap<7>() const;
+template MatrixInt TriangulationBase<11>::dualBoundaryMap<8>() const;
+template MatrixInt TriangulationBase<11>::dualBoundaryMap<9>() const;
+template MatrixInt TriangulationBase<11>::dualBoundaryMap<10>() const;
+template MatrixInt TriangulationBase<11>::dualToPrimal<0>() const;
+template MatrixInt TriangulationBase<11>::dualToPrimal<1>() const;
+template MatrixInt TriangulationBase<11>::dualToPrimal<2>() const;
+template MatrixInt TriangulationBase<11>::dualToPrimal<3>() const;
+template MatrixInt TriangulationBase<11>::dualToPrimal<4>() const;
+template MatrixInt TriangulationBase<11>::dualToPrimal<5>() const;
+template MatrixInt TriangulationBase<11>::dualToPrimal<6>() const;
+template MatrixInt TriangulationBase<11>::dualToPrimal<7>() const;
+template MatrixInt TriangulationBase<11>::dualToPrimal<8>() const;
+template MatrixInt TriangulationBase<11>::dualToPrimal<9>() const;
+template MatrixInt TriangulationBase<11>::dualToPrimal<10>() const;
 
 template const GroupPresentation& TriangulationBase<11>::fundamentalGroup() const;
 
