@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -57,6 +57,15 @@ template MatrixInt TriangulationBase<5>::boundaryMap<2>() const;
 template MatrixInt TriangulationBase<5>::boundaryMap<3>() const;
 template MatrixInt TriangulationBase<5>::boundaryMap<4>() const;
 template MatrixInt TriangulationBase<5>::boundaryMap<5>() const;
+template MatrixInt TriangulationBase<5>::dualBoundaryMap<1>() const;
+template MatrixInt TriangulationBase<5>::dualBoundaryMap<2>() const;
+template MatrixInt TriangulationBase<5>::dualBoundaryMap<3>() const;
+template MatrixInt TriangulationBase<5>::dualBoundaryMap<4>() const;
+template MatrixInt TriangulationBase<5>::dualToPrimal<0>() const;
+template MatrixInt TriangulationBase<5>::dualToPrimal<1>() const;
+template MatrixInt TriangulationBase<5>::dualToPrimal<2>() const;
+template MatrixInt TriangulationBase<5>::dualToPrimal<3>() const;
+template MatrixInt TriangulationBase<5>::dualToPrimal<4>() const;
 
 template const GroupPresentation& TriangulationBase<5>::fundamentalGroup() const;
 
