@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -61,6 +61,19 @@ template MatrixInt TriangulationBase<7>::boundaryMap<4>() const;
 template MatrixInt TriangulationBase<7>::boundaryMap<5>() const;
 template MatrixInt TriangulationBase<7>::boundaryMap<6>() const;
 template MatrixInt TriangulationBase<7>::boundaryMap<7>() const;
+template MatrixInt TriangulationBase<7>::dualBoundaryMap<1>() const;
+template MatrixInt TriangulationBase<7>::dualBoundaryMap<2>() const;
+template MatrixInt TriangulationBase<7>::dualBoundaryMap<3>() const;
+template MatrixInt TriangulationBase<7>::dualBoundaryMap<4>() const;
+template MatrixInt TriangulationBase<7>::dualBoundaryMap<5>() const;
+template MatrixInt TriangulationBase<7>::dualBoundaryMap<6>() const;
+template MatrixInt TriangulationBase<7>::dualToPrimal<0>() const;
+template MatrixInt TriangulationBase<7>::dualToPrimal<1>() const;
+template MatrixInt TriangulationBase<7>::dualToPrimal<2>() const;
+template MatrixInt TriangulationBase<7>::dualToPrimal<3>() const;
+template MatrixInt TriangulationBase<7>::dualToPrimal<4>() const;
+template MatrixInt TriangulationBase<7>::dualToPrimal<5>() const;
+template MatrixInt TriangulationBase<7>::dualToPrimal<6>() const;
 
 template const GroupPresentation& TriangulationBase<7>::fundamentalGroup() const;
 
