@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -213,9 +213,7 @@ class FaceEmbedding : public detail::FaceEmbeddingBase<dim, subdim> {
  */
 template <int dim, int subdim>
 class Face : public detail::FaceBase<dim, subdim> {
-    static_assert(dim == 2 || dim > 4 ||
-        (dim == 3 && subdim == 1) ||
-        (dim == 4 && subdim >= 2 && subdim <= 3),
+    static_assert(dim == 2 || dim > 4,
         "The generic implementation of Face<dim, subdim> "
         "should not be used for those face class that are specialised "
         "in Regina's standard dimensions.");
