@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -51,6 +51,8 @@
 #include "maths/perm.h"
 #include "utilities/exception.h"
 #include "utilities/randutils.h"
+#include "utilities/tightencoding.h"
+#include <sys/types.h> // for ssize_t
 #include <algorithm>
 
 namespace regina {
@@ -112,19 +114,20 @@ namespace regina {
 template <int dim>
 class Isomorphism :
         public Output<Isomorphism<dim>>,
+        public TightEncodable<Isomorphism<dim>>,
         public alias::IsomorphismImage<Isomorphism<dim>, dim> {
     static_assert(dim >= 2, "Isomorphism requires dimension >= 2.");
 
     protected:
-        unsigned nSimplices_;
+        size_t size_;
             /**< The number of simplices in the source triangulation. */
-        int* simpImage_;
+        ssize_t* simpImage_;
             /**< Stores the simplex of the destination triangulation that
                  each simplex of the source triangulation maps to.
-                 This array has size nSimplices_. */
+                 This array has size size_. */
         Perm<dim+1>* facetPerm_;
             /**< The permutation applied to the facets of each
-                 source simplex.  This array has size nSimplices_. */
+                 source simplex.  This array has size size_. */
 
     public:
         /**
@@ -140,7 +143,7 @@ class Isomorphism :
          * triangulation associated with this isomorphism.
          * This is allowed to be zero.
          */
-        Isomorphism(unsigned nSimplices);
+        Isomorphism(size_t nSimplices);
         /**
          * Creates a copy of the given isomorphism.
          *
@@ -207,16 +210,20 @@ class Isomorphism :
          *
          * @return the number of simplices in the source triangulation.
          */
-        unsigned size() const;
+        size_t size() const;
 
         /**
-         * Determines the image of the given source simplex under
-         * this isomorphism.
+         * Returns a read-write reference to the image of the given source
+         * simplex under this isomorphism.
          *
          * If the dimension \a dim is 2, 3 or 4, then you can also access
          * this image through the dimension-specific alias triImage(),
          * tetImage() or pentImage() respectively.
          *
+         * This image is stored using type \c ssize_t, not \c size_t,
+         * and so you can safely use the special value -1 as a marker for an
+         * image that is unknown or not yet initialised.
+         *
          * \ifacespython Python users can only access the read-only version
          * of this function that returns by value: you cannot use simpImage()
          * to edit the isomorphism.  As an alternative however, Python users
@@ -227,7 +234,7 @@ class Isomorphism :
          * @return a reference to the index of the destination simplex
          * that the source simplex maps to.
          */
-        int& simpImage(unsigned sourceSimp);
+        ssize_t& simpImage(size_t sourceSimp);
         /**
          * Determines the image of the given source simplex under
          * this isomorphism.
@@ -241,7 +248,7 @@ class Isomorphism :
          * @return the index of the destination simplex
          * that the source simplex maps to.
          */
-        int simpImage(unsigned sourceSimp) const;
+        ssize_t simpImage(size_t sourceSimp) const;
         /**
          * Returns a read-write reference to the permutation that is
          * applied to the (\a dim + 1) facets of the given source simplex
@@ -265,7 +272,7 @@ class Isomorphism :
          * @return a read-write reference to the permutation applied to the
          * facets of the source simplex.
          */
-        Perm<dim+1>& facetPerm(unsigned sourceSimp);
+        Perm<dim+1>& facetPerm(size_t sourceSimp);
         /**
          * Determines the permutation that is applied to the (\a dim + 1)
          * facets of the given source simplex under this isomorphism.
@@ -283,7 +290,7 @@ class Isomorphism :
          * @return the permutation applied to the facets of the
          * source simplex.
          */
-        Perm<dim+1> facetPerm(unsigned sourceSimp) const;
+        Perm<dim+1> facetPerm(size_t sourceSimp) const;
         /**
          * Determines the image of the given source simplex facet
          * under this isomorphism.  This operator returns by value:
@@ -321,7 +328,7 @@ class Isomorphism :
          * In more detail: A new triangulation \a U is returned, so that this
          * isomorphism represents a one-to-one, onto and boundary complete
          * isomorphism from \a T to \a U.  That is, \a T and \a U will be
-         * combinatorially identical triangulations, and this isomorphism
+         * combinatorially isomorphic triangulations, and this isomorphism
          * describes the mapping from the simplices of \a T and their facets
          * to the simplices of \a U and their facets.
          *
@@ -332,39 +339,125 @@ class Isomorphism :
          * \todo Lock the topological properties of the underlying manifold,
          * to avoid recomputing them after the isomorphism is applied.
          *
-         * \exception InvalidArgument the number of simplices in the given
-         * triangulation is not equal to size() for this isomorphism.
+         * \exception InvalidArgument the number of top-dimensional simplices
+         * in the given triangulation is not equal to size() for this
+         * isomorphism.
          *
-         * @param original the triangulation to which this isomorphism
+         * @param tri the triangulation to which this isomorphism
          * should be applied.
          * @return the new isomorphic triangulation.
          */
-        Triangulation<dim> apply(const Triangulation<dim>& original) const;
+        Triangulation<dim> operator ()(const Triangulation<dim>& tri) const;
 
         /**
-         * Applies this isomorphism to the given triangulation,
-         * modifying the given triangulation directly.
+         * Returns the image of the given facet-of-simplex under this
+         * isomorphism.
          *
-         * This is similar to apply(), except that instead of creating a
-         * new triangulation, the simplices and vertices of the given
-         * triangulation are modified in-place.
+         * Specifically:
          *
-         * See apply() for further details on how this operation is performed.
+         * - If <tt>f.simp</tt> is in the range 0,1,...,size()-1 inclusive
+         *   (i.e., \a f denotes a facet of an actual top-dimensional simplex),
+         *   then this routine will return an object denoting facet
+         *   <tt>facetPerm(f.facet)</tt> of simplex <tt>simpImage(f.simp)</tt>.
+         *
+         * - If <tt>f.simp</tt> is negative (i.e., \a f takes a
+         *   before-the-start value), or if <tt>f.simp</tt> is at least size()
+         *   (i.e., \a f takes a boundary or past-the-end value), then this
+         *   routine will return \a f unchanged (but see the precondition
+         *   below).
+         *
+         * \pre If this isomorphism maps a smaller triangulation into a larger
+         * triangulation (in particular, if the simplex images under this
+         * isomorphism are not just some reordering of 0,1,...,size()-1),
+         * then \a f must not denote a boundary or past-the-end value.
+         * This is because a boundary or past-the-end value is encoded
+         * by using a past-the-end value of <tt>FacetSpec::simp</tt>.
+         * If this isomorphism maps into a larger triangulation then this
+         * past-the-end simplex number would need to change, but the
+         * isomorphism does not actually know what the new value of
+         * <tt>FacetSpec::simp</tt> should be.
+         *
+         * @param f the facet-of-simplex which should be transformed by
+         * this isomorphism.
+         * @return the image of \a f under this isomorphism.
+         */
+        FacetSpec<dim> operator ()(const FacetSpec<dim>& f) const;
+
+        /**
+         * Applies this isomorphism to the given facet pairing, and
+         * returns the result as a new facet pairing.
+         *
+         * Although the Isomorphism class was designed to represent
+         * mappings between isomorphic triangulations, it can just as well
+         * describe mappings between isomorphic facet pairings.
+         * In particular, if \a iso represents this isomorphism and if \a p
+         * were the facet pairing of some triangulation \a tri, then
+         * <tt>iso(p)</tt> would be the facet pairing for the triangulation
+         * <tt>iso(tri)</tt>.  Of course, this routine works directly with
+         * the facet pairing, and does not actually construct any
+         * triangulations at all.
+         *
+         * This routine behaves correctly even if some facets of \a p are
+         * unmatched (i.e., if \a p models a triangulation with boundary
+         * facets).
          *
          * \pre The simplex images are precisely 0,1,...,size()-1 in some
          * order (i.e., this isomorphism does not represent a mapping from a
          * smaller triangulation into a larger triangulation).
          *
-         * \todo Lock the topological properties of the underlying manifold,
-         * to avoid recomputing them after the isomorphism is applied.
+         * \exception InvalidArgument the number of top-dimensional simplices
+         * described by the given facet pairing is not equal to size() for
+         * this isomorphism.
          *
-         * \exception InvalidArgument the number of simplices in the given
-         * triangulation is not equal to size() for this isomorphism.
+         * @param p the facet pairing to which this isomorphism should be
+         * applied.
+         * @return the new isomorphic facet pairing.
+         */
+        FacetPairing<dim> operator ()(const FacetPairing<dim>& p) const;
+
+        /**
+         * Deprecated routine that applies this isomorphism to the given
+         * triangulation, and returns the result as a new triangulation.
+         *
+         * \deprecated If this isomorphism is \a iso, then this routine
+         * is equivalent to calling <tt>iso(tri)</tt>.  See the bracket
+         * operator for further details.
+         *
+         * \pre The simplex images are precisely 0,1,...,size()-1 in some
+         * order (i.e., this isomorphism does not represent a mapping from a
+         * smaller triangulation into a larger triangulation).
+         *
+         * \exception InvalidArgument the number of top-dimensional simplices
+         * in the given triangulation is not equal to size() for this
+         * isomorphism.
+         *
+         * @param tri the triangulation to which this isomorphism
+         * should be applied.
+         * @return the new isomorphic triangulation.
+         */
+        [[deprecated]] Triangulation<dim> apply(const Triangulation<dim>& tri)
+            const;
+
+        /**
+         * Deprecated routine that applies this isomorphism to the given
+         * triangulation, modifying the given triangulation directly.
+         *
+         * \deprecated If this isomorphism is \a iso, then this routine
+         * is equivalent to calling <tt>tri = iso(tri)</tt>.  See the bracket
+         * operator for further details.
+         *
+         * \pre The simplex images are precisely 0,1,...,size()-1 in some
+         * order (i.e., this isomorphism does not represent a mapping from a
+         * smaller triangulation into a larger triangulation).
+         *
+         * \exception InvalidArgument the number of top-dimensional simplices
+         * in the given triangulation is not equal to size() for this
+         * isomorphism.
          *
          * @param tri the triangulation to which this isomorphism
          * should be applied.
          */
-        void applyInPlace(Triangulation<dim>& tri) const;
+        [[deprecated]] void applyInPlace(Triangulation<dim>& tri) const;
 
         /**
          * Returns the composition of this isomorphism with the given
@@ -415,6 +508,103 @@ class Isomorphism :
          */
         Isomorphism inverse() const;
 
+        /**
+         * A preincrement operator that changes this to be the next
+         * isomorphism in an iteration through all possible isomorphisms
+         * of this size.
+         *
+         * The order of iteration is lexicographical, by the sequence of
+         * simplex images and then by the sequence of facet permutations.
+         * Facet permutations, in turn, are ordered by their indices in
+         * the array Perm<dim>::Sn.
+         *
+         * In particular, the identity isomorphism is the first in such an
+         * iteration.  If this isomorphism is the \e last in such an
+         * iteration, then this operator will "wrap around" and set this
+         * to the identity.
+         *
+         * \pre The class Perm<dim+1> supports the preincrement operator;
+         * currently this means that \a dim must be at most 6.
+         *
+         * \ifacespython This routine is named inc() since python does
+         * not support the increment operator.  Unlike other Regina
+         * classes, here inc() wraps the preincrement operator (not the
+         * postincrement operator), since the postincrement operator is
+         * significantly more expensive.  To avoid confusion, the python
+         * inc() function returns \c None (not this isomorphism).
+         *
+         * @return a reference to this isomorphism after the increment.
+         */
+        Isomorphism<dim>& operator ++();
+
+        /**
+         * A postincrement operator that changes this to be the next
+         * isomorphism in an iteration through all possible isomorphisms
+         * of this size.
+         *
+         * The order of iteration is lexicographical, by the sequence of
+         * simplex images and then by the sequence of facet permutations.
+         * Facet permutations, in turn, are ordered by their indices in
+         * the array Perm<dim>::Sn.
+         *
+         * In particular, the identity isomorphism is the first in such an
+         * iteration.  If this isomorphism is the \e last in such an
+         * iteration, then this operator will "wrap around" and set this
+         * to the identity.
+         *
+         * \warning Since the postincrement operator returns an
+         * isomorphism by value, it is significantly more expensive than
+         * the preincrement operator (since it involves a deep copy of a
+         * large object).  You should use the preincrement operator unless
+         * you actually need a copy of the old value of this isomorphism.
+         *
+         * \pre The class Perm<dim+1> supports the preincrement operator;
+         * currently this means that \a dim must be at most 6.
+         *
+         * \ifacespython Not present, although the preincrement operator is
+         * present in python as the member function inc().
+         *
+         * @return a copy of this isomorphism before the increment took place.
+         */
+        Isomorphism<dim> operator ++(int);
+
+        /**
+         * Writes the tight encoding of this isomorphism to the given output
+         * stream.  See the page on \ref tight "tight encodings" for details.
+         *
+         * \ifacespython Not present; use tightEncoding() instead, which
+         * returns a string.
+         *
+         * @param out the output stream to which the encoded string will
+         * be written.
+         */
+        void tightEncode(std::ostream& out) const;
+
+        /**
+         * Reconstructs an isomorphism from its given tight encoding.
+         * See the page on \ref tight "tight encodings" for details.
+         *
+         * The tight encoding will be read from the given input stream.
+         * If the input stream contains leading whitespace then it will be
+         * treated as an invalid encoding (i.e., this routine will throw an
+         * exception).  The input routine \e may contain further data: if this
+         * routine is successful then the input stream will be left positioned
+         * immediately after the encoding, without skipping any trailing
+         * whitespace.
+         *
+         * \exception InvalidInput the given input stream does not begin with
+         * a tight encoding of an isomorphism on <i>dim</i>-dimensional
+         * triangulations.
+         *
+         * \ifacespython Not present; use tightDecoding() instead, which takes
+         * a string as its argument.
+         *
+         * @param input an input stream that begins with the tight encoding
+         * for an isomorphism on <i>dim</i>-dimensional triangulations.
+         * @return the isomorphism represented by the given tight encoding.
+         */
+        static Isomorphism tightDecode(std::istream& input);
+
         /**
          * Writes a short text representation of this object to the
          * given output stream.
@@ -488,7 +678,7 @@ class Isomorphism :
          * isomorphism should operate upon.
          * @return the identity isomorphism.
          */
-        static Isomorphism<dim> identity(unsigned nSimplices);
+        static Isomorphism<dim> identity(size_t nSimplices);
 
         /**
          * Returns a random isomorphism for the given number of simplices.
@@ -511,7 +701,7 @@ class Isomorphism :
          * it will preserve the orientation.
          * @return the new random isomorphism.
          */
-        static Isomorphism<dim> random(unsigned nSimplices, bool even = false);
+        static Isomorphism<dim> random(size_t nSimplices, bool even = false);
 };
 
 /**
@@ -531,24 +721,27 @@ void swap(Isomorphism<dim>& a, Isomorphism<dim>& b) noexcept;
 // Inline functions for Isomorphism
 
 template <int dim>
-inline Isomorphism<dim>::Isomorphism(unsigned nSimplices) :
-        nSimplices_(nSimplices),
-        simpImage_(new int[nSimplices]),
+inline Isomorphism<dim>::Isomorphism(size_t nSimplices) :
+        size_(nSimplices),
+        simpImage_(new ssize_t[nSimplices]),
         facetPerm_(new Perm<dim+1>[nSimplices]) {
 }
 
 template <int dim>
 inline Isomorphism<dim>::Isomorphism(const Isomorphism<dim>& src) :
-        nSimplices_(src.nSimplices_),
-        simpImage_(new int[src.nSimplices_]),
-        facetPerm_(new Perm<dim+1>[src.nSimplices_]) {
-    std::copy(src.simpImage_, src.simpImage_ + nSimplices_, simpImage_);
-    std::copy(src.facetPerm_, src.facetPerm_ + nSimplices_, facetPerm_);
+        size_(src.size_),
+        simpImage_(new ssize_t[src.size_]),
+        facetPerm_(new Perm<dim+1>[src.size_]) {
+    // We use src.size_, not size_, since otherwise gcc12 issues
+    // loud warnings about copying data from an empty range in the case where
+    // src is empty.  (The logic does work, but the compiler doesn't realise).
+    std::copy(src.simpImage_, src.simpImage_ + src.size_, simpImage_);
+    std::copy(src.facetPerm_, src.facetPerm_ + src.size_, facetPerm_);
 }
 
 template <int dim>
 inline Isomorphism<dim>::Isomorphism(Isomorphism<dim>&& src) noexcept:
-        nSimplices_(src.nSimplices_),
+        size_(src.size_),
         simpImage_(src.simpImage_),
         facetPerm_(src.facetPerm_) {
     src.simpImage_ = nullptr;
@@ -567,25 +760,28 @@ Isomorphism<dim>& Isomorphism<dim>::operator = (const Isomorphism<dim>& src) {
     if (std::addressof(src) == this)
         return *this;
 
-    if (nSimplices_ != src.nSimplices_) {
+    if (size_ != src.size_) {
         delete[] simpImage_;
         delete[] facetPerm_;
 
-        nSimplices_ = src.nSimplices_;
+        size_ = src.size_;
 
-        simpImage_ = new int[nSimplices_];
-        facetPerm_ = new Perm<dim+1>[nSimplices_];
+        simpImage_ = new ssize_t[size_];
+        facetPerm_ = new Perm<dim+1>[size_];
     }
 
-    std::copy(src.simpImage_, src.simpImage_ + nSimplices_, simpImage_);
-    std::copy(src.facetPerm_, src.facetPerm_ + nSimplices_, facetPerm_);
+    // We use src.size_, not size_, since otherwise gcc12 issues
+    // loud warnings about copying data from an empty range in the case where
+    // src is empty.  (The logic does work, but the compiler doesn't realise).
+    std::copy(src.simpImage_, src.simpImage_ + src.size_, simpImage_);
+    std::copy(src.facetPerm_, src.facetPerm_ + src.size_, facetPerm_);
     return *this;
 }
 
 template <int dim>
 Isomorphism<dim>& Isomorphism<dim>::operator = (Isomorphism<dim>&& src)
         noexcept{
-    nSimplices_ = src.nSimplices_;
+    size_ = src.size_;
     std::swap(simpImage_, src.simpImage_);
     std::swap(facetPerm_, src.facetPerm_);
     // Let src dispose of the original contents in its own destructor.
@@ -594,33 +790,33 @@ Isomorphism<dim>& Isomorphism<dim>::operator = (Isomorphism<dim>&& src)
 
 template <int dim>
 void Isomorphism<dim>::swap(Isomorphism<dim>& other) noexcept {
-    std::swap(nSimplices_, other.nSimplices_);
+    std::swap(size_, other.size_);
     std::swap(simpImage_, other.simpImage_);
     std::swap(facetPerm_, other.facetPerm_);
 }
 
 template <int dim>
-inline unsigned Isomorphism<dim>::size() const {
-    return nSimplices_;
+inline size_t Isomorphism<dim>::size() const {
+    return size_;
 }
 
 template <int dim>
-inline int& Isomorphism<dim>::simpImage(unsigned sourceSimp) {
+inline ssize_t& Isomorphism<dim>::simpImage(size_t sourceSimp) {
     return simpImage_[sourceSimp];
 }
 
 template <int dim>
-inline int Isomorphism<dim>::simpImage(unsigned sourceSimp) const {
+inline ssize_t Isomorphism<dim>::simpImage(size_t sourceSimp) const {
     return simpImage_[sourceSimp];
 }
 
 template <int dim>
-inline Perm<dim+1>& Isomorphism<dim>::facetPerm(unsigned sourceSimp) {
+inline Perm<dim+1>& Isomorphism<dim>::facetPerm(size_t sourceSimp) {
     return facetPerm_[sourceSimp];
 }
 
 template <int dim>
-inline Perm<dim+1> Isomorphism<dim>::facetPerm(unsigned sourceSimp) const {
+inline Perm<dim+1> Isomorphism<dim>::facetPerm(size_t sourceSimp) const {
     return facetPerm_[sourceSimp];
 }
 
@@ -633,8 +829,8 @@ inline FacetSpec<dim> Isomorphism<dim>::operator [] (
 
 template <int dim>
 bool Isomorphism<dim>::isIdentity() const {
-    for (unsigned p = 0; p < nSimplices_; ++p) {
-        if (simpImage_[p] != p)
+    for (size_t p = 0; p < size_; ++p) {
+        if (simpImage_[p] != static_cast<ssize_t>(p))
             return false;
         if (! facetPerm_[p].isIdentity())
             return false;
@@ -643,39 +839,35 @@ bool Isomorphism<dim>::isIdentity() const {
 }
 
 template <int dim>
-Triangulation<dim> Isomorphism<dim>::apply(
+Triangulation<dim> Isomorphism<dim>::operator ()(
         const Triangulation<dim>& original) const {
-    if (original.size() != nSimplices_)
-        throw InvalidArgument("Isomorphism::apply() was given "
-            "an input triangulation of the wrong size");
+    if (original.size() != size_)
+        throw InvalidArgument("Isomorphism::operator() was given "
+            "a triangulation of the wrong size");
 
-    if (nSimplices_ == 0)
+    if (size_ == 0)
         return Triangulation<dim>();
 
     Triangulation<dim> ans;
-    auto* tet = new Simplex<dim>*[nSimplices_];
-    unsigned long t;
-    int f;
+    auto* tet = new Simplex<dim>*[size_];
 
     // Ensure only one event pair is fired in this sequence of changes.
     typename Triangulation<dim>::ChangeEventSpan span(ans);
-    for (t = 0; t < nSimplices_; t++)
+    for (size_t t = 0; t < size_; t++)
         tet[t] = ans.newSimplex();
 
-    for (t = 0; t < nSimplices_; t++)
+    for (size_t t = 0; t < size_; t++)
         tet[simpImage_[t]]->setDescription(
             original.simplex(t)->description());
 
     const Simplex<dim> *myTet, *adjTet;
-    unsigned long adjTetIndex;
-    Perm<dim+1> gluingPerm;
-    for (t = 0; t < nSimplices_; t++) {
+    for (size_t t = 0; t < size_; t++) {
         myTet = original.simplex(t);
-        for (f = 0; f <= dim; f++)
+        for (int f = 0; f <= dim; f++)
             if ((adjTet = myTet->adjacentSimplex(f))) {
                 // We have an adjacent simplex.
-                adjTetIndex = adjTet->index();
-                gluingPerm = myTet->adjacentGluing(f);
+                size_t adjTetIndex = adjTet->index();
+                Perm<dim+1> gluingPerm = myTet->adjacentGluing(f);
 
                 // Make the gluing from one side only.
                 if (adjTetIndex > t || (adjTetIndex == t &&
@@ -692,15 +884,46 @@ Triangulation<dim> Isomorphism<dim>::apply(
 }
 
 template <int dim>
-void Isomorphism<dim>::applyInPlace(Triangulation<dim>& tri) const {
-    Triangulation<dim> staging = apply(tri);
-    tri.swap(staging);
+inline FacetSpec<dim> Isomorphism<dim>::operator ()(const FacetSpec<dim>& f)
+        const {
+    if (f.simp >= 0 && f.simp < static_cast<ssize_t>(size_)) {
+        return FacetSpec<dim>(simpImage_[f.simp], facetPerm_[f.simp][f.facet]);
+    } else {
+        // Past-the-end or before-the-start values should be left alone.
+        return f;
+    }
+}
+
+template <int dim>
+FacetPairing<dim> Isomorphism<dim>::operator ()(
+        const FacetPairing<dim>& p) const {
+    if (p.size() != size_)
+        throw InvalidArgument("Isomorphism::operator() was given "
+            "a facet pairing of the wrong size");
+
+    FacetPairing<dim> ans(size_);
+
+    for (FacetSpec<dim> f(0, 0); f.simp < static_cast<ssize_t>(size_); ++f)
+        ans.dest((*this)(f)) = (*this)(p[f]);
+
+    return ans;
+}
+
+template <int dim>
+inline Triangulation<dim> Isomorphism<dim>::apply(
+        const Triangulation<dim>& original) const {
+    return (*this)(original);
+}
+
+template <int dim>
+inline void Isomorphism<dim>::applyInPlace(Triangulation<dim>& tri) const {
+    tri = (*this)(tri);
 }
 
 template <int dim>
 Isomorphism<dim> Isomorphism<dim>::operator * (const Isomorphism& rhs) const {
-    Isomorphism<dim> ans(rhs.nSimplices_);
-    for (unsigned i = 0; i < rhs.nSimplices_; ++i) {
+    Isomorphism<dim> ans(rhs.size_);
+    for (size_t i = 0; i < rhs.size_; ++i) {
         ans.simpImage_[i] = simpImage_[rhs.simpImage_[i]];
         ans.facetPerm_[i] = facetPerm_[rhs.simpImage_[i]] * rhs.facetPerm_[i];
     }
@@ -710,7 +933,7 @@ Isomorphism<dim> Isomorphism<dim>::operator * (const Isomorphism& rhs) const {
 template <int dim>
 Isomorphism<dim> Isomorphism<dim>::operator * (Isomorphism&& rhs) const {
     // We will construct the result by overwriting rhs.
-    for (unsigned i = 0; i < rhs.nSimplices_; ++i) {
+    for (size_t i = 0; i < rhs.size_; ++i) {
         rhs.facetPerm_[i] = facetPerm_[rhs.simpImage_[i]] * rhs.facetPerm_[i];
         rhs.simpImage_[i] = simpImage_[rhs.simpImage_[i]];
     }
@@ -719,17 +942,97 @@ Isomorphism<dim> Isomorphism<dim>::operator * (Isomorphism&& rhs) const {
 
 template <int dim>
 Isomorphism<dim> Isomorphism<dim>::inverse() const {
-    Isomorphism<dim> ans(nSimplices_);
-    for (unsigned i = 0; i < nSimplices_; ++i) {
+    Isomorphism<dim> ans(size_);
+    for (size_t i = 0; i < size_; ++i) {
         ans.simpImage_[simpImage_[i]] = i;
         ans.facetPerm_[simpImage_[i]] = facetPerm_[i].inverse();
     }
     return ans;
 }
 
+template <int dim>
+Isomorphism<dim>& Isomorphism<dim>::operator ++() {
+    static_assert(dim <= 6,
+        "Currently the Isomorphism<dim> pre/postincrement operators "
+        "are only available for dimensions dim <= 6.");
+
+    if (size_ == 0)
+        return *this;
+
+    for (ssize_t i = size_ - 1; i >= 0; --i)
+        if (! (++facetPerm_[i]).isIdentity())
+            return *this;
+
+    std::next_permutation(simpImage_, simpImage_ + size_);
+    return *this;
+}
+
+template <int dim>
+inline Isomorphism<dim> Isomorphism<dim>::operator ++(int) {
+    Isomorphism<dim> prev(*this);
+    ++(*this);
+    return prev;
+}
+
+template <int dim>
+void Isomorphism<dim>::tightEncode(std::ostream& out) const {
+    regina::detail::tightEncodeIndex(out, size_);
+    for (size_t i = 0; i < size_; ++i)
+        regina::detail::tightEncodeIndex(out, simpImage_[i]);
+    if constexpr (dim == 2) {
+        // We can fit two permutations per character here.
+        for (size_t i = 0; i < size_; i += 2) {
+            if (i + 1 == size_)
+                regina::detail::tightEncodeIndex(out, static_cast<unsigned>(
+                    facetPerm_[i].SnIndex()));
+            else
+                regina::detail::tightEncodeIndex(out, static_cast<unsigned>(
+                    facetPerm_[i].SnIndex() + 6 * facetPerm_[i+1].SnIndex()));
+        }
+    } else {
+        for (size_t i = 0; i < size_; ++i)
+            facetPerm_[i].tightEncode(out);
+    }
+}
+
+template <int dim>
+Isomorphism<dim> Isomorphism<dim>::tightDecode(std::istream& input) {
+    auto n = regina::detail::tightDecodeIndex<size_t>(input);
+    Isomorphism ans(n);
+
+    // We don't check the values of simpImage_[...], since we want to
+    // support the negative "unknown" placeholder value for simpImage_[...].
+    // Perm<dim+1>::tightDecode() will check the validity of the permutations
+    // that are read.
+    for (size_t i = 0; i < n; ++i)
+        ans.simpImage_[i] = regina::detail::tightDecodeIndex<ssize_t>(input);
+    if constexpr (dim == 2) {
+        for (size_t i = 0; i < n; i += 2) {
+            auto p = regina::detail::tightDecodeIndex<unsigned>(input);
+            if (i + 1 == n) {
+                if (p >= 6)
+                    throw InvalidInput("The tight encoding contains "
+                        "invalid permutations");
+                ans.facetPerm_[i] = Perm<dim+1>::Sn[p];
+            } else {
+                if (p >= 36)
+                    throw InvalidInput("The tight encoding contains "
+                        "invalid permutations");
+                ans.facetPerm_[i] = Perm<dim+1>::Sn[p % 6];
+                ans.facetPerm_[i+1] = Perm<dim+1>::Sn[p / 6];
+            }
+        }
+    } else {
+        for (size_t i = 0; i < n; ++i)
+            ans.facetPerm_[i] = Perm<dim+1>::tightDecode(input);
+    }
+
+    return ans;
+}
+
 template <int dim>
 inline void Isomorphism<dim>::writeTextShort(std::ostream& out) const {
-    for (unsigned i = 0; i < nSimplices_; ++i) {
+    for (size_t i = 0; i < size_; ++i) {
         if (i > 0)
             out << ", ";
         out << i << " -> " << simpImage_[i] << " (" << facetPerm_[i] << ')';
@@ -738,15 +1041,15 @@ inline void Isomorphism<dim>::writeTextShort(std::ostream& out) const {
 
 template <int dim>
 inline void Isomorphism<dim>::writeTextLong(std::ostream& out) const {
-    for (unsigned i = 0; i < nSimplices_; ++i)
+    for (size_t i = 0; i < size_; ++i)
         out << i << " -> " << simpImage_[i] << " (" << facetPerm_[i] << ")\n";
 }
 
 template <int dim>
 inline bool Isomorphism<dim>::operator == (const Isomorphism& other) const {
-    return nSimplices_ == other.nSimplices_ &&
-        std::equal(simpImage_, simpImage_ + nSimplices_, other.simpImage_) &&
-        std::equal(facetPerm_, facetPerm_ + nSimplices_, other.facetPerm_);
+    return size_ == other.size_ &&
+        std::equal(simpImage_, simpImage_ + size_, other.simpImage_) &&
+        std::equal(facetPerm_, facetPerm_ + size_, other.facetPerm_);
 }
 
 template <int dim>
@@ -755,19 +1058,19 @@ inline bool Isomorphism<dim>::operator != (const Isomorphism& other) const {
 }
 
 template <int dim>
-inline Isomorphism<dim> Isomorphism<dim>::identity(unsigned nSimplices) {
+inline Isomorphism<dim> Isomorphism<dim>::identity(size_t nSimplices) {
     Isomorphism<dim> id(nSimplices);
-    for (unsigned i = 0; i < nSimplices; ++i)
+    for (size_t i = 0; i < nSimplices; ++i)
         id.simpImage_[i] = i;
     return id;
 }
 
 template <int dim>
-Isomorphism<dim> Isomorphism<dim>::random(unsigned nSimplices, bool even) {
+Isomorphism<dim> Isomorphism<dim>::random(size_t nSimplices, bool even) {
     Isomorphism<dim> ans(nSimplices);
 
     // Prepare the destination simplices.
-    unsigned i;
+    size_t i;
     for (i = 0; i < nSimplices; ++i)
         ans.simpImage_[i] = i;
 
