@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -299,16 +299,6 @@ class Triangulation : public detail::TriangulationBase<dim> {
          * swapped with this.
          */
         void swap(Triangulation<dim>& other);
-        /**
-         * Deprecated routine that swaps the contents of this and the
-         * given triangulation.
-         *
-         * \deprecated Use swap() instead.
-         *
-         * @param other the triangulation whose contents should be
-         * swapped with this.
-         */
-        [[deprecated]] void swapContents(Triangulation<dim>& other);
 
         /*@}*/
 
@@ -498,11 +488,6 @@ void Triangulation<dim>::swap(Triangulation<dim>& other) {
     this->swapBaseData(other);
 }
 
-template <int dim>
-inline void Triangulation<dim>::swapContents(Triangulation<dim>& other) {
-    swap(other);
-}
-
 // Inline functions for DegreeLessThan / DegreeGreaterThan
 
 template <int dim, int subdim>
