@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -496,7 +496,7 @@ class Bitmask {
          *
          * @return the index of the first \c true bit.
          */
-        long firstBit() const;
+        ssize_t firstBit() const;
 
         /**
          * Returns the index of the last \c true bit in this bitmask,
@@ -504,7 +504,7 @@ class Bitmask {
          *
          * @return the index of the last \c true bit.
          */
-        long lastBit() const;
+        ssize_t lastBit() const;
 
         /**
          * Determines whether at most one bit is set to \c true in this
@@ -939,7 +939,7 @@ class Bitmask1 {
          *
          * @return the index of the first \c true bit.
          */
-        inline long firstBit() const {
+        inline ssize_t firstBit() const {
             return BitManipulator<T>::firstBit(mask);
         }
 
@@ -949,7 +949,7 @@ class Bitmask1 {
          *
          * @return the index of the last \c true bit.
          */
-        inline long lastBit() const {
+        inline ssize_t lastBit() const {
             return BitManipulator<T>::lastBit(mask);
         }
 
@@ -1400,7 +1400,7 @@ class Bitmask2 {
          *
          * @return the index of the first \c true bit.
          */
-        inline long firstBit() const {
+        inline ssize_t firstBit() const {
             // -1 case does not work out of the box in the second IF branch
             // due to the 8 * sizeof(T).
             if (low)
@@ -1417,7 +1417,7 @@ class Bitmask2 {
          *
          * @return the index of the last \c true bit.
          */
-        inline long lastBit() const {
+        inline ssize_t lastBit() const {
             // -1 case works out of the box in the second IF branch.
             if (high)
                 return 8 * sizeof(T) + BitManipulator<U>::lastBit(high);
@@ -1444,86 +1444,6 @@ class Bitmask2 {
         const Bitmask2<T, U>& mask);
 };
 
-/**
- * A deprecated type alias that gives a small and extremely fast bitmask
- * class capable of holding at least 8 true-or-false bits.
- *
- * This bitmask class is guaranteed to be an instantiation of the
- * template class Bitmask1.
- *
- * The particular instantiation is subject to change between different
- * platforms, different compilers and/or different versions of Regina.
- *
- * \deprecated This type alias is deprecated; just use Bitmask1<uint8_t
- * instead.
- *
- * \ifacespython Not present, but in Python you can access essentially
- * the same optimised bitmask class via the name Bitmask8.
- *
- * \ingroup utilities
- */
-using BitmaskLen8 [[deprecated]] = Bitmask1<uint8_t>;
-
-/**
- * A deprecated type alias that gives a small and extremely fast bitmask
- * class capable of holding at least 16 true-or-false bits.
- *
- * This bitmask class is guaranteed to be an instantiation of the
- * template class Bitmask1.
- *
- * The particular instantiation is subject to change between different
- * platforms, different compilers and/or different versions of Regina.
- *
- * \deprecated This type alias is deprecated; just use Bitmask1<uint16_t>
- * instead.
- *
- * \ifacespython Not present, but in Python you can access essentially
- * the same optimised bitmask class via the name Bitmask16.
- *
- * \ingroup utilities
- */
-using BitmaskLen16 [[deprecated]] = Bitmask1<uint16_t>;
-
-/**
- * A deprecated type alias that gives a small and extremely fast bitmask
- * class capable of holding at least 32 true-or-false bits.
- *
- * This bitmask class is guaranteed to be an instantiation of the
- * template class Bitmask1.
- *
- * The particular instantiation is subject to change between different
- * platforms, different compilers and/or different versions of Regina.
- *
- * \deprecated This type alias is deprecated; just use Bitmask1<uint32_t>
- * instead.
- *
- * \ifacespython Not present, but in Python you can access essentially
- * the same optimised bitmask class via the name Bitmask32.
- *
- * \ingroup utilities
- */
-using BitmaskLen32 [[deprecated]] = Bitmask1<uint32_t>;
-
-/**
- * A deprecated type alias that gives a small and extremely fast bitmask
- * class capable of holding at least 64 true-or-false bits.
- *
- * This bitmask class is guaranteed to be an instantiation of
- * \e either the template class Bitmask1 or the template class Bitmask2.
- *
- * The particular instantiation is subject to change between different
- * platforms, different compilers and/or different versions of Regina.
- *
- * \deprecated This type alias is deprecated; just use Bitmask1<uint64_t>
- * instead.
- *
- * \ifacespython Not present, but in Python you can access essentially
- * the same optimised bitmask class via the name Bitmask64.
- *
- * \ingroup utilities
- */
-using BitmaskLen64 [[deprecated]] = Bitmask1<uint64_t>;
-
 // Inline functions for Bitmask
 
 inline Bitmask::Bitmask() : pieces(0), mask(nullptr) {
@@ -1656,7 +1576,7 @@ inline bool Bitmask::operator != (const Bitmask& other) const {
 }
 
 inline bool Bitmask::lessThan(const Bitmask& other) const {
-    for (long i = pieces - 1; i >= 0; --i)
+    for (ssize_t i = pieces - 1; i >= 0; --i)
         if (mask[i] < other.mask[i])
             return true;
         else if (mask[i] > other.mask[i])
@@ -1692,7 +1612,7 @@ inline size_t Bitmask::bits() const {
     return ans;
 }
 
-inline long Bitmask::firstBit() const {
+inline ssize_t Bitmask::firstBit() const {
     for (size_t i = 0; i < pieces; ++i)
         if (mask[i])
             return 8 * sizeof(Piece) * i +
@@ -1700,8 +1620,8 @@ inline long Bitmask::firstBit() const {
     return -1;
 }
 
-inline long Bitmask::lastBit() const {
-    for (long i = pieces - 1; i >= 0; --i)
+inline ssize_t Bitmask::lastBit() const {
+    for (ssize_t i = pieces - 1; i >= 0; --i)
         if (mask[i])
             return 8 * sizeof(Piece) * i +
                 BitManipulator<Piece>::lastBit(mask[i]);
