@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -66,32 +66,6 @@ class BoolSet {
         static constexpr char stringCodes[4][3] = { "--", "T-", "-F", "TF" };
             /**< The string codes for all four boolean sets. */
 
-    public:
-        /**
-         * The empty set.
-         *
-         * \deprecated Just use BoolSet() instead.
-         */
-        static const BoolSet sNone [[deprecated]];
-        /**
-         * The set containing only \c true.
-         *
-         * \deprecated Just use \c true instead.
-         */
-        static const BoolSet sTrue [[deprecated]];
-        /**
-         * The set containing only \c false.
-         *
-         * \deprecated Just use \c false instead.
-         */
-        static const BoolSet sFalse [[deprecated]];
-        /**
-         * The set containing both \c true and \c false.
-         *
-         * \deprecated Just use BoolSet(true, true) instead.
-         */
-        static const BoolSet sBoth [[deprecated]];
-
     public:
         /**
          * Creates a new empty set.
