@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -150,8 +150,10 @@ class Qitmask1 {
          * @return the value of the (\a index)th qit; this will be
          * either 0, 1, 2 or 3.
          */
-        inline uint8_t get(unsigned index) const {
-            T bit = T(1) << index;
+        inline uint8_t get(size_t index) const {
+            // We cast back down to T because, if T is smaller than int, then
+            // the shift operator promotes up to an int.
+            T bit = static_cast<T>(T(1) << index);
             return ((mask1 & bit) ? 1 : 0) | ((mask2 & bit) ? 2 : 0);
         }
 
@@ -163,7 +165,7 @@ class Qitmask1 {
          * @param value the value that will be assigned to the (\a index)th
          * qit; this must be 0, 1, 2 or 3.
          */
-        inline void set(unsigned index, uint8_t value) {
+        inline void set(size_t index, uint8_t value) {
             mask1 |= (T(1) << index);
             if (! (value & 1))
                 mask1 ^= (T(1) << index);
@@ -386,7 +388,7 @@ class Qitmask2 {
          * @return the value of the (\a index)th qit; this will be
          * either 0, 1, 2 or 3.
          */
-        inline uint8_t get(unsigned index) const {
+        inline uint8_t get(size_t index) const {
             if (index < 8 * sizeof(T)) {
                 T bit = T(1) << index;
                 return ((low1 & bit) ? 1 : 0) | ((low2 & bit) ? 2 : 0);
@@ -404,7 +406,7 @@ class Qitmask2 {
          * @param value the value that will be assigned to the (\a index)th
          * qit; this must be 0, 1, 2 or 3.
          */
-        inline void set(unsigned index, uint8_t value) {
+        inline void set(size_t index, uint8_t value) {
             if (index < 8 * sizeof(T)) {
                 low1 |= (T(1) << index);
                 if (! (value & 1))
@@ -531,86 +533,6 @@ class Qitmask2 {
         const Qitmask2<T, U>& mask);
 };
 
-/**
- * A deprecated type alias that gives a small and extremely fast qitmask
- * class capable of holding at least 8 true-or-false bits.
- *
- * This qitmask class is guaranteed to be an instantiation of the
- * template class Qitmask1.
- *
- * The particular instantiation is subject to change between different
- * platforms, different compilers and/or different versions of Regina.
- *
- * \deprecated This type alias is deprecated; just use Qitmask1<uint8_t>
- * instead.
- *
- * \ifacespython Not present, but in Python you can access essentially
- * the same optimised qitmask class via the name Qitmask8.
- *
- * \ingroup utilities
- */
-using QitmaskLen8 [[deprecated]] = Qitmask1<uint8_t>;
-
-/**
- * A deprecated type alias that gives a small and extremely fast qitmask
- * class capable of holding at least 16 true-or-false bits.
- *
- * This qitmask class is guaranteed to be an instantiation of the
- * template class Qitmask1.
- *
- * The particular instantiation is subject to change between different
- * platforms, different compilers and/or different versions of Regina.
- *
- * \deprecated This type alias is deprecated; just use Qitmask1<uint16_t>
- * instead.
- *
- * \ifacespython Not present, but in Python you can access essentially
- * the same optimised qitmask class via the name Qitmask16.
- *
- * \ingroup utilities
- */
-using QitmaskLen16 [[deprecated]] = Qitmask1<uint16_t>;
-
-/**
- * A deprecated type alias that gives a small and extremely fast qitmask
- * class capable of holding at least 32 true-or-false bits.
- *
- * This qitmask class is guaranteed to be an instantiation of the
- * template class Qitmask1.
- *
- * The particular instantiation is subject to change between different
- * platforms, different compilers and/or different versions of Regina.
- *
- * \deprecated This type alias is deprecated; just use Qitmask1<uint32_t>
- * instead.
- *
- * \ifacespython Not present, but in Python you can access essentially
- * the same optimised qitmask class via the name Qitmask32.
- *
- * \ingroup utilities
- */
-using QitmaskLen32 [[deprecated]] = Qitmask1<uint32_t>;
-
-/**
- * A deprecated type alias that gives a small and extremely fast qitmask
- * class capable of holding at least 64 true-or-false bits.
- *
- * This qitmask class is guaranteed to be an instantiation of
- * \e either the template class Qitmask1 or the template class Qitmask2.
- *
- * The particular instantiation is subject to change between different
- * platforms, different compilers and/or different versions of Regina.
- *
- * \deprecated This type alias is deprecated; just use Qitmask1<uint64_t>
- * instead.
- *
- * \ifacespython Not present, but in Python you can access essentially
- * the same optimised qitmask class via the name Qitmask64.
- *
- * \ingroup utilities
- */
-using QitmaskLen64 [[deprecated]] = Qitmask1<uint64_t>;
-
 } // namespace regina
 
 #endif
