@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Computational Engine                                                  *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -210,7 +210,7 @@ class SnapshotWriteError : public std::exception {
  */
 template <class T>
 class Snapshot {
-    static_assert(std::is_base_of<Snapshottable<T>, T>::value,
+    static_assert(std::is_base_of_v<Snapshottable<T>, T>,
         "Snapshot<T> requires T to be derived from Snapshottable<T>.");
 
     private:
