@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -46,9 +46,9 @@ void addAbelianGroup(pybind11::module_& m) {
     auto c = pybind11::class_<AbelianGroup>(m, "AbelianGroup")
         .def(pybind11::init<>())
         .def(pybind11::init<const AbelianGroup&>())
-        .def(pybind11::init<unsigned>())
-        .def(pybind11::init<unsigned, const std::vector<int>&>())
-        .def(pybind11::init<unsigned, const std::vector<Integer>&>())
+        .def(pybind11::init<size_t>())
+        .def(pybind11::init<size_t, const std::vector<int>&>())
+        .def(pybind11::init<size_t, const std::vector<Integer>&>())
         .def(pybind11::init<MatrixInt>())
         .def(pybind11::init<MatrixInt, MatrixInt>())
         .def(pybind11::init<MatrixInt, MatrixInt, const Integer&>())
@@ -56,31 +56,6 @@ void addAbelianGroup(pybind11::module_& m) {
         .def("addRank", &AbelianGroup::addRank,
             pybind11::arg("extraRank") = 1)
         .def("addTorsion", &AbelianGroup::addTorsion)
-        .def("addTorsionElement", // deprecated
-                [](AbelianGroup& g, const regina::Integer& d, unsigned m) {
-            for ( ; m > 0; --m)
-                g.addTorsion(d);
-        }, pybind11::arg(), pybind11::arg("mult") = 1)
-        .def("addTorsionElement", // deprecated
-                [](AbelianGroup& g, unsigned long d, unsigned m) {
-            for ( ; m > 0; --m)
-                g.addTorsion(d);
-        }, pybind11::arg(), pybind11::arg("mult") = 1)
-        .def("addTorsionElements", [](AbelianGroup& g, pybind11::list l) {
-            // This routine is now deprecated.
-            // Reimplement it using addTorsionElement() instead.
-            for (auto item : l) {
-                // Accept any type that we know how to convert to Integer.
-                // This includes (at least) regina::Integer, python integers
-                // (both int and long), and strings.
-                try {
-                    g.addTorsion(item.cast<regina::Integer>());
-                } catch (pybind11::cast_error const &) {
-                    throw regina::InvalidArgument(
-                        "List element not convertible to Integer");
-                }
-            }
-        })
         .def("addGroup", overload_cast<MatrixInt>(
             &AbelianGroup::addGroup))
         .def("addGroup", overload_cast<const AbelianGroup&>(
@@ -97,13 +72,13 @@ void addAbelianGroup(pybind11::module_& m) {
         .def("isFree", &AbelianGroup::isFree)
         .def("isZn", &AbelianGroup::isZn)
         .def("utf8", &AbelianGroup::utf8)
-        .def("tightEncoding", &AbelianGroup::tightEncoding)
         .def("writeXMLData", [](const AbelianGroup& g, pybind11::object file) {
             pybind11::scoped_ostream_redirect stream(std::cout, file);
             g.writeXMLData(std::cout);
         })
     ;
     regina::python::add_output(c);
+    regina::python::add_tight_encoding(c);
     regina::python::add_eq_operators(c);
 
     m.def("swap", (void(*)(AbelianGroup&, AbelianGroup&))(regina::swap));
