@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -62,11 +62,6 @@ void addAngleStructure(pybind11::module_& m) {
             }
             return new AngleStructure(t, std::move(v));
         }))
-        .def("clone", [](const AngleStructure& a) {
-            // Since clone() is deprecated, we reimplement it here to
-            // avoid noisy compiler warnings.
-            return new AngleStructure(a);
-        })
         .def("swap", &AngleStructure::swap)
         .def("angle", &AngleStructure::angle)
         .def("triangulation", &AngleStructure::triangulation,
@@ -76,8 +71,6 @@ void addAngleStructure(pybind11::module_& m) {
         .def("isVeering", &AngleStructure::isVeering)
         .def("vector", &AngleStructure::vector,
             pybind11::return_value_policy::reference_internal)
-        .def("rawVector", &AngleStructure::vector, // deprecated
-            pybind11::return_value_policy::reference_internal)
         .def("writeXMLData", [](const AngleStructure& s,
                 pybind11::object file) {
             pybind11::scoped_ostream_redirect stream(std::cout, file);
