@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -36,6 +36,7 @@
 #include "../helpers.h"
 #include "algebra/grouppresentation.h"
 #include "triangulation/dim2.h"
+#include "triangulation/facetpairing.h"
 #include "triangulation/isosigtype.h"
 #include "triangulation/detail/isosig-impl.h"
 #include "../generic/facehelper.h"
@@ -98,7 +99,6 @@ void addTriangulation2(pybind11::module_& m) {
         .def("removeAllTriangles", &Triangulation<2>::removeAllTriangles)
         .def("removeAllSimplices", &Triangulation<2>::removeAllSimplices)
         .def("swap", &Triangulation<2>::swap)
-        .def("swapContents", &Triangulation<2>::swap) // deprecated
         .def("moveContentsTo", &Triangulation<2>::moveContentsTo)
         .def("countComponents", &Triangulation<2>::countComponents)
         .def("countBoundaryComponents",
@@ -130,7 +130,11 @@ void addTriangulation2(pybind11::module_& m) {
             pybind11::return_value_policy::reference_internal)
         .def("edge", &Triangulation<2>::edge,
             pybind11::return_value_policy::reference_internal)
-        .def("isIdenticalTo", &Triangulation<2>::operator ==) // deprecated
+        .def("translate", &Triangulation<2>::translate<0>,
+            pybind11::return_value_policy::reference_internal)
+        .def("translate", &Triangulation<2>::translate<1>,
+            pybind11::return_value_policy::reference_internal)
+        .def("pairing", &Triangulation<2>::pairing)
         .def("isIsomorphicTo", &Triangulation<2>::isIsomorphicTo)
         .def("findAllIsomorphisms", &Triangulation<2>::findAllIsomorphisms<
                 const std::function<bool(const Isomorphism<2>)>&>)
@@ -165,10 +169,14 @@ void addTriangulation2(pybind11::module_& m) {
         .def("hasBoundaryEdges", &Triangulation<2>::hasBoundaryEdges)
         .def("countBoundaryFacets", &Triangulation<2>::countBoundaryFacets)
         .def("countBoundaryEdges", &Triangulation<2>::countBoundaryEdges)
+        .def("countBoundaryFaces", (size_t (Triangulation<2>::*)(int) const)(
+            &Triangulation<2>::countBoundaryFaces))
         .def("isOrientable", &Triangulation<2>::isOrientable)
         .def("isOriented", &Triangulation<2>::isOriented)
         .def("isIdeal", &Triangulation<2>::isIdeal)
         .def("isConnected", &Triangulation<2>::isConnected)
+        .def("fundamentalGroup", &Triangulation<2>::fundamentalGroup,
+            pybind11::return_value_policy::reference_internal)
         .def("isMinimal", &Triangulation<2>::isMinimal)
         .def("orient", &Triangulation<2>::orient)
         .def("reflect", &Triangulation<2>::reflect)
@@ -177,13 +185,16 @@ void addTriangulation2(pybind11::module_& m) {
             (regina::AbelianGroup (Triangulation<2>::*)(int) const)(
             &Triangulation<2>::homology),
             pybind11::arg("k") = 1)
-        .def("homologyH1", &Triangulation<2>::homology<1>) // deprecated
         .def("markedHomology",
             (regina::MarkedAbelianGroup (Triangulation<2>::*)(int) const)(
             &Triangulation<2>::markedHomology),
             pybind11::arg("k") = 1)
         .def("boundaryMap", (MatrixInt (Triangulation<2>::*)(int) const)(
             &Triangulation<2>::boundaryMap))
+        .def("dualBoundaryMap", (MatrixInt (Triangulation<2>::*)(int) const)(
+            &Triangulation<2>::dualBoundaryMap))
+        .def("dualToPrimal", (MatrixInt (Triangulation<2>::*)(int) const)(
+            &Triangulation<2>::dualToPrimal))
         .def("pachner", &Triangulation<2>::pachner<2>,
             pybind11::arg(),
             pybind11::arg("check") = true,
@@ -219,6 +230,7 @@ void addTriangulation2(pybind11::module_& m) {
         .def_readonly_static("dimension", &Triangulation<2>::dimension)
     ;
     regina::python::add_output(c);
+    regina::python::add_tight_encoding(c);
     regina::python::packet_eq_operators(c);
     regina::python::add_packet_data(c);
 
