@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -42,12 +42,14 @@
 #include "snappea/snappeatriangulation.h"
 #include "surface/normalsurface.h"
 #include "triangulation/example3.h"
+#include "triangulation/facetpairing3.h"
 #include "triangulation/isosigtype.h"
 #include "triangulation/detail/isosig-impl.h"
 #include "../generic/facehelper.h"
 #include "../generic/isosig-bindings.h"
 
 using pybind11::overload_cast;
+using regina::python::GILCallbackManager;
 using regina::Example;
 using regina::Isomorphism;
 using regina::MatrixInt;
@@ -157,7 +159,6 @@ void addTriangulation3(pybind11::module_& m) {
         .def("removeAllTetrahedra", &Triangulation<3>::removeAllTetrahedra)
         .def("removeAllSimplices", &Triangulation<3>::removeAllSimplices)
         .def("swap", &Triangulation<3>::swap)
-        .def("swapContents", &Triangulation<3>::swap) // deprecated
         .def("moveContentsTo", &Triangulation<3>::moveContentsTo)
         .def("countComponents", &Triangulation<3>::countComponents)
         .def("countBoundaryComponents",
@@ -196,7 +197,13 @@ void addTriangulation3(pybind11::module_& m) {
         .def("triangle", (regina::Face<3, 2>* (Triangulation<3>::*)(size_t))(
             &Triangulation<3>::triangle),
             pybind11::return_value_policy::reference_internal)
-        .def("isIdenticalTo", &Triangulation<3>::operator ==) // deprecated
+        .def("translate", &Triangulation<3>::translate<0>,
+            pybind11::return_value_policy::reference_internal)
+        .def("translate", &Triangulation<3>::translate<1>,
+            pybind11::return_value_policy::reference_internal)
+        .def("translate", &Triangulation<3>::translate<2>,
+            pybind11::return_value_policy::reference_internal)
+        .def("pairing", &Triangulation<3>::pairing)
         .def("isIsomorphicTo", &Triangulation<3>::isIsomorphicTo)
         .def("findAllIsomorphisms", &Triangulation<3>::findAllIsomorphisms<
                 const std::function<bool(const Isomorphism<3>)>&>)
@@ -226,6 +233,8 @@ void addTriangulation3(pybind11::module_& m) {
             &Triangulation<3>::hasTwoSphereBoundaryComponents)
         .def("hasNegativeIdealBoundaryComponents",
             &Triangulation<3>::hasNegativeIdealBoundaryComponents)
+        .def("hasMinimalBoundary", &Triangulation<3>::hasMinimalBoundary)
+        .def("hasMinimalVertices", &Triangulation<3>::hasMinimalVertices)
         .def("isEmpty", &Triangulation<3>::isEmpty)
         .def("eulerCharTri", &Triangulation<3>::eulerCharTri)
         .def("eulerCharManifold", &Triangulation<3>::eulerCharManifold)
@@ -236,6 +245,8 @@ void addTriangulation3(pybind11::module_& m) {
         .def("hasBoundaryTriangles", &Triangulation<3>::hasBoundaryTriangles)
         .def("countBoundaryFacets", &Triangulation<3>::countBoundaryFacets)
         .def("countBoundaryTriangles", &Triangulation<3>::countBoundaryTriangles)
+        .def("countBoundaryFaces", (size_t (Triangulation<3>::*)(int) const)(
+            &Triangulation<3>::countBoundaryFaces))
         .def("isClosed", &Triangulation<3>::isClosed)
         .def("isOrientable", &Triangulation<3>::isOrientable)
         .def("isOriented", &Triangulation<3>::isOriented)
@@ -249,8 +260,6 @@ void addTriangulation3(pybind11::module_& m) {
             (regina::AbelianGroup (Triangulation<3>::*)(int) const)(
             &Triangulation<3>::homology),
             pybind11::arg("k") = 1)
-        .def("homologyH1", &Triangulation<3>::homology<1>) // deprecated
-        .def("homologyH2", &Triangulation<3>::homology<2>) // deprecated
         .def("homologyRel", &Triangulation<3>::homologyRel,
             pybind11::return_value_policy::reference_internal)
         .def("homologyBdry", &Triangulation<3>::homologyBdry,
@@ -262,37 +271,40 @@ void addTriangulation3(pybind11::module_& m) {
             pybind11::arg("k") = 1)
         .def("boundaryMap", (MatrixInt (Triangulation<3>::*)(int) const)(
             &Triangulation<3>::boundaryMap))
+        .def("dualBoundaryMap", (MatrixInt (Triangulation<3>::*)(int) const)(
+            &Triangulation<3>::dualBoundaryMap))
+        .def("dualToPrimal", (MatrixInt (Triangulation<3>::*)(int) const)(
+            &Triangulation<3>::dualToPrimal))
         .def("turaevViro", &Triangulation<3>::turaevViro,
             pybind11::arg(), pybind11::arg("parity") = true,
             pybind11::arg("alg") = regina::ALG_DEFAULT,
             pybind11::arg("tracker") = nullptr,
-            pybind11::call_guard<pybind11::gil_scoped_release>())
+            pybind11::call_guard<regina::python::GILScopedRelease>())
         .def("turaevViroApprox", &Triangulation<3>::turaevViroApprox,
             pybind11::arg(), pybind11::arg("whichRoot") = 1,
             pybind11::arg("alg") = regina::ALG_DEFAULT)
         .def("allCalculatedTuraevViro",
             &Triangulation<3>::allCalculatedTuraevViro)
+        .def("longitudeCuts", &Triangulation<3>::longitudeCuts)
         .def("longitude", &Triangulation<3>::longitude,
             pybind11::return_value_policy::reference_internal)
+        .def("meridian", &Triangulation<3>::meridian,
+            pybind11::return_value_policy::reference_internal)
         .def("meridianLongitude", &Triangulation<3>::meridianLongitude,
             pybind11::return_value_policy::reference_internal)
+        .def("linkingSurface", &Triangulation<3>::linkingSurface<0>)
+        .def("linkingSurface", &Triangulation<3>::linkingSurface<1>)
+        .def("linkingSurface", &Triangulation<3>::linkingSurface<2>)
         .def("isZeroEfficient", &Triangulation<3>::isZeroEfficient)
         .def("knowsZeroEfficient", &Triangulation<3>::knowsZeroEfficient)
         .def("hasSplittingSurface", &Triangulation<3>::hasSplittingSurface)
         .def("nonTrivialSphereOrDisc",
             &Triangulation<3>::nonTrivialSphereOrDisc)
-        .def("hasNonTrivialSphereOrDisc", // deprecated
-            &Triangulation<3>::nonTrivialSphereOrDisc)
         .def("octagonalAlmostNormalSphere",
             &Triangulation<3>::octagonalAlmostNormalSphere)
-        .def("hasOctagonalAlmostNormalSphere", // deprecated
-            &Triangulation<3>::octagonalAlmostNormalSphere)
         .def("strictAngleStructure",
             &Triangulation<3>::strictAngleStructure,
             pybind11::return_value_policy::reference_internal)
-        .def("findStrictAngleStructure", // deprecated
-            &Triangulation<3>::strictAngleStructure,
-            pybind11::return_value_policy::reference_internal)
         .def("hasStrictAngleStructure",
             &Triangulation<3>::hasStrictAngleStructure)
         .def("knowsStrictAngleStructure",
@@ -317,23 +329,25 @@ void addTriangulation3(pybind11::module_& m) {
             pybind11::arg("height") = 1,
             pybind11::arg("nThreads") = 1,
             pybind11::arg("tracker") = nullptr,
-            pybind11::call_guard<pybind11::gil_scoped_release>())
+            pybind11::call_guard<regina::python::GILScopedRelease>())
         .def("retriangulate", [](const Triangulation<3>& tri, int height,
                 int threads, const std::function<bool(const std::string&,
                     Triangulation<3>&&)>& action) {
             if (threads == 1) {
                 return tri.retriangulate(height, 1, nullptr, action);
             } else {
-                pybind11::gil_scoped_release release;
+                GILCallbackManager manager;
                 return tri.retriangulate(height, threads, nullptr,
                     [&](const std::string& sig, Triangulation<3>&& t) -> bool {
-                        pybind11::gil_scoped_acquire acquire;
+                        GILCallbackManager<>::ScopedAcquire acquire(manager);
                         return action(sig, std::move(t));
                     });
             }
         })
         .def("minimiseBoundary", &Triangulation<3>::minimiseBoundary)
         .def("minimizeBoundary", &Triangulation<3>::minimizeBoundary)
+        .def("minimiseVertices", &Triangulation<3>::minimiseVertices)
+        .def("minimizeVertices", &Triangulation<3>::minimizeVertices)
         .def("pachner", &Triangulation<3>::pachner<3>,
             pybind11::arg(),
             pybind11::arg("check") = true,
@@ -373,34 +387,33 @@ void addTriangulation3(pybind11::module_& m) {
             pybind11::arg("check") = true,
             pybind11::arg("perform") = true)
         .def("zeroTwoMove",
-            overload_cast< regina::EdgeEmbedding<3>, int,
-            regina::EdgeEmbedding<3>, int, bool, bool >(
-                &Triangulation<3>::zeroTwoMove ),
+            overload_cast<regina::EdgeEmbedding<3>, int,
+                regina::EdgeEmbedding<3>, int, bool, bool>(
+                &Triangulation<3>::zeroTwoMove),
             pybind11::arg(),
             pybind11::arg(),
             pybind11::arg(),
             pybind11::arg(),
             pybind11::arg("check") = true,
-            pybind11::arg("perform") = true )
+            pybind11::arg("perform") = true)
         .def("zeroTwoMove",
-            overload_cast< regina::Edge<3>*, size_t, size_t,
-            bool, bool >(
-                &Triangulation<3>::zeroTwoMove ),
+            overload_cast<regina::Edge<3>*, size_t, size_t, bool, bool>(
+                &Triangulation<3>::zeroTwoMove),
             pybind11::arg(),
             pybind11::arg(),
             pybind11::arg(),
             pybind11::arg("check") = true,
-            pybind11::arg("perform") = true )
+            pybind11::arg("perform") = true)
         .def("zeroTwoMove",
-            overload_cast< regina::Triangle<3>*, int,
-            regina::Triangle<3>*, int, bool, bool >(
-                &Triangulation<3>::zeroTwoMove ),
+            overload_cast<regina::Triangle<3>*, int,
+                regina::Triangle<3>*, int, bool, bool>(
+                &Triangulation<3>::zeroTwoMove),
             pybind11::arg(),
             pybind11::arg(),
             pybind11::arg(),
             pybind11::arg(),
             pybind11::arg("check") = true,
-            pybind11::arg("perform") = true )
+            pybind11::arg("perform") = true)
         .def("openBook", &Triangulation<3>::openBook,
             pybind11::arg(),
             pybind11::arg("check") = true,
@@ -426,13 +439,13 @@ void addTriangulation3(pybind11::module_& m) {
         .def("triangulateComponents", &Triangulation<3>::triangulateComponents)
         .def("summands", &Triangulation<3>::summands)
         .def("isSphere", &Triangulation<3>::isSphere)
-        .def("isThreeSphere", &Triangulation<3>::isSphere) // deprecated
         .def("knowsSphere", &Triangulation<3>::knowsSphere)
-        .def("knowsThreeSphere", &Triangulation<3>::knowsSphere) // deprecated
         .def("isBall", &Triangulation<3>::isBall)
         .def("knowsBall", &Triangulation<3>::knowsBall)
         .def("isSolidTorus", &Triangulation<3>::isSolidTorus)
         .def("knowsSolidTorus", &Triangulation<3>::knowsSolidTorus)
+        .def("isHandlebody", &Triangulation<3>::isHandlebody)
+        .def("knowsHandlebody", &Triangulation<3>::knowsHandlebody)
         .def("isTxI", &Triangulation<3>::isTxI)
         .def("knowsTxI", &Triangulation<3>::knowsTxI)
         .def("isIrreducible", &Triangulation<3>::isIrreducible)
@@ -467,32 +480,8 @@ void addTriangulation3(pybind11::module_& m) {
         .def("insertLayeredSolidTorus",
             &Triangulation<3>::insertLayeredSolidTorus,
             pybind11::return_value_policy::reference)
-        .def("insertLayeredLensSpace", // deprecated
-                [](Triangulation<3>& t, size_t p, size_t q) {
-            t.insertTriangulation(Example<3>::lens(p, q));
-        })
-        .def("insertLayeredLoop", // deprecated
-                [](Triangulation<3>& t, size_t len, bool twisted) {
-            t.insertTriangulation(Example<3>::layeredLoop(len, twisted));
-        })
-        .def("insertAugTriSolidTorus", // deprecated
-                [](Triangulation<3>& t, long a1, long a2, long b1, long b2,
-                    long c1, long c2) {
-            t.insertTriangulation(Example<3>::augTriSolidTorus(
-                a1, a2, b1, b2, c1, c2));
-        })
-        .def("insertSFSOverSphere", // deprecated
-                [](Triangulation<3>& t, long a1, long a2, long b1, long b2,
-                    long c1, long c2) {
-            t.insertTriangulation(Example<3>::sfsOverSphere(
-                a1, a2, b1, b2, c1, c2));
-        })
         .def("connectedSumWith", &Triangulation<3>::connectedSumWith)
         .def("insertTriangulation", &Triangulation<3>::insertTriangulation)
-        .def("insertRehydration", [](Triangulation<3>& tri,
-                const std::string& str) { // deprecated
-            tri.insertTriangulation(Triangulation<3>::rehydrate(str));
-        })
         .def("dehydrate", &Triangulation<3>::dehydrate)
         .def_static("rehydrate", &Triangulation<3>::rehydrate)
         .def("isoSig", &Triangulation<3>::isoSig<>)
@@ -524,13 +513,10 @@ void addTriangulation3(pybind11::module_& m) {
         .def("saveRecogniser", &Triangulation<3>::saveRecogniser)
         .def("saveRecognizer", &Triangulation<3>::saveRecognizer)
         .def_static("fromSnapPea", &Triangulation<3>::fromSnapPea)
-        .def_static("enterTextTriangulation", []() { // deprecated
-            return Triangulation<3>::enterTextTriangulation(
-                std::cin, std::cout);
-        })
         .def_readonly_static("dimension", &Triangulation<3>::dimension)
     ;
     regina::python::add_output(c);
+    regina::python::add_tight_encoding(c);
     regina::python::packet_eq_operators(c);
     regina::python::add_packet_data(c);
 
@@ -546,10 +532,9 @@ void addTriangulation3(pybind11::module_& m) {
         m, "PacketOfTriangulation3");
     regina::python::add_packet_constructor<>(wrap);
     regina::python::add_packet_constructor<const Triangulation<3>&, bool>(wrap);
+    regina::python::add_packet_constructor<const regina::Link&, bool>(wrap,
+        pybind11::arg(), pybind11::arg("simplify") = true);
     regina::python::add_packet_constructor<const std::string&>(wrap);
-    wrap.def(pybind11::init([](const regina::Link& link) { // deprecated
-        return regina::make_packet<Triangulation<3>>(link.complement());
-    }));
     wrap.def(pybind11::init([](const regina::python::SnapPyObject& obj) {
         return regina::make_packet<Triangulation<3>>(std::in_place,
             obj.string_);
