@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -35,14 +35,17 @@
 #include "../pybind11/stl.h"
 #include "../helpers.h"
 #include "algebra/grouppresentation.h"
+#include "algebra/intersectionform.h"
 #include "progress/progresstracker.h"
 #include "triangulation/dim4.h"
+#include "triangulation/facetpairing.h"
 #include "triangulation/isosigtype.h"
 #include "triangulation/detail/isosig-impl.h"
 #include "../generic/facehelper.h"
 #include "../generic/isosig-bindings.h"
 
 using pybind11::overload_cast;
+using regina::python::GILCallbackManager;
 using regina::Isomorphism;
 using regina::MatrixInt;
 using regina::Triangulation;
@@ -99,7 +102,6 @@ void addTriangulation4(pybind11::module_& m) {
         .def("removeAllPentachora", &Triangulation<4>::removeAllPentachora)
         .def("removeAllSimplices", &Triangulation<4>::removeAllSimplices)
         .def("swap", &Triangulation<4>::swap)
-        .def("swapContents", &Triangulation<4>::swap) // deprecated
         .def("moveContentsTo", &Triangulation<4>::moveContentsTo)
         .def("countComponents", &Triangulation<4>::countComponents)
         .def("countBoundaryComponents",
@@ -144,7 +146,15 @@ void addTriangulation4(pybind11::module_& m) {
         .def("tetrahedron", (regina::Face<4, 3>* (Triangulation<4>::*)(size_t))(
             &Triangulation<4>::tetrahedron),
             pybind11::return_value_policy::reference_internal)
-        .def("isIdenticalTo", &Triangulation<4>::operator ==) // deprecated
+        .def("translate", &Triangulation<4>::translate<0>,
+            pybind11::return_value_policy::reference_internal)
+        .def("translate", &Triangulation<4>::translate<1>,
+            pybind11::return_value_policy::reference_internal)
+        .def("translate", &Triangulation<4>::translate<2>,
+            pybind11::return_value_policy::reference_internal)
+        .def("translate", &Triangulation<4>::translate<3>,
+            pybind11::return_value_policy::reference_internal)
+        .def("pairing", &Triangulation<4>::pairing)
         .def("isIsomorphicTo", &Triangulation<4>::isIsomorphicTo)
         .def("makeCanonical", &Triangulation<4>::makeCanonical)
         .def("isContainedIn", &Triangulation<4>::isContainedIn)
@@ -180,6 +190,8 @@ void addTriangulation4(pybind11::module_& m) {
         .def("countBoundaryFacets", &Triangulation<4>::countBoundaryFacets)
         .def("countBoundaryTetrahedra",
             &Triangulation<4>::countBoundaryTetrahedra)
+        .def("countBoundaryFaces", (size_t (Triangulation<4>::*)(int) const)(
+            &Triangulation<4>::countBoundaryFaces))
         .def("isClosed", &Triangulation<4>::isClosed)
         .def("isOrientable", &Triangulation<4>::isOrientable)
         .def("isOriented", &Triangulation<4>::isOriented)
@@ -192,14 +204,17 @@ void addTriangulation4(pybind11::module_& m) {
             (regina::AbelianGroup (Triangulation<4>::*)(int) const)(
             &Triangulation<4>::homology),
             pybind11::arg("k") = 1)
-        .def("homologyH1", &Triangulation<4>::homology<1>) // deprecated
-        .def("homologyH2", &Triangulation<4>::homology<2>) // deprecated
         .def("markedHomology",
             (regina::MarkedAbelianGroup (Triangulation<4>::*)(int) const)(
             &Triangulation<4>::markedHomology),
             pybind11::arg("k") = 1)
         .def("boundaryMap", (MatrixInt (Triangulation<4>::*)(int) const)(
             &Triangulation<4>::boundaryMap))
+        .def("dualBoundaryMap", (MatrixInt (Triangulation<4>::*)(int) const)(
+            &Triangulation<4>::dualBoundaryMap))
+        .def("dualToPrimal", (MatrixInt (Triangulation<4>::*)(int) const)(
+            &Triangulation<4>::dualToPrimal))
+        .def("intersectionForm", &Triangulation<4>::intersectionForm)
         .def("orient", &Triangulation<4>::orient)
         .def("reflect", &Triangulation<4>::reflect)
         .def("triangulateComponents", &Triangulation<4>::triangulateComponents)
@@ -211,17 +226,17 @@ void addTriangulation4(pybind11::module_& m) {
             pybind11::arg("height") = 1,
             pybind11::arg("nThreads") = 1,
             pybind11::arg("tracker") = nullptr,
-            pybind11::call_guard<pybind11::gil_scoped_release>())
+            pybind11::call_guard<regina::python::GILScopedRelease>())
         .def("retriangulate", [](const Triangulation<4>& tri, int height,
                 int threads, const std::function<bool(const std::string&,
                     Triangulation<4>&&)>& action) {
             if (threads == 1) {
                 return tri.retriangulate(height, 1, nullptr, action);
             } else {
-                pybind11::gil_scoped_release release;
+                GILCallbackManager manager;
                 return tri.retriangulate(height, threads, nullptr,
                     [&](const std::string& sig, Triangulation<4>&& t) -> bool {
-                        pybind11::gil_scoped_acquire acquire;
+                        GILCallbackManager<>::ScopedAcquire acquire(manager);
                         return action(sig, std::move(t));
                     });
             }
@@ -289,6 +304,10 @@ void addTriangulation4(pybind11::module_& m) {
             pybind11::arg(),
             pybind11::arg("check") = true,
             pybind11::arg("perform") = true )
+        .def("linkingSurface", &Triangulation<4>::linkingSurface<0>)
+        .def("linkingSurface", &Triangulation<4>::linkingSurface<1>)
+        .def("linkingSurface", &Triangulation<4>::linkingSurface<2>)
+        .def("linkingSurface", &Triangulation<4>::linkingSurface<3>)
         .def("insertTriangulation", &Triangulation<4>::insertTriangulation)
         .def("isoSig", &Triangulation<4>::isoSig<>)
         .def("isoSig_EdgeDegrees",
@@ -308,6 +327,7 @@ void addTriangulation4(pybind11::module_& m) {
         .def_readonly_static("dimension", &Triangulation<4>::dimension)
     ;
     regina::python::add_output(c);
+    regina::python::add_tight_encoding(c);
     regina::python::packet_eq_operators(c);
     regina::python::add_packet_data(c);
 
