@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -36,20 +36,29 @@
 #include "enumerate/doubledescription.h"
 #include "../helpers.h"
 
+using regina::python::GILCallbackManager;
 using regina::DoubleDescription;
 using regina::VectorInt;
 
 void addDoubleDescription(pybind11::module_& m) {
     auto c = pybind11::class_<DoubleDescription>(m, "DoubleDescription")
-        .def_static("enumerate", &DoubleDescription::enumerate<VectorInt,
-                const std::function<void(VectorInt&&)>&>,
+        .def_static("enumerate", [](
+                const std::function<void(VectorInt&&)>& action,
+                const regina::MatrixInt& s,
+                const regina::ValidityConstraints& c,
+                regina::ProgressTracker* p, size_t r) {
+            GILCallbackManager<false> manager;
+            DoubleDescription::enumerate<VectorInt>([&](VectorInt&& v) {
+                GILCallbackManager<false>::ScopedAcquire acquire(manager);
+                action(std::move(v));
+            }, s, c, p, r);
+        },
             pybind11::arg(), pybind11::arg(), pybind11::arg(),
             pybind11::arg("tracker") = nullptr,
-            pybind11::arg("initialRows") = 0,
-            pybind11::call_guard<pybind11::gil_scoped_release>())
+            pybind11::arg("initialRows") = 0)
         .def_static("enumerate", [](const regina::MatrixInt& s,
                 const regina::ValidityConstraints& c,
-                regina::ProgressTracker* p, unsigned long r) {
+                regina::ProgressTracker* p, size_t r) {
             std::vector<VectorInt> ans;
             DoubleDescription::enumerate<VectorInt>([&ans](VectorInt&& v) {
                 ans.push_back(std::move(v));
@@ -58,7 +67,7 @@ void addDoubleDescription(pybind11::module_& m) {
         }, pybind11::arg(), pybind11::arg(),
             pybind11::arg("tracker") = nullptr,
             pybind11::arg("initialRows") = 0,
-            pybind11::call_guard<pybind11::gil_scoped_release>())
+            pybind11::call_guard<regina::python::GILScopedRelease>())
     ;
     regina::python::no_eq_operators(c);
 }
