@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -43,7 +43,7 @@ template <int dim>
 void addFacetSpec(pybind11::module_& m, const char* name) {
     auto c = pybind11::class_<FacetSpec<dim>>(m, name)
         .def(pybind11::init<>())
-        .def(pybind11::init<int, int>())
+        .def(pybind11::init<ssize_t, int>())
         .def(pybind11::init<const FacetSpec<dim>&>())
         .def_readwrite("simp", &FacetSpec<dim>::simp)
         .def_readwrite("facet", &FacetSpec<dim>::facet)
@@ -64,6 +64,7 @@ void addFacetSpec(pybind11::module_& m, const char* name) {
         .def(pybind11::self <= pybind11::self)
     ;
     regina::python::add_output_ostream(c);
+    regina::python::add_tight_encoding(c);
     regina::python::add_eq_operators(c);
 }
 
