@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -33,6 +33,7 @@
 #include "../pybind11/pybind11.h"
 #include "../pybind11/operators.h"
 #include "triangulation/generic.h"
+#include "triangulation/facetpairing.h"
 #include "../helpers.h"
 
 using pybind11::overload_cast;
@@ -42,31 +43,47 @@ template <int dim>
 void addIsomorphism(pybind11::module_& m, const char* name) {
     auto c = pybind11::class_<Isomorphism<dim>>(m, name)
         .def(pybind11::init<const Isomorphism<dim>&>())
-        .def(pybind11::init<unsigned>())
+        .def(pybind11::init<size_t>())
         .def("swap", &Isomorphism<dim>::swap)
         .def("size", &Isomorphism<dim>::size)
-        .def("simpImage", overload_cast<unsigned>(
+        .def("simpImage", overload_cast<size_t>(
             &Isomorphism<dim>::simpImage, pybind11::const_))
-        .def("setSimpImage", [](Isomorphism<dim>& iso, unsigned s, int image) {
+        .def("setSimpImage", [](Isomorphism<dim>& iso, size_t s, ssize_t image) {
             iso.simpImage(s) = image;
         })
-        .def("facetPerm", overload_cast<unsigned>(
+        .def("facetPerm", overload_cast<size_t>(
             &Isomorphism<dim>::facetPerm, pybind11::const_))
-        .def("setFacetPerm", [](Isomorphism<dim>& iso, unsigned s,
+        .def("setFacetPerm", [](Isomorphism<dim>& iso, size_t s,
                 regina::Perm<dim+1> p) {
             iso.facetPerm(s) = p;
         })
         .def("__getitem__", &Isomorphism<dim>::operator[])
         .def("isIdentity", &Isomorphism<dim>::isIdentity)
-        .def("apply", &Isomorphism<dim>::apply)
-        .def("applyInPlace", &Isomorphism<dim>::applyInPlace)
+        .def("__call__", overload_cast<const regina::Triangulation<dim>&>(
+            &Isomorphism<dim>::operator(), pybind11::const_))
+        .def("__call__", overload_cast<const regina::FacetSpec<dim>&>(
+            &Isomorphism<dim>::operator(), pybind11::const_))
+        .def("__call__", overload_cast<const regina::FacetPairing<dim>&>(
+            &Isomorphism<dim>::operator(), pybind11::const_))
+        .def("apply", overload_cast<const regina::Triangulation<dim>&>(
+            &Isomorphism<dim>::operator(), pybind11::const_)) // deprecated
+        .def("applyInPlace", // deprecated
+                [](const Isomorphism<dim>& iso, regina::Triangulation<dim>& t) {
+            t = iso(t);
+        })
         .def(pybind11::self * pybind11::self)
         .def("inverse", &Isomorphism<dim>::inverse)
         .def_static("random", &Isomorphism<dim>::random,
             pybind11::arg(), pybind11::arg("even") = false)
         .def_static("identity", &Isomorphism<dim>::identity)
     ;
+    if constexpr (dim <= 6) {
+        c.def("inc", [](Isomorphism<dim>& iso) {
+            ++iso;
+        });
+    }
     regina::python::add_output(c);
+    regina::python::add_tight_encoding(c);
     regina::python::add_eq_operators(c);
 
     m.def("swap",
