@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -43,8 +43,12 @@
 #define __PYTHONINTERPRETER_H
 
 #include <Python.h>
+#include <iostream>
+#include <memory>
 #include <mutex>
 #include <string>
+#include <thread>
+#include "utilities/exception.h"
 
 namespace regina {
     class Packet;
@@ -133,6 +137,9 @@ class PythonInterpreter {
         static bool pythonInitialised;
             /**< Has the python system been initialised yet? */
 
+        std::thread::id thread;
+            /**< The ID of the C++ thread to be used exclusively with this
+                 subinterpreter. */
         PyThreadState* state;
             /**< The first thread state created in this particular
                  subinterpreter. */
@@ -162,11 +169,31 @@ class PythonInterpreter {
 
     public:
         /**
-         * Constructor and destructor.
+         * Constructor.
+         *
+         * In the constructor, if \a fixPythonPath is \c true then:
+         *
+         * - if Regina ships its own copy of Python, then this constructor
+         *   will adjust the Python path so that Regina's copy of the core
+         *   Python libraries can be found;
+         *
+         * - if necessary, this constructor will also adjust the Python path
+         *   so that Regina's own module can be found.
+         *
+         * If \a fixPythonPath is \c false then the Python path will not be
+         * touched at all.
          */
         PythonInterpreter(
             regina::python::PythonOutputStream& pyStdOut,
-            regina::python::PythonOutputStream& pyStdErr);
+            regina::python::PythonOutputStream& pyStdErr,
+            bool fixPythonPath = true);
+
+        /**
+         * Destructor.
+         *
+         * \pre This is called from the same C++ thread that created
+         * this Python interpreter.
+         */
         ~PythonInterpreter();
 
         /**
@@ -177,6 +204,9 @@ class PythonInterpreter {
          * This is intended for use in an interactive Python session.
          *
          * You should always test exitAttempted() after executing user code.
+         *
+         * \pre This is called from the same C++ thread that created
+         * this Python interpreter.
          */
         bool executeLine(const std::string& command);
 
@@ -186,15 +216,24 @@ class PythonInterpreter {
          * This function also sets up a completer object, for use with
          * complete().
          *
-         * Returns \c true on success or \c false on failure.
+         * If \a fixPythonPath is \c true, then this routine will (before the
+         * import) adjust the Python path so that Regina's module can be found.
+         *
+         * \pre This is called from the same C++ thread that created
+         * this Python interpreter.
+         *
+         * @return \c true on success or \c false on failure.
          */
-        bool importRegina();
+        bool importRegina(bool fixPythonPath = true);
 
         /**
          * Set the given variable in Python's main namespace to
          * represent the given Regina packet.
          *
-         * Returns \c true on success or \c false on failure.
+         * \pre This is called from the same C++ thread that created
+         * this Python interpreter.
+         *
+         * @return \c true on success or \c false on failure.
          */
         bool setVar(const char* name, std::shared_ptr<Packet> value);
 
@@ -207,7 +246,10 @@ class PythonInterpreter {
          *
          * You should always test exitAttempted() after executing user code.
          *
-         * Returns \c true on success or \c false on failure.
+         * \pre This is called from the same C++ thread that created
+         * this Python interpreter.
+         *
+         * @return \c true on success or \c false on failure.
          */
         bool runScript(const regina::Script* script);
 
@@ -245,6 +287,9 @@ class PythonInterpreter {
          * (i) \a completer returns \c false to signify that no more
          * completions are required, or (ii) no more completions can be found.
          *
+         * \pre This is called from the same C++ thread that created
+         * this Python interpreter.
+         *
          * @param text the Python text to complete.
          * @param completer the callback object that will receive the
          * resulting completions (if any).
@@ -257,8 +302,23 @@ class PythonInterpreter {
         int complete(const std::string& text, PythonCompleter& completer);
 
     private:
+        /**
+         * Ensures that the current C++ thread is in fact the thread that
+         * created this subinterpreter.
+         *
+         * When this function is called, it is typically verifying a
+         * precondition to some other member function of PythonInterpreter.
+         * Therefore the behaviour when this check \e fails is subject to
+         * change in future versions of Regina (since in theory, it should
+         * never happen).
+         */
+        void verifyThread() const;
+
         /**
          * Run the given Python code in Python's main namespace.
+         *
+         * \pre This is called from the same C++ thread that created
+         * this Python interpreter.
          */
         bool runCode(const char* code);
 
@@ -292,9 +352,59 @@ class PythonInterpreter {
          * that is referenced by the current python thread state, to which
          * the given namespace must belong.
          *
+         * If \a fixPythonPath is \c true, then this routine will (before the
+         * import) adjust the Python path so that Regina's module can be found.
+         *
          * Returns \c true on success or \c false on failure.
          */
-        static bool importReginaIntoNamespace(PyObject* useNamespace);
+        static bool importReginaIntoNamespace(PyObject* useNamespace,
+            bool fixPythonPath = true);
+
+        /**
+         * An object that calls PyEval_RestoreThread() on construction
+         * and PyEval_SaveThread() on destruction.  This manages both
+         * (i) the Python thread state for the subinterpreter being used,
+         * and also (ii) the Python global interpreter lock.
+         *
+         * This class insists that the same C++ that created the Python
+         * subinterpreter is also used for both construction and destruction
+         * of the ScopedThreadRestore object.
+         *
+         * An object of this type cannot be copied or moved.
+         */
+        class ScopedThreadRestore {
+            private:
+                PythonInterpreter& interpreter_;
+                    /**< The Python subinterpreter being used. */
+
+            public:
+                /**
+                 * Restores the primary Python thread for the given Python
+                 * subinterpreter.
+                 *
+                 * \pre This constructor is called from the same C++ thread
+                 * that created the given interpreter.
+                 */
+                ScopedThreadRestore(PythonInterpreter& interpreter) :
+                        interpreter_(interpreter) {
+                    interpreter.verifyThread();
+                    PyEval_RestoreThread(interpreter.state);
+                }
+
+                /**
+                 * Saves the current Python thread.
+                 *
+                 * \pre This destructor is called from the same C++ thread
+                 * as the corresponding constructor.
+                 */
+                ~ScopedThreadRestore() {
+                    interpreter_.state = PyEval_SaveThread();
+                }
+
+                ScopedThreadRestore(const ScopedThreadRestore&) = delete;
+                ScopedThreadRestore& operator = (const ScopedThreadRestore&)
+                    = delete;
+        };
 };
 
 inline PrefixCompleter::PrefixCompleter() : initialised_(false) {
@@ -308,6 +418,19 @@ inline bool PythonInterpreter::exitAttempted() const {
     return caughtSystemExit;
 }
 
+inline void PythonInterpreter::verifyThread() const {
+    if (std::this_thread::get_id() != thread) {
+        /*
+        std::cerr << "ERROR: A PythonInterpreter was used from the wrong "
+            "C++ thread.\n\n"
+            "Please report this to the Regina developers!\n"
+            << std::endl;
+        */
+        throw regina::FailedPrecondition(
+            "PythonInterpreter used from the wrong C++ thread");
+    }
+}
+
 } // namespace regina::python
 
 #endif
