@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -241,9 +241,9 @@ namespace add_eq_operators_detail {
         using IneqType = decltype(makeRef() != makeRef());
 
         static constexpr bool hasEqOperator =
-            ! std::is_same<void, EqType>::value;
+            ! std::is_same_v<void, EqType>;
         static constexpr bool hasIneqOperator =
-            ! std::is_same<void, IneqType>::value;
+            ! std::is_same_v<void, IneqType>;
     };
 
     /**
