@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -58,27 +58,9 @@ void addNormalHypersurfaces(pybind11::module_& m) {
             pybind11::arg("which") = regina::HS_LIST_DEFAULT,
             pybind11::arg("algHints") = regina::HS_ALG_DEFAULT,
             pybind11::arg("tracker") = nullptr,
-            pybind11::call_guard<pybind11::gil_scoped_release>())
+            pybind11::call_guard<GILScopedRelease>())
         .def(pybind11::init<const NormalHypersurfaces&>())
         .def("swap", &NormalHypersurfaces::swap)
-        .def_static("enumerate", [](Triangulation<4>& owner,
-                HyperCoords coords, regina::HyperList which,
-                regina::HyperAlg algHints) {
-            // This is deprecated, so we reimplement it here ourselves.
-            // This means we can't use the progress tracker variant, which
-            // requires threading code internal to NormalHypersurfaces.
-            try {
-                auto ans = regina::make_packet<NormalHypersurfaces>(
-                    std::in_place, owner, coords, which, algHints);
-                if (auto p = owner.packet())
-                    p->insertChildLast(ans);
-                return ans;
-            } catch (const regina::ReginaException&) {
-                return std::shared_ptr<regina::PacketOf<NormalHypersurfaces>>();
-            }
-        }, pybind11::arg(), pybind11::arg(),
-            pybind11::arg("which") = regina::HS_LIST_DEFAULT,
-            pybind11::arg("algHints") = regina::HS_ALG_DEFAULT)
         .def("sort", &NormalHypersurfaces::sort<const std::function<
             bool(const NormalHypersurface&, const NormalHypersurface&)>&>)
         .def("recreateMatchingEquations",
@@ -92,6 +74,8 @@ void addNormalHypersurfaces(pybind11::module_& m) {
         .def("size", &NormalHypersurfaces::size)
         .def("hypersurface", &NormalHypersurfaces::hypersurface,
             pybind11::return_value_policy::reference_internal)
+        .def("__getitem__", &NormalHypersurfaces::operator[],
+            pybind11::return_value_policy::reference_internal)
         .def("__iter__", [](const NormalHypersurfaces& list) {
             return pybind11::make_iterator(list);
         }, pybind11::keep_alive<0, 1>()) // iterator keeps list alive
@@ -112,7 +96,7 @@ void addNormalHypersurfaces(pybind11::module_& m) {
         pybind11::arg("which") = regina::HS_LIST_DEFAULT,
         pybind11::arg("algHints") = regina::HS_ALG_DEFAULT,
         pybind11::arg("tracker") = nullptr,
-        pybind11::call_guard<pybind11::gil_scoped_release>());
+        pybind11::call_guard<GILScopedRelease>());
 
     m.def("swap",
         (void(*)(NormalHypersurfaces&, NormalHypersurfaces&))(regina::swap));
