@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -40,6 +40,7 @@
 #include "../helpers.h"
 
 using pybind11::overload_cast;
+using regina::python::GILCallbackManager;
 using regina::Crossing;
 using regina::StrandRef;
 using regina::Link;
@@ -68,6 +69,7 @@ void addLink(pybind11::module_& m) {
         .def("next", &StrandRef::next)
         .def("prev", &StrandRef::prev)
         .def("jump", &StrandRef::jump)
+        .def("__bool__", &StrandRef::operator bool)
     ;
     regina::python::add_output_ostream(s);
     regina::python::add_eq_operators(s);
@@ -104,6 +106,7 @@ void addLink(pybind11::module_& m) {
             pybind11::keep_alive<0, 1>())
         .def("strand", &Link::strand)
         .def("translate", &Link::translate)
+        .def("graph", &Link::graph)
         // In the following overloads, we define functions twice because
         // overload_cast gets confused between templated/non-templated variants.
         // Also: the versions that take a std::vector must come *last*,
@@ -177,7 +180,6 @@ void addLink(pybind11::module_& m) {
         .def_static("fromKnotSig", &Link::fromKnotSig)
         .def_static("fromSig", &Link::fromSig)
         .def("swap", &Link::swap)
-        .def("swapContents", &Link::swap) // deprecated
         .def("reflect", &Link::reflect)
         .def("rotate", &Link::rotate)
         .def("reverse", &Link::reverse)
@@ -201,27 +203,27 @@ void addLink(pybind11::module_& m) {
             pybind11::return_value_policy::reference_internal,
             pybind11::arg("alg") = regina::ALG_DEFAULT,
             pybind11::arg("tracker") = nullptr,
-            pybind11::call_guard<pybind11::gil_scoped_release>())
+            pybind11::call_guard<regina::python::GILScopedRelease>())
         .def("jones", &Link::jones,
             pybind11::return_value_policy::reference_internal,
             pybind11::arg("alg") = regina::ALG_DEFAULT,
             pybind11::arg("tracker") = nullptr,
-            pybind11::call_guard<pybind11::gil_scoped_release>())
+            pybind11::call_guard<regina::python::GILScopedRelease>())
         .def("homfly", &Link::homfly,
             pybind11::return_value_policy::reference_internal,
             pybind11::arg("alg") = regina::ALG_DEFAULT,
             pybind11::arg("tracker") = nullptr,
-            pybind11::call_guard<pybind11::gil_scoped_release>())
+            pybind11::call_guard<regina::python::GILScopedRelease>())
         .def("homflyAZ", &Link::homflyAZ,
             pybind11::return_value_policy::reference_internal,
             pybind11::arg("alg") = regina::ALG_DEFAULT,
             pybind11::arg("tracker") = nullptr,
-            pybind11::call_guard<pybind11::gil_scoped_release>())
+            pybind11::call_guard<regina::python::GILScopedRelease>())
         .def("homflyLM", &Link::homflyLM,
             pybind11::return_value_policy::reference_internal,
             pybind11::arg("alg") = regina::ALG_DEFAULT,
             pybind11::arg("tracker") = nullptr,
-            pybind11::call_guard<pybind11::gil_scoped_release>())
+            pybind11::call_guard<regina::python::GILScopedRelease>())
         .def("knowsBracket", &Link::knowsBracket)
         .def("knowsJones", &Link::knowsJones)
         .def("knowsHomfly", &Link::knowsHomfly)
@@ -299,16 +301,16 @@ void addLink(pybind11::module_& m) {
             pybind11::arg("height") = 1,
             pybind11::arg("nThreads") = 1,
             pybind11::arg("tracker") = nullptr,
-            pybind11::call_guard<pybind11::gil_scoped_release>())
+            pybind11::call_guard<regina::python::GILScopedRelease>())
         .def("rewrite", [](const Link& link, int height, int threads,
                 const std::function<bool(const std::string&, Link&&)>& action) {
             if (threads == 1) {
                 return link.rewrite(height, 1, nullptr, action);
             } else {
-                pybind11::gil_scoped_release release;
+                GILCallbackManager manager;
                 return link.rewrite(height, threads, nullptr,
                     [&](const std::string& sig, Link&& link) -> bool {
-                        pybind11::gil_scoped_acquire acquire;
+                        GILCallbackManager<>::ScopedAcquire acquire(manager);
                         return action(sig, std::move(link));
                     });
             }
@@ -326,6 +328,7 @@ void addLink(pybind11::module_& m) {
         .def_readonly_static("homflyLMVarY", Link::homflyLMVarY)
     ;
     regina::python::add_output(l);
+    regina::python::add_tight_encoding(l);
     regina::python::packet_eq_operators(l);
     regina::python::add_packet_data(l);
 
