@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -59,6 +59,7 @@ void addModelLinkGraph(pybind11::module_& m) {
         .def("dec", [](ModelLinkGraphArc& a) {
            return a--;
         })
+        .def("__bool__", &ModelLinkGraphArc::operator bool)
     ;
     regina::python::add_output_ostream(a);
     regina::python::add_eq_operators(a);
@@ -73,6 +74,7 @@ void addModelLinkGraph(pybind11::module_& m) {
 
     auto g = pybind11::class_<ModelLinkGraph>(m, "ModelLinkGraph")
         .def(pybind11::init<>())
+        .def(pybind11::init<const regina::Link&>())
         .def(pybind11::init<const ModelLinkGraph&>())
         .def("size", &ModelLinkGraph::size)
         .def("node", &ModelLinkGraph::node,
@@ -80,7 +82,6 @@ void addModelLinkGraph(pybind11::module_& m) {
         .def("nodes", &ModelLinkGraph::nodes,
             pybind11::keep_alive<0, 1>())
         .def("swap", &ModelLinkGraph::swap)
-        .def("swapContents", &ModelLinkGraph::swap) // deprecated
         .def("reflect", &ModelLinkGraph::reflect)
         .def("cells", &ModelLinkGraph::cells,
             pybind11::return_value_policy::reference_internal)
