@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -43,10 +43,6 @@ void addHandlebody(pybind11::module_& m) {
         .def(pybind11::init<const Handlebody&>())
         .def("swap", &Handlebody::swap)
         .def("genus", &Handlebody::genus)
-        .def("handles", &Handlebody::genus) // deprecated
-        .def("isOrientable", [](const Handlebody&) { // deprecated
-            return true;
-        })
     ;
     regina::python::add_eq_operators(c);
     regina::python::add_output(c);
