@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -33,13 +33,14 @@
 #include "../pybind11/pybind11.h"
 #include "../pybind11/stl.h"
 #include "maths/numbertheory.h"
+#include <numeric>
 
 void addNumberTheory(pybind11::module_& m) {
     m.def("reducedMod", regina::reducedMod);
-    m.def("gcd", regina::gcd);
+    m.def("gcd", std::gcd<long, long>); // deprecated
     m.def("gcdWithCoeffs", pybind11::overload_cast<long, long>(
         &regina::gcdWithCoeffs));
-    m.def("lcm", regina::lcm);
+    m.def("lcm", std::lcm<long, long>); // deprecated
     m.def("modularInverse", regina::modularInverse);
 }
 
