@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -84,12 +84,12 @@ namespace {
     };
 
     template <int n>
-    ConstArray<decltype(Perm<n>::Sn)> Perm_Sn_arr(
+    ConstArray<decltype(Perm<n>::Sn), typename Perm<n>::Index> Perm_Sn_arr(
         Perm<n>::Sn, Perm<n>::nPerms);
 
     template <int n>
-    ConstArray<decltype(Perm<n>::orderedSn)> Perm_orderedSn_arr(
-        Perm<n>::orderedSn, Perm<n>::nPerms);
+    ConstArray<decltype(Perm<n>::orderedSn), typename Perm<n>::Index>
+        Perm_orderedSn_arr(Perm<n>::orderedSn, Perm<n>::nPerms);
 }
 
 template <int n>
@@ -117,14 +117,9 @@ void addPerm(pybind11::module_& m, const char* name) {
         .def("sign", &Perm<n>::sign)
         .def("__getitem__", &Perm<n>::operator[])
         .def("pre", &Perm<n>::pre)
-        .def("preImageOf", &Perm<n>::pre) // deprecated
         .def("compareWith", &Perm<n>::compareWith)
         .def("isIdentity", &Perm<n>::isIdentity)
         .def_static("rot", &Perm<n>::rot)
-        // index and atIndex are deprecated, so do not call them directly.
-        .def_static("atIndex",
-            [](typename Perm<n>::Index i) { return Perm<n>::orderedSn[i]; })
-        .def("index", &Perm<n>::orderedSnIndex)
         .def_static("rand", (Perm<n> (*)(bool))(&Perm<n>::rand),
             pybind11::arg("even") = false)
         .def("trunc", &Perm<n>::trunc)
@@ -142,6 +137,7 @@ void addPerm(pybind11::module_& m, const char* name) {
     Perm_extend<n, n-1>::add_bindings(c);
     Perm_contract<n, n+1>::add_bindings(c);
     regina::python::add_output_basic(c);
+    regina::python::add_tight_encoding(c);
     regina::python::add_eq_operators(c);
 }
 
