@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -42,8 +42,8 @@ using regina::python::ConstArray;
 
 namespace {
     // Note that S2 and S1 are the same C++ type.
-    ConstArray<decltype(Perm<2>::S2)> Perm2_S2_arr(Perm<2>::S2, 2);
-    ConstArray<decltype(Perm<2>::S2)> Perm2_S1_arr(Perm<2>::S1, 1);
+    ConstArray<decltype(Perm<2>::S2), int> Perm2_S2_arr(Perm<2>::S2, 2);
+    ConstArray<decltype(Perm<2>::S2), int> Perm2_S1_arr(Perm<2>::S1, 1);
 
     template <int k>
     struct Perm2_contract {
@@ -83,7 +83,6 @@ void addPerm2(pybind11::module_& m) {
         .def("sign", &Perm<2>::sign)
         .def("__getitem__", &Perm<2>::operator[])
         .def("pre", &Perm<2>::pre)
-        .def("preImageOf", &Perm<2>::pre) // deprecated
         .def("compareWith", &Perm<2>::compareWith)
         .def("isIdentity", &Perm<2>::isIdentity)
         .def("inc", [](Perm<2>& p) {
@@ -91,10 +90,6 @@ void addPerm2(pybind11::module_& m) {
         })
         .def(pybind11::self < pybind11::self)
         .def_static("rot", &Perm<2>::rot)
-        // index and atIndex are deprecated, so do not call them directly.
-        .def_static("atIndex",
-            [](Perm<2>::Index i) { return Perm<2>::orderedSn[i]; })
-        .def("index", &Perm<2>::orderedSnIndex)
         .def_static("rand", (Perm<2> (*)(bool))(&Perm<2>::rand),
             pybind11::arg("even") = false)
         .def("trunc", &Perm<2>::trunc)
@@ -116,6 +111,7 @@ void addPerm2(pybind11::module_& m) {
     ;
     Perm2_contract<3>::add_bindings(c);
     regina::python::add_output_basic(c);
+    regina::python::add_tight_encoding(c);
     regina::python::add_eq_operators(c);
 }
 
