@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -41,11 +41,11 @@ using regina::Perm;
 using regina::python::ConstArray;
 
 namespace {
-    ConstArray<decltype(Perm<3>::S3)>
+    ConstArray<decltype(Perm<3>::S3), int>
         Perm3_S3_arr(Perm<3>::S3, 6);
-    ConstArray<decltype(Perm<3>::orderedS3)>
+    ConstArray<decltype(Perm<3>::orderedS3), int>
         Perm3_orderedS3_arr(Perm<3>::orderedS3, 6);
-    ConstArray<decltype(Perm<3>::S2)>
+    ConstArray<decltype(Perm<3>::S2), int>
         Perm3_S2_arr(Perm<3>::S2, 2);
 
     template <int k>
@@ -90,7 +90,6 @@ void addPerm3(pybind11::module_& m) {
         .def("sign", &Perm<3>::sign)
         .def("__getitem__", &Perm<3>::operator[])
         .def("pre", &Perm<3>::pre)
-        .def("preImageOf", &Perm<3>::pre) // deprecated
         .def("compareWith", &Perm<3>::compareWith)
         .def("isIdentity", &Perm<3>::isIdentity)
         .def("inc", [](Perm<3>& p) {
@@ -98,10 +97,6 @@ void addPerm3(pybind11::module_& m) {
         })
         .def(pybind11::self < pybind11::self)
         .def_static("rot", &Perm<3>::rot)
-        // index and atIndex are deprecated, so do not call them directly.
-        .def_static("atIndex",
-            [](Perm<3>::Index i) { return Perm<3>::orderedSn[i]; })
-        .def("index", &Perm<3>::orderedSnIndex)
         .def_static("rand", (Perm<3> (*)(bool))(&Perm<3>::rand),
             pybind11::arg("even") = false)
         .def("trunc", &Perm<3>::trunc)
@@ -131,6 +126,7 @@ void addPerm3(pybind11::module_& m) {
     ;
     Perm3_contract<4>::add_bindings(c);
     regina::python::add_output_basic(c);
+    regina::python::add_tight_encoding(c);
     regina::python::add_eq_operators(c);
 }
 
