@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -70,15 +70,7 @@ void addRational(pybind11::module_& m) {
         .def(pybind11::self <= pybind11::self)
         .def(pybind11::self >= pybind11::self)
         .def("doubleApprox", &Rational::doubleApprox)
-        .def("doubleApproxCheck", [](const Rational& r) {
-            try {
-                return pybind11::make_tuple(r.doubleApprox(), true);
-            } catch (const regina::UnsolvedCase&) {
-                return pybind11::make_tuple(0.0, false);
-            }
-        })
         .def("tex", &Rational::tex)
-        .def("TeX", &Rational::tex) // deprecated
         .def_readonly_static("zero", &Rational::zero)
         .def_readonly_static("one", &Rational::one)
         .def_readonly_static("infinity", &Rational::infinity)
