@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -128,8 +128,6 @@ void addPacketListener(pybind11::module_& m) {
         .def(pybind11::init<>()) // necessary for pure python subclasses
         .def("isListening", &PacketListener::isListening)
         .def("unlisten", &PacketListener::unlisten)
-        .def("unregisterFromAllPackets", // deprecated
-            &PacketListener::unlisten)
         .def("packetToBeChanged", overload_cast<Packet&>(
             &PacketListener::packetToBeChanged))
         .def("packetWasChanged", overload_cast<Packet&>(
