@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -54,8 +54,6 @@ void addPluggedTorusBundle(pybind11::module_& m) {
         .def("matchingReln", &PluggedTorusBundle::matchingReln,
             pybind11::return_value_policy::reference_internal)
         .def_static("recognise", &PluggedTorusBundle::recognise)
-        .def_static("isPluggedTorusBundle", // deprecated
-            &PluggedTorusBundle::recognise)
     ;
     regina::python::add_output(c);
     regina::python::add_eq_operators(c);
