@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Python Interface                                                      *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -44,23 +44,14 @@ using regina::SnappedTwoSphere;
 void addSnappedTwoSphere(pybind11::module_& m) {
     auto c = pybind11::class_<SnappedTwoSphere>(m, "SnappedTwoSphere")
         .def(pybind11::init<const SnappedTwoSphere&>())
-        .def("clone", [](const SnappedTwoSphere& s) { // deprecated
-            return SnappedTwoSphere(s);
-        })
         .def("snappedBall", &SnappedTwoSphere::snappedBall,
             pybind11::return_value_policy::reference_internal)
         .def_static("recognise",
             overload_cast<const SnappedBall&, const SnappedBall&>(
             &SnappedTwoSphere::recognise))
-        .def_static("formsSnappedTwoSphere", // deprecated
-            overload_cast<const SnappedBall&, const SnappedBall&>(
-            &SnappedTwoSphere::recognise))
         .def_static("recognise",
             overload_cast<regina::Tetrahedron<3>*, regina::Tetrahedron<3>*>(
             &SnappedTwoSphere::recognise))
-        .def_static("formsSnappedTwoSphere", // deprecated
-            overload_cast<regina::Tetrahedron<3>*, regina::Tetrahedron<3>*>(
-            &SnappedTwoSphere::recognise))
     ;
     regina::python::add_output(c);
     regina::python::add_eq_operators(c);
