@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Qt User Interface                                                     *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -38,9 +38,10 @@
 #ifndef __PACKETEXPORTER_H
 #define __PACKETEXPORTER_H
 
+#include "reginaqt.h"
+
 class PacketFilter;
 class QString;
-class QTextCodec;
 class QWidget;
 
 namespace regina {
