@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Qt User Interface                                                     *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -338,7 +338,8 @@ EltMoveDialog3::EltMoveDialog3(QWidget* parent,
 
     connect(buttons, SIGNAL(clicked(QAbstractButton*)), this,
         SLOT(clicked(QAbstractButton*)));
-    connect(moveTypes, SIGNAL(buttonClicked(int)), this, SLOT(updateApply()));
+    connect(moveTypes, SIGNAL(buttonClicked(QAbstractButton*)), this,
+        SLOT(updateApply()));
 
     packetWasRenamed(*tri);
     packetWasChanged(*tri);
