@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Qt User Interface                                                     *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -64,8 +64,13 @@
 // #define LIBGVC_DYNAMIC_PLUGINS 1
 
 #ifndef LIBGVC_DYNAMIC_PLUGINS
+#if defined(_WIN32)
+__declspec(dllimport) gvplugin_library_t gvplugin_dot_layout_LTX_library;
+__declspec(dllimport) gvplugin_library_t gvplugin_core_LTX_library;
+#else
 extern gvplugin_library_t gvplugin_dot_layout_LTX_library;
 extern gvplugin_library_t gvplugin_core_LTX_library;
+#endif
 
 lt_symlist_t link_lt_preloaded_symbols[] = {
     { "gvplugin_dot_layout_LTX_library", &gvplugin_dot_layout_LTX_library },
@@ -185,7 +190,7 @@ void LinkGraphUI::refresh() {
     }
 
     std::string dot;
-    int width, bags;
+    size_t width, bags;
     switch (chooseType->currentIndex()) {
         case 1:
             dot = treeDecomp(true, bags, width);
@@ -255,7 +260,7 @@ void LinkGraphUI::showError(const QString& msg) {
     stack->setCurrentWidget(layerError);
 }
 
-std::string LinkGraphUI::treeDecomp(bool nice, int& bags, int& width) {
+std::string LinkGraphUI::treeDecomp(bool nice, size_t& bags, size_t& width) {
     regina::TreeDecomposition t(*link);
     if (nice)
         t.makeNice();
