@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Qt User Interface                                                     *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -38,13 +38,14 @@
 #include "examplecreator.h"
 #include "tri2creator.h"
 #include "reginasupport.h"
+#include "reginaprefset.h"
 
 #include <QComboBox>
 #include <QHBoxLayout>
 #include <QLabel>
 #include <QLayout>
 #include <QLineEdit>
-#include <QRegExp>
+#include <QRegularExpression>
 #include <QValidator>
 #include <QStackedWidget>
 
@@ -82,7 +83,7 @@ namespace {
     /**
      * Regular expressions describing different sets of parameters.
      */
-    QRegExp reIsoSig("^([A-Za-z0-9+-]+)$");
+    const QRegularExpression reIsoSig("^([A-Za-z0-9+-]+)$");
 }
 
 Tri2Creator::Tri2Creator(ReginaMain*) {
@@ -192,7 +193,7 @@ Tri2Creator::Tri2Creator(ReginaMain*) {
     label->setWhatsThis(expln);
     hLayout->addWidget(label);
     isoSig = new QLineEdit();
-    isoSig->setValidator(new QRegExpValidator(reIsoSig, hArea));
+    isoSig->setValidator(new QRegularExpressionValidator(reIsoSig, hArea));
     isoSig->setWhatsThis(expln);
     hLayout->addWidget(isoSig, 1);
     details->addWidget(hArea);//, TRI_ISOSIG);
@@ -218,8 +219,16 @@ Tri2Creator::Tri2Creator(ReginaMain*) {
     details->addWidget(hArea);//, TRI_EXAMPLE);
 
     // Tidy up.
-    type->setCurrentIndex(0);
-    details->setCurrentIndex((int)0);
+    {
+        int typeId = ReginaPrefSet::global().triDim2CreationType;
+        if (typeId >= 0 && typeId < type->count()) {
+            type->setCurrentIndex(typeId);
+            details->setCurrentIndex(typeId);
+        } else {
+            type->setCurrentIndex(0);
+            details->setCurrentIndex((int)0);
+        }
+    }
 
     QObject::connect(type, SIGNAL(activated(int)), details,
         SLOT(setCurrentIndex(int)));
@@ -232,6 +241,9 @@ QWidget* Tri2Creator::getInterface() {
 std::shared_ptr<regina::Packet> Tri2Creator::createPacket(
         std::shared_ptr<regina::Packet>, QWidget* parentWidget) {
     int typeId = type->currentIndex();
+    // Remember our selection for next time.
+    ReginaPrefSet::global().triDim2CreationType = typeId;
+
     if (typeId == TRI_EMPTY) {
         auto ans = regina::make_packet<Triangulation<2>>();
         ans->setLabel("2-D triangulation");
@@ -269,7 +281,8 @@ std::shared_ptr<regina::Packet> Tri2Creator::createPacket(
         return regina::make_packet(Example<2>::nonOrientable(genus, punctures),
             s.str());
     } else if (typeId == TRI_ISOSIG) {
-        if (! reIsoSig.exactMatch(isoSig->text())) {
+        auto match = reIsoSig.match(isoSig->text());
+        if (! match.hasMatch()) {
             ReginaSupport::sorry(parentWidget,
                 QObject::tr("The isomorphism signature is not valid."),
                 QObject::tr("<qt>An isomorphism "
@@ -286,7 +299,7 @@ std::shared_ptr<regina::Packet> Tri2Creator::createPacket(
             return nullptr;
         }
 
-        std::string sig = reIsoSig.cap(1).toUtf8().constData();
+        std::string sig = match.captured(1).toUtf8().constData();
         try {
             return regina::make_packet(Triangulation<2>::fromIsoSig(sig), sig);
         } catch (const regina::InvalidArgument&) {
