@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Qt User Interface                                                     *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -41,14 +41,16 @@
 #include "examplecreator.h"
 #include "tri3creator.h"
 #include "reginasupport.h"
+#include "reginaprefset.h"
 
+#include <numeric> // for std::gcd()
 #include <QCheckBox>
 #include <QComboBox>
 #include <QHBoxLayout>
 #include <QLabel>
 #include <QLayout>
 #include <QLineEdit>
-#include <QRegExp>
+#include <QRegularExpression>
 #include <QTextDocument>
 #include <QValidator>
 #include <QStackedWidget>
@@ -66,6 +68,7 @@ namespace {
         TRI_LAYERED_LENS_SPACE,
         TRI_SFS_SPHERE,
         TRI_LAYERED_SOLID_TORUS,
+        TRI_HANDLEBODY,
         TRI_ISOSIG,
         TRI_DEHYDRATION,
         TRI_SPLITTING_SURFACE,
@@ -100,21 +103,18 @@ namespace {
     /**
      * Regular expressions describing different sets of parameters.
      */
-    QRegExp reLensParams(R"(^[^0-9\-]*(\d+)[^0-9\-]+(\d+)[^0-9\-]*$)");
-    QRegExp reLSTParams(
+    const QRegularExpression reLensParams(
+        R"(^[^0-9\-]*(\d+)[^0-9\-]+(\d+)[^0-9\-]*$)");
+    const QRegularExpression reLSTParams(
         R"(^[^0-9\-]*(\d+)[^0-9\-]+(\d+)[^0-9\-]+(\d+)[^0-9\-]*$)");
-    QRegExp reSFS3Params(
-        R"(^[^0-9\-]*(-?\d+)[^0-9\-]+(-?\d+))"
-        R"([^0-9\-]+(-?\d+)[^0-9\-]+(-?\d+))"
-        R"([^0-9\-]+(-?\d+)[^0-9\-]+(-?\d+)[^0-9\-]*$)");
-    QRegExp reSFSAllParams(
+    const QRegularExpression reSFSAllParams(
         R"(^[^0-9\-]*(-?\d+)[^0-9\-]+(-?\d+))"
         R"((?:[^0-9\-]+(-?\d+)[^0-9\-]+(-?\d+))*)"
         R"([^0-9\-]*$)");
-    QRegExp reSFSParamPair(R"((-?\d+)[^0-9\-]+(-?\d+))");
-    QRegExp reIsoSig(R"(^([A-Za-z0-9+-]+)$)");
-    QRegExp reDehydration(R"(^([A-Za-z]+)$)");
-    QRegExp reSignature(R"(^([\(\)\.,;:\|\-A-Za-z]+)$)");
+    const QRegularExpression reSFSParamPair(R"((-?\d+)[^0-9\-]+(-?\d+))");
+    const QRegularExpression reIsoSig(R"(^([A-Za-z0-9+-]+)$)");
+    const QRegularExpression reDehydration(R"(^([A-Za-z]+)$)");
+    const QRegularExpression reSignature(R"(^([\(\)\.,;:\|\-A-Za-z]+)$)");
 }
 
 Tri3Creator::Tri3Creator(ReginaMain*) {
@@ -159,7 +159,8 @@ Tri3Creator::Tri3Creator(ReginaMain*) {
     label->setWhatsThis(expln);
     hLayout->addWidget(label);
     lensParams = new QLineEdit();
-    lensParams->setValidator(new QRegExpValidator(reLensParams, hArea));
+    lensParams->setValidator(new QRegularExpressionValidator(
+        reLensParams, hArea));
     lensParams->setWhatsThis(expln);
     hLayout->addWidget(lensParams, 1);
     details->addWidget(hArea);//, TRI_LAYERED_LENS_SPACE);
@@ -193,7 +194,8 @@ Tri3Creator::Tri3Creator(ReginaMain*) {
     label->setWhatsThis(expln);
     hLayout->addWidget(label);
     sfsParams = new QLineEdit();
-    sfsParams->setValidator(new QRegExpValidator(reSFSAllParams, hArea));
+    sfsParams->setValidator(new QRegularExpressionValidator(
+        reSFSAllParams, hArea));
     sfsParams->setWhatsThis(expln);
     hLayout->addWidget(sfsParams, 1);
     details->addWidget(hArea);//, TRI_SFS_SPHERE);
@@ -212,11 +214,30 @@ Tri3Creator::Tri3Creator(ReginaMain*) {
     label->setWhatsThis(expln);
     hLayout->addWidget(label);
     lstParams = new QLineEdit();
-    lstParams->setValidator(new QRegExpValidator(reLSTParams, hArea));
+    lstParams->setValidator(new QRegularExpressionValidator(
+        reLSTParams, hArea));
     lstParams->setWhatsThis(expln);
     hLayout->addWidget(lstParams, 1);
     details->addWidget(hArea);//, TRI_LAYERED_SOLID_TORUS);
 
+    type->addItem(QObject::tr("Orientable handlebody"));
+    hArea = new QWidget();
+    hLayout = new QHBoxLayout();
+    hLayout->setContentsMargins(0, 0, 0, 0);
+    hArea->setLayout(hLayout);
+    expln = QObject::tr("<qt>The genus of the new orientable handlebody.  "
+        "This must be a non-negative integer.</qt>");
+    label = new QLabel(QObject::tr("Genus:"));
+    label->setWhatsThis(expln);
+    hLayout->addWidget(label);
+    handlebodyGenus = new QLineEdit();
+    auto* val = new QIntValidator(hArea);
+    val->setBottom(0);
+    handlebodyGenus->setValidator(val);
+    handlebodyGenus->setWhatsThis(expln);
+    hLayout->addWidget(handlebodyGenus, 1);
+    details->addWidget(hArea);//, TRI_HANDLEBODY);
+
     type->addItem(QObject::tr("From isomorphism signature"));
     hArea = new QWidget();
     hLayout = new QHBoxLayout();
@@ -234,7 +255,7 @@ Tri3Creator::Tri3Creator(ReginaMain*) {
     label->setWhatsThis(expln);
     hLayout->addWidget(label);
     isoSig = new QLineEdit();
-    isoSig->setValidator(new QRegExpValidator(reIsoSig, hArea));
+    isoSig->setValidator(new QRegularExpressionValidator(reIsoSig, hArea));
     isoSig->setWhatsThis(expln);
     hLayout->addWidget(isoSig, 1);
     details->addWidget(hArea);//, TRI_ISOSIG);
@@ -255,7 +276,8 @@ Tri3Creator::Tri3Creator(ReginaMain*) {
     label->setWhatsThis(expln);
     hLayout->addWidget(label);
     dehydrationString = new QLineEdit(hArea);
-    dehydrationString->setValidator(new QRegExpValidator(reDehydration, hArea));
+    dehydrationString->setValidator(new QRegularExpressionValidator(
+        reDehydration, hArea));
     dehydrationString->setWhatsThis(expln);
     hLayout->addWidget(dehydrationString, 1);
     details->addWidget(hArea);//, TRI_DEHYDRATION);
@@ -275,7 +297,8 @@ Tri3Creator::Tri3Creator(ReginaMain*) {
     label->setWhatsThis(expln);
     hLayout->addWidget(label);
     splittingSignature = new QLineEdit(hArea);
-    splittingSignature->setValidator(new QRegExpValidator(reSignature, hArea));
+    splittingSignature->setValidator(new QRegularExpressionValidator(
+        reSignature, hArea));
     splittingSignature->setWhatsThis(expln);
     hLayout->addWidget(splittingSignature, 1);
     details->addWidget(hArea);//, TRI_SPLITTING_SURFACE);
@@ -301,8 +324,16 @@ Tri3Creator::Tri3Creator(ReginaMain*) {
     details->addWidget(hArea);//, TRI_EXAMPLE);
 
     // Tidy up.
-    type->setCurrentIndex(0);
-    details->setCurrentIndex((int)0);
+    {
+        int typeId = ReginaPrefSet::global().triDim3CreationType;
+        if (typeId >= 0 && typeId < type->count()) {
+            type->setCurrentIndex(typeId);
+            details->setCurrentIndex(typeId);
+        } else {
+            type->setCurrentIndex(0);
+            details->setCurrentIndex((int)0);
+        }
+    }
 
     QObject::connect(type, SIGNAL(activated(int)), details,
         SLOT(setCurrentIndex(int)));
@@ -315,12 +346,16 @@ QWidget* Tri3Creator::getInterface() {
 std::shared_ptr<regina::Packet> Tri3Creator::createPacket(
         std::shared_ptr<regina::Packet>, QWidget* parentWidget) {
     int typeId = type->currentIndex();
+    // Remember our selection for next time.
+    ReginaPrefSet::global().triDim3CreationType = typeId;
+
     if (typeId == TRI_EMPTY) {
         auto ans = regina::make_packet<Triangulation<3>>();
         ans->setLabel("3-D triangulation");
         return ans;
     } else if (typeId == TRI_LAYERED_LENS_SPACE) {
-        if (! reLensParams.exactMatch(lensParams->text())) {
+        auto match = reLensParams.match(lensParams->text());
+        if (! match.hasMatch()) {
             ReginaSupport::sorry(parentWidget, 
                 QObject::tr("<qt>The lens space "
                 "parameters (<i>p</i>,<i>q</i>) "
@@ -330,8 +365,8 @@ std::shared_ptr<regina::Packet> Tri3Creator::createPacket(
             return nullptr;
         }
 
-        unsigned long p = reLensParams.cap(1).toULong();
-        unsigned long q = reLensParams.cap(2).toULong();
+        unsigned long p = match.captured(1).toULong();
+        unsigned long q = match.captured(2).toULong();
 
         if (p <= q && ! (p == 0 && q == 1)) {
             ReginaSupport::sorry(parentWidget,
@@ -342,7 +377,7 @@ std::shared_ptr<regina::Packet> Tri3Creator::createPacket(
                 "are not.</qt>"));
             return nullptr;
         }
-        if (regina::gcd(p, q) != 1) {
+        if (std::gcd(p, q) != 1) {
             ReginaSupport::sorry(parentWidget,
                 QObject::tr("The two lens space "
                 "parameters must be relatively prime."));
@@ -354,7 +389,8 @@ std::shared_ptr<regina::Packet> Tri3Creator::createPacket(
 
         return regina::make_packet(Example<3>::lens(p, q), s.str());
     } else if (typeId == TRI_LAYERED_SOLID_TORUS) {
-        if (! reLSTParams.exactMatch(lstParams->text())) {
+        auto match = reLSTParams.match(lstParams->text());
+        if (! match.hasMatch()) {
             ReginaSupport::sorry(parentWidget,
                 QObject::tr("<qt>The layered solid "
                 "torus parameters (<i>a</i>,<i>b</i>,<i>c</i>) "
@@ -364,9 +400,9 @@ std::shared_ptr<regina::Packet> Tri3Creator::createPacket(
         }
 
         unsigned long param[3] {
-            reLSTParams.cap(1).toULong(),
-            reLSTParams.cap(2).toULong(),
-            reLSTParams.cap(3).toULong()
+            match.captured(1).toULong(),
+            match.captured(2).toULong(),
+            match.captured(3).toULong()
         };
 
         std::sort(param, param + 3);
@@ -379,7 +415,7 @@ std::shared_ptr<regina::Packet> Tri3Creator::createPacket(
                 "positive."));
             return nullptr;
         }
-        if (regina::gcd(param[0], param[1]) != 1) {
+        if (std::gcd(param[0], param[1]) != 1) {
             ReginaSupport::sorry(parentWidget,
                 QObject::tr("The layered "
                 "solid torus parameters must be relatively prime."));
@@ -401,8 +437,21 @@ std::shared_ptr<regina::Packet> Tri3Creator::createPacket(
 
         return regina::make_packet(Example<3>::lst(param[0], param[1]),
             s.str());
+    } else if (typeId == TRI_HANDLEBODY) {
+        if (handlebodyGenus->text().isEmpty()) {
+            ReginaSupport::sorry(parentWidget,
+                QObject::tr("Please enter a genus for the handlebody.  "
+                "This should be a non-negative integer."));
+            return nullptr;
+        }
+        unsigned long genus = handlebodyGenus->text().toULong();
+
+        std::ostringstream s;
+        s << "Handlebody of genus " << genus;
+
+        return regina::make_packet(Example<3>::handlebody(genus), s.str());
     } else if (typeId == TRI_SFS_SPHERE) {
-        if (! reSFSAllParams.exactMatch(sfsParams->text())) {
+        if (! reSFSAllParams.match(sfsParams->text()).hasMatch()) {
             ReginaSupport::sorry(parentWidget,
                 QObject::tr("The Seifert fibred space parameters "
                 "are not valid."),
@@ -434,12 +483,13 @@ std::shared_ptr<regina::Packet> Tri3Creator::createPacket(
         // Build the Seifert fibred space.
         regina::SFSpace sfs;
         long a, b;
-        long pos = 0;
         long whichPair = 1;
 
-        while ((pos = reSFSParamPair.indexIn(sfsParams->text(), pos)) >= 0) {
-            a = reSFSParamPair.cap(1).toLong();
-            b = reSFSParamPair.cap(2).toLong();
+        auto matchIt = reSFSParamPair.globalMatch(sfsParams->text());
+        while (matchIt.hasNext()) {
+            auto match = matchIt.next();
+            a = match.captured(1).toLong();
+            b = match.captured(2).toLong();
 
             if (a == 0) {
                 ReginaSupport::sorry(parentWidget,
@@ -467,7 +517,6 @@ std::shared_ptr<regina::Packet> Tri3Creator::createPacket(
             else
                 sfs.insertFibre(a, b);
 
-            pos += reSFSParamPair.matchedLength();
             whichPair++;
         }
 
@@ -475,7 +524,8 @@ std::shared_ptr<regina::Packet> Tri3Creator::createPacket(
         // is implemented for all such manifolds.
         return regina::make_packet(sfs.construct(), sfs.structure());
     } else if (typeId == TRI_ISOSIG) {
-        if (! reIsoSig.exactMatch(isoSig->text())) {
+        auto match = reIsoSig.match(isoSig->text());
+        if (! match.hasMatch()) {
             ReginaSupport::sorry(parentWidget,
                 QObject::tr("The isomorphism signature is not valid."),
                 QObject::tr("<qt>An isomorphism "
@@ -489,7 +539,7 @@ std::shared_ptr<regina::Packet> Tri3Creator::createPacket(
             return nullptr;
         }
 
-        std::string sig = reIsoSig.cap(1).toUtf8().constData();
+        std::string sig = match.captured(1).toUtf8().constData();
         try {
             return regina::make_packet(Triangulation<3>::fromIsoSig(sig), sig);
         } catch (const regina::InvalidArgument&) {
@@ -503,7 +553,8 @@ std::shared_ptr<regina::Packet> Tri3Creator::createPacket(
             return nullptr;
         }
     } else if (typeId == TRI_DEHYDRATION) {
-        if (! reDehydration.exactMatch(dehydrationString->text())) {
+        auto match = reDehydration.match(dehydrationString->text());
+        if (! match.hasMatch()) {
             ReginaSupport::sorry(parentWidget, 
                 QObject::tr("The dehydration string is not valid."),
                 QObject::tr("<qt>A dehydration "
@@ -516,7 +567,7 @@ std::shared_ptr<regina::Packet> Tri3Creator::createPacket(
             return nullptr;
         }
 
-        std::string dehydString = reDehydration.cap(1).toUtf8().constData();
+        std::string dehydString = match.captured(1).toUtf8().constData();
         try {
             return regina::make_packet(Triangulation<3>::rehydrate(dehydString),
                 dehydString);
@@ -532,7 +583,8 @@ std::shared_ptr<regina::Packet> Tri3Creator::createPacket(
             return nullptr;
         }
     } else if (typeId == TRI_SPLITTING_SURFACE) {
-        if (! reSignature.exactMatch(splittingSignature->text())) {
+        auto match = reSignature.match(splittingSignature->text());
+        if (! match.hasMatch()) {
             ReginaSupport::sorry(parentWidget, 
                 QObject::tr("The splitting surface signature is not valid."),
                 QObject::tr("<qt>A splitting "
@@ -547,7 +599,7 @@ std::shared_ptr<regina::Packet> Tri3Creator::createPacket(
             return nullptr;
         }
 
-        std::string sigString = reSignature.cap(1).toUtf8().constData();
+        std::string sigString = match.captured(1).toUtf8().constData();
         try {
             return regina::make_packet(
                 regina::Signature(sigString).triangulate(), sigString);
