@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Qt User Interface                                                     *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -63,7 +63,7 @@
 #include <QMessageBox>
 #include <QLabel>
 #include <QPushButton>
-#include <QRegExp>
+#include <QRegularExpression>
 #include <QTextDocument>
 #include <QToolBar>
 #include <set>
@@ -75,7 +75,7 @@ namespace {
     /**
      * Represents a destination for a single face gluing.
      */
-    QRegExp reFaceGluing(
+    const QRegularExpression reFaceGluing(
         "^\\s*"
         "(\\d+)"
         "(?:\\s*\\(\\s*|\\s+)"
@@ -85,7 +85,7 @@ namespace {
     /**
      * Represents a single tetrahedron face.
      */
-    QRegExp reFace("^[0-3][0-3][0-3]$");
+    const QRegularExpression reFace("^[0-3][0-3][0-3]$");
 }
 
 GluingsModel3::GluingsModel3(regina::Triangulation<3>* tri, bool readWrite) :
@@ -191,31 +191,34 @@ bool GluingsModel3::setData(const QModelIndex& index, const QVariant& value,
     if (text.isEmpty()) {
         // Boundary face.
         newAdjTet = -1;
-    } else if (! reFaceGluing.exactMatch(text)) {
-        // Bad string.
-        showError(tr("<qt>The face gluing should be of the "
-            "form: <i>tet (face)</i>.  An example is <i>5 (032)</i>, "
-            "which represents face 032 of tetrahedron 5.</qt>"));
-        return false;
     } else {
-        // Real face.
-        newAdjTet = reFaceGluing.cap(1).toInt();
-        QString tetFace = reFaceGluing.cap(2);
-
-        // Check explicitly for a negative tetrahedron number
-        // since isFaceStringValid() takes an unsigned integer.
-        if (newAdjTet < 0 || newAdjTet >= tri_->size()) {
-            showError(tr("There is no tetrahedron number %1.").
-                arg(newAdjTet));
+        auto match = reFaceGluing.match(text);
+        if (! match.hasMatch()) {
+            // Bad string.
+            showError(tr("<qt>The face gluing should be of the "
+                "form: <i>tet (face)</i>.  An example is <i>5 (032)</i>, "
+                "which represents face 032 of tetrahedron 5.</qt>"));
             return false;
-        }
+        } else {
+            // Real face.
+            newAdjTet = match.captured(1).toInt();
+            QString tetFace = match.captured(2);
 
-        // Do we have a valid gluing?
-        QString err = isFaceStringValid(index.row(), face, newAdjTet, tetFace,
-            &newAdjPerm);
-        if (! err.isNull()) {
-            showError(err);
-            return false;
+            // Check explicitly for a negative tetrahedron number
+            // since isFaceStringValid() takes an unsigned integer.
+            if (newAdjTet < 0 || newAdjTet >= tri_->size()) {
+                showError(tr("There is no tetrahedron number %1.").
+                    arg(newAdjTet));
+                return false;
+            }
+
+            // Do we have a valid gluing?
+            QString err = isFaceStringValid(index.row(), face, newAdjTet, tetFace,
+                &newAdjPerm);
+            if (! err.isNull()) {
+                showError(err);
+                return false;
+            }
         }
     }
 
@@ -259,7 +262,7 @@ QString GluingsModel3::isFaceStringValid(unsigned long srcTet, int srcFace,
     if (destTet >= tri_->size())
         return tr("There is no tetrahedron number %1.").arg(destTet);
 
-    if (! reFace.exactMatch(destFace))
+    if (! reFace.match(destFace).hasMatch())
         return tr("<qt>%1 is not a valid tetrahedron face.  A tetrahedron "
             "face must be described by a sequence of three vertices, each "
             "between 0 and 3 inclusive.  An example is <i>032</i>.</qt>").
@@ -988,7 +991,7 @@ void Tri3GluingsUI::drillEdge() {
             }
             if (ans) {
                 ans->setLabel(tri->adornedLabel("Drilled"));
-                tri->insertChildLast(ans);
+                tri->append(ans);
                 enclosingPane->getMainWindow()->packetView(*ans, true, true);
             }
         }
@@ -1033,7 +1036,7 @@ void Tri3GluingsUI::boundaryComponents() {
                 chosen->build());
             ans->setLabel(tr("Boundary component %1").arg(chosen->index()).
                 toUtf8().constData());
-            tri->insertChildLast(ans);
+            tri->append(ans);
             enclosingPane->getMainWindow()->packetView(*ans, true, true);
         }
     }
@@ -1063,7 +1066,7 @@ void Tri3GluingsUI::vertexLinks() {
                 chosen->buildLink());
             ans->setLabel(tr("Link of vertex %1").arg(chosen->index()).
                 toUtf8().constData());
-            tri->insertChildLast(ans);
+            tri->append(ans);
             enclosingPane->getMainWindow()->packetView(*ans, true, true);
         }
     }
@@ -1086,7 +1089,7 @@ void Tri3GluingsUI::splitIntoComponents() {
         std::shared_ptr<Packet> base;
         if (tri->firstChild()) {
             base = std::make_shared<regina::Container>();
-            tri->insertChildLast(base);
+            tri->append(base);
             base->setLabel(tri->adornedLabel("Components"));
         } else
             base = tri->shared_from_this();
@@ -1096,8 +1099,7 @@ void Tri3GluingsUI::splitIntoComponents() {
         for (auto& c : tri->triangulateComponents()) {
             std::ostringstream label;
             label << "Component #" << ++which;
-            base->insertChildLast(regina::make_packet(std::move(c),
-                label.str()));
+            base->append(regina::make_packet(std::move(c), label.str()));
         }
 
         // Make sure the new components are visible.
@@ -1158,7 +1160,7 @@ void Tri3GluingsUI::connectedSumDecomposition() {
             std::shared_ptr<Packet> base;
             if (tri->firstChild()) {
                 base = std::make_shared<regina::Container>();
-                tri->insertChildLast(base);
+                tri->append(base);
                 base->setLabel(tri->adornedLabel("Summands"));
             } else
                 base = tri->shared_from_this();
@@ -1167,8 +1169,7 @@ void Tri3GluingsUI::connectedSumDecomposition() {
             for (auto& s : ans) {
                 std::ostringstream label;
                 label << "Summand #" << ++which;
-                base->insertChildLast(regina::make_packet(std::move(s),
-                    label.str()));
+                base->append(regina::make_packet(std::move(s), label.str()));
             }
 
             // Make sure the new summands are visible.
@@ -1276,11 +1277,10 @@ void Tri3GluingsUI::makeZeroEfficient() {
         for (auto& s : summands) {
             std::ostringstream label;
             label << "Summand #" << ++which;
-            decomp->insertChildLast(regina::make_packet(std::move(s),
-                label.str()));
+            decomp->append(regina::make_packet(std::move(s), label.str()));
         }
 
-        tri->insertChildLast(decomp);
+        tri->append(decomp);
         enclosingPane->getMainWindow()->ensureVisibleInTree(
             *decomp->lastChild());
 
@@ -1412,7 +1412,7 @@ void Tri3GluingsUI::toSnapPea() {
             "(shortest, second shortest) basis on each cusp.</qt>"));
 
     ans->setLabel(tri->label());
-    tri->insertChildLast(ans);
+    tri->append(ans);
     enclosingPane->getMainWindow()->packetView(*ans, true, true);
 }
 
