@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Qt User Interface                                                     *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -33,16 +33,21 @@
 #include "regina-config.h"
 #include "libnormaliz/version.h"
 
+#ifdef BUILD_PYTHON_BINDINGS
+#include "Python.h" // for PY_VERSION
+#endif
+
 #include "iconcache.h"
 #include "reginaabout.h"
 #include "reginaprefset.h"
 #include "reginasupport.h"
 
 #include <cstdlib>
-#include <qdatetime.h>
-#include <qfile.h>
 
+#include <QtGlobal>
+#include <QDateTime>
 #include <QDialogButtonBox>
+#include <QFile>
 #include <QLabel>
 #include <QScrollBar>
 #include <QTabWidget>
@@ -55,24 +60,24 @@
 #define NMZ_VERSION_STRING REGINA_XSTR(NMZ_VERSION)
 
 const QString ReginaAbout::regCopyright(
-    tr("Copyright (c) 1999-2021, The Regina development team"));
+    tr("Copyright (c) 1999-2022, The Regina development team"));
 
 const QString ReginaAbout::regDescription(
     tr("Software for low-dimensional topology"));
 
 const QString ReginaAbout::regReleased(
-    tr("Released %1").arg(QDate(2021, 12, 19).toString(Qt::TextDate)));
+    tr("Released %1").arg(QDate(2022, 9, 30).toString(Qt::TextDate)));
 
 const QString ReginaAbout::regWebsite("http://regina-normal.github.io/");
 
 const QString ReginaAbout::regLicense( tr( 
-    "Copyright (c) 1999-2021, The Regina development team\n\n"
+    "Copyright (c) 1999-2022, The Regina development team\n\n"
     "CITATION:\n\n"
     "If you find Regina useful in your research, please consider citing it as\n"
     "you would any other paper that you use.  A suggested form of reference is:\n\n"
     "  Benjamin A. Burton, Ryan Budney, William Pettersson, et al.,\n"
     "  \"Regina: Software for low-dimensional topology\",\n"
-    "  http://regina-normal.github.io/, 1999-2021.\n\n"
+    "  http://regina-normal.github.io/, 1999-2022.\n\n"
     "COPYING AND MODIFICATION:\n\n"
     "This program is free software; you can redistribute it and/or modify it\n"
     "under the terms of the GNU General Public License as published by the\n"
@@ -111,7 +116,7 @@ const QString ReginaAbout::regLicense( tr(
     "NORMALIZ LIBRARY:\n\n"
     "Regina includes a copy of libnormaliz, which it uses to help with the\n"
     "enumeration of fundamental normal surfaces.  Normaliz was written by\n"
-    "Winfried Bruns, Bogdan Ichim and Christof Soeger.  It is distributed\n"
+    "Winfried Bruns, Christof Soeger and others.  It is distributed\n"
     "under the terms of the GNU General Public License as published by the\n"
     "Free Software Foundation, either version 3 of the License, or (at your\n"
     "option) any later version.\n\n"
@@ -159,7 +164,7 @@ ReginaAbout::ReginaAbout(QWidget* parent) :
 
     // Bundled software:
     bundled << SoftwareInfo("Normaliz", NMZ_VERSION_STRING,
-        "http://www.home.uni-osnabrueck.de/wbruns/normaliz/");
+        "https://www.normaliz.uni-osnabrueck.de/");
     bundled << SoftwareInfo("SnapPy", SNAPPY_VERSION,
         "http://snappy.computop.org/");
 
@@ -219,7 +224,6 @@ ReginaAbout::ReginaAbout(QWidget* parent) :
 
     auto* titleGrid = new QGridLayout(title);
     titleGrid->setColumnStretch(1, 1);
-    titleGrid->setMargin(6);
 
     auto* titleIcon = new QLabel(this);
     titleIcon->setPixmap(IconCache::icon(IconCache::regina).pixmap(64));
@@ -290,6 +294,13 @@ ReginaAbout::ReginaAbout(QWidget* parent) :
             "<a href=\"%1\">%1</a></p>").arg(info.website);
         bundledText += "<p style=\"margin: 0px;\">&nbsp;</p>";
     }
+#ifdef BUILD_PYTHON_BINDINGS
+    bundledText += tr("<p style=\"margin: 0px;\">"
+        "Built against Qt " QT_VERSION_STR " and Python " PY_VERSION "</p>");
+#else
+    bundledText += tr("<p style=\"margin: 0px;\">"
+        "Built against Qt " QT_VERSION_STR " without Python</p>");
+#endif
 
     auto* softwarePage = new QTextBrowser;
     softwarePage->setFrameStyle(QFrame::NoFrame);
