@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Qt User Interface                                                     *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -38,8 +38,9 @@
 #ifndef __SHORTRUNNER_H
 #define __SHORTRUNNER_H
 
+#include "reginaqt.h"
+#include <QMutex>
 #include <QProcess>
-#include <qmutex.h>
 
 /**
  * A one-shot class for running a process and collecting output.
