@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Combine several data files into a single larger data file             *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -95,7 +95,7 @@ int main(int argc, char* argv[]) {
             continue;
         }
 
-        ans->insertChildLast(data);
+        ans->append(data);
     }
 
     // Tidy up the final data file and write it.
