@@ -4,7 +4,7 @@
  *  Regina - A Normal Surface Theory Calculator                           *
  *  Form a census of triangulations that satisfy given properties         *
  *                                                                        *
- *  Copyright (c) 1999-2021, Ben Burton                                   *
+ *  Copyright (c) 1999-2022, Ben Burton                                   *
  *  For further details contact Ben Burton (bab@debian.org).              *
  *                                                                        *
  *  This program is free software; you can redistribute it and/or         *
@@ -30,10 +30,14 @@
  *                                                                        *
  **************************************************************************/
 
+#include <condition_variable>
 #include <cctype>
 #include <fstream>
 #include <memory>
+#include <mutex>
+#include <queue>
 #include <sstream>
+#include <thread>
 #include <popt.h>
 #include <unistd.h>
 #include "census/gluingpermsearcher2.h"
@@ -53,10 +57,10 @@
 #define WORD_Tetrahedron (dim4 ? "Pentachoron" : dim2 ? "Triangle" : "Tetrahedron")
 
 // Constants.
-const long MAXTET = 15;
+constexpr int MAXTET = 20;
 
 // Census parameters.
-long nTet = 0, nBdryFaces = -1;
+int nTet = 0, nBdryFaces = -1;
 regina::BoolSet
     finiteness(true, true),
     orientability(true, true),
@@ -65,14 +69,18 @@ int minimal = 0;
 int minimalPrime = 0;
 int minimalPrimeP2 = 0;
 int minimalHyp = 0;
+int allowInvalid = 0;
 int dim2 = 0;
 int dim4 = 0;
 int usePairs = 0;
 int sigs = 0;
+int canonical = 0;
+int tight = 0;
 regina::CensusPurge whichPurge;
 int genPairs = 0;
 int subContainers = 0;
 std::string outFile;
+int threads = 1;
 
 // Variables used for a dump of face pairings.
 std::unique_ptr<std::ostream> dumpStream;
@@ -82,11 +90,24 @@ unsigned long totPairings = 0;
 long long nSolns;
 std::ofstream sigStream;
 
+// Variables used for multithreaded operation.
+std::mutex casesMutex;
+std::mutex outputMutex;
+std::condition_variable cv;
+using Case = std::pair<std::string, std::shared_ptr<regina::Packet>>;
+std::queue<Case> cases;
+bool noMoreCases = false;
+
 // Forward declarations:
 template <int dim>
 void foundGluingPerms(const regina::GluingPerms<dim>&,
     const std::shared_ptr<regina::Packet>&);
 
+template <int dim>
+void foundPartial(const regina::GluingPerms<dim>&,
+    const regina::GluingPermSearcher<dim>&,
+    const std::shared_ptr<regina::Packet>&);
+
 template <int dim>
 void findAllPerms(const regina::FacetPairing<dim>&,
     typename regina::FacetPairing<dim>::IsoList,
@@ -104,8 +125,15 @@ inline void findAllPerms<2>(const regina::FacetPairing<2>& p,
         regina::FacetPairing<2>::IsoList autos, bool orientableOnly,
         bool /* finiteOnly */, regina::CensusPurge,
         const std::shared_ptr<regina::Packet>& dest) {
-    regina::GluingPermSearcher<2>::findAllPerms(p, std::move(autos),
-        orientableOnly, foundGluingPerms<2>, dest);
+    if (threads > 1) {
+        constexpr int depth = 6; // want (dim!)^depth >> threads
+        auto searcher = regina::GluingPermSearcher<2>::bestSearcher(
+            p, std::move(autos), orientableOnly);
+        searcher->partialSearch(depth, foundPartial<2>, *searcher, dest);
+    } else {
+        regina::GluingPermSearcher<2>::findAllPerms(p, std::move(autos),
+            orientableOnly, foundGluingPerms<2>, dest);
+    }
 }
 
 template <>
@@ -118,8 +146,15 @@ inline void findAllPerms<3>(const regina::FacetPairing<3>& p,
         regina::FacetPairing<3>::IsoList autos, bool orientableOnly,
         bool finiteOnly, regina::CensusPurge usePurge,
         const std::shared_ptr<regina::Packet>& dest) {
-    regina::GluingPermSearcher<3>::findAllPerms(p, std::move(autos),
-        orientableOnly, finiteOnly, usePurge, foundGluingPerms<3>, dest);
+    if (threads > 1) {
+        constexpr int depth = 3; // want (dim!)^depth >> threads
+        auto searcher = regina::GluingPermSearcher<3>::bestSearcher(
+            p, std::move(autos), orientableOnly, finiteOnly, usePurge);
+        searcher->partialSearch(depth, foundPartial<3>, *searcher, dest);
+    } else {
+        regina::GluingPermSearcher<3>::findAllPerms(p, std::move(autos),
+            orientableOnly, finiteOnly, usePurge, foundGluingPerms<3>, dest);
+    }
 }
 
 template <>
@@ -132,8 +167,15 @@ inline void findAllPerms<4>(const regina::FacetPairing<4>& p,
         regina::FacetPairing<4>::IsoList autos, bool orientableOnly,
         bool finiteOnly, regina::CensusPurge,
         const std::shared_ptr<regina::Packet>& dest) {
-    regina::GluingPermSearcher<4>::findAllPerms(p, std::move(autos),
-        orientableOnly, finiteOnly, foundGluingPerms<4>, dest);
+    if (threads > 1) {
+        constexpr int depth = 2; // want (dim!)^depth >> threads
+        auto searcher = regina::GluingPermSearcher<4>::bestSearcher(
+            p, std::move(autos), orientableOnly, finiteOnly);
+        searcher->partialSearch(depth, foundPartial<4>, *searcher, dest);
+    } else {
+        regina::GluingPermSearcher<4>::findAllPerms(p, std::move(autos),
+            orientableOnly, finiteOnly, foundGluingPerms<4>, dest);
+    }
 }
 
 template <>
@@ -156,7 +198,7 @@ void foundGluingPerms(const regina::GluingPerms<dim>& perms,
     // no such moves are possible (since it ensures no internal vertices
     // and no low-degree edges).
 
-    if (! tri.isValid())
+    if (! (allowInvalid || tri.isValid()))
         return;
     if ((! finiteness.hasFalse()) && tri.isIdeal())
         return;
@@ -170,17 +212,68 @@ void foundGluingPerms(const regina::GluingPerms<dim>& perms,
 
     // Put it in the census!
     if (sigs) {
-        sigStream << tri.isoSig() << std::endl;
+        std::string sig = tri.isoSig();
+        if (threads > 1) {
+            std::unique_lock<std::mutex> lock(outputMutex);
+            sigStream << sig << std::endl;
+        } else {
+            sigStream << sig << std::endl;
+        }
+    } else if (canonical) {
+        auto [sig, iso] = tri.isoSigDetail();
+        std::string isoEnc = iso.inverse().tightEncoding();
+        if (threads > 1) {
+            std::unique_lock<std::mutex> lock(outputMutex);
+            sigStream << sig << ' ' << isoEnc << std::endl;
+        } else {
+            sigStream << sig << ' ' << isoEnc << std::endl;
+        }
+    } else if (tight) {
+        std::string enc = tri.tightEncoding();
+        if (threads > 1) {
+            std::unique_lock<std::mutex> lock(outputMutex);
+            sigStream << enc << std::endl;
+        } else {
+            sigStream << enc << std::endl;
+        }
     } else {
         std::ostringstream out;
         out << "Item " << (nSolns + 1);
 
-        container->insertChildLast(
-            regina::make_packet(std::move(tri), out.str()));
+        auto packet = regina::make_packet(std::move(tri), out.str());
+        if (threads > 1) {
+            std::unique_lock<std::mutex> lock(outputMutex);
+            container->append(packet);
+        } else {
+            container->append(packet);
+        }
     }
     ++nSolns;
 }
 
+/**
+ * What to do with each partially-constructed triangulation in a
+ * multi-threaded search.
+ */
+template <int dim>
+void foundPartial(const regina::GluingPerms<dim>& perms,
+        const regina::GluingPermSearcher<dim>& searcher,
+        const std::shared_ptr<regina::Packet>& container) {
+    if (searcher.isComplete()) {
+        // The search tree was extremely shallow, which means everything
+        // has essentially been enumerated by the main thread.
+        foundGluingPerms(perms, container);
+        return;
+    }
+
+    std::string data = searcher.taggedData();
+    {
+        std::unique_lock<std::mutex> lock(casesMutex);
+        cases.emplace(std::move(data), container);
+    }
+    cv.notify_one();
+}
+
 /**
  * What to do with each face/facet pairing that is generated.
  */
@@ -194,7 +287,7 @@ void foundFacePairing(const regina::FacetPairing<dim>& pairing,
     if (subContainers) {
         auto subContainer = std::make_shared<regina::Container>();
         subContainer->setLabel(pairing.str());
-        container->insertChildLast(subContainer);
+        container->append(subContainer);
 
         findAllPerms<dim>(pairing, std::move(autos),
             ! orientability.hasFalse(), ! finiteness.hasFalse(),
@@ -206,6 +299,34 @@ void foundFacePairing(const regina::FacetPairing<dim>& pairing,
     }
 }
 
+/**
+ * The work performed by each thread in a multithreaded census.
+ */
+template <int dim>
+void processCases() {
+    Case c;
+    while (true) {
+        {
+            std::unique_lock<std::mutex> lock(casesMutex);
+            if (! cases.empty()) {
+                c = std::move(cases.front());
+                cases.pop();
+            } else if (noMoreCases)
+                return;
+            else {
+                // There are no cases available right now, but some
+                // might appear later.
+                cv.wait(lock);
+                continue;
+            }
+        }
+
+        // Process the subsearch encoded by data.
+        regina::GluingPermSearcher<dim>::fromTaggedData(c.first)->runSearch(
+            foundGluingPerms<dim>, c.second);
+    }
+}
+
 /**
  * Return a new text packet storing the census parameters.
  */
@@ -284,13 +405,13 @@ int main(int argc, const char* argv[]) {
     int argFinite = 0;
     int argIdeal = 0;
     poptOption opts[] = {
-        { "tetrahedra", 't', POPT_ARG_LONG, &nTet, 0,
+        { "tetrahedra", 't', POPT_ARG_INT, &nTet, 0,
             "Number of tetrahedra.", "<tetrahedra>" },
         { "boundary", 'b', POPT_ARG_NONE, &argBdry, 0,
             "Must have at least one boundary face.", nullptr },
         { "internal", 'i', POPT_ARG_NONE, &argNoBdry, 0,
             "Must have all faces internal (no boundary faces).", nullptr },
-        { "bdryfaces", 'B', POPT_ARG_LONG, &nBdryFaces, 0,
+        { "bdryfaces", 'B', POPT_ARG_INT, &nBdryFaces, 0,
             "Must have fixed number (>= 1) of boundary faces.", "<faces>" },
         { "orientable", 'o', POPT_ARG_NONE, &argOr, 0,
             "Must be orientable.", nullptr },
@@ -303,13 +424,18 @@ int main(int argc, const char* argv[]) {
         { "minimal", 'm', POPT_ARG_NONE, &minimal, 0,
             "Ignore obviously non-minimal triangulations.", nullptr },
         { "minprime", 'M', POPT_ARG_NONE, &minimalPrime, 0,
-            "Ignore obviously non-minimal, non-prime and/or disc-reducible triangulations.", nullptr },
+            "Ignore obviously non-minimal, non-prime and/or "
+            "disc-reducible triangulations.", nullptr },
         { "minprimep2", 'N', POPT_ARG_NONE, &minimalPrimeP2, 0,
-            "Ignore obviously non-minimal, non-prime, disc-reducible and/or P2-reducible triangulations.", nullptr },
+            "Ignore obviously non-minimal, non-prime, disc-reducible and/or "
+            "P2-reducible triangulations.", nullptr },
         { "minhyp", 'h', POPT_ARG_NONE, &minimalHyp, 0,
             "Ignore triangulations that are obviously not minimal ideal "
             "triangulations of cusped finite-volume hyperbolic 3-manifolds.  "
             "Implies --internal and --ideal.", nullptr },
+        { "allowinvalid", 0, POPT_ARG_NONE, &allowInvalid, 0,
+            "Do not test triangulations for validity before output.",
+            nullptr },
         { "dim2", '2', POPT_ARG_NONE, &dim2, 0,
             "Run a census of 2-manifold triangulations, "
             "not 3-manifold triangulations.  Here --tetrahedra counts "
@@ -321,13 +447,23 @@ int main(int argc, const char* argv[]) {
         { "sigs", 's', POPT_ARG_NONE, &sigs, 0,
             "Write isomorphism signatures only, not full Regina data files.",
             nullptr },
+        { "canonical", 'S', POPT_ARG_NONE, &canonical, 0,
+            "Write isomorphism signatures with matching isomorphisms "
+            "that yield canonical facet pairings.",
+            nullptr },
+        { "encodings", 'e', POPT_ARG_NONE, &tight, 0,
+            "Write tight encodings only, not full Regina data files.",
+            nullptr },
         { "subcontainers", 'c', POPT_ARG_NONE, &subContainers, 0,
-            "For each face pairing, place resulting triangulations into different subcontainers",
+            "For each face pairing, place resulting triangulations into "
+            "different subcontainers",
             nullptr },
         { "genpairs", 'p', POPT_ARG_NONE, &genPairs, 0,
             "Only generate face pairings, not triangulations.", nullptr },
         { "usepairs", 'P', POPT_ARG_NONE, &usePairs, 0,
             "Only use face pairings read from standard input.", nullptr },
+        { "threads", 0, POPT_ARG_INT, &threads, 0,
+            "Number of parallel threads (default = 1).", "<threads>" },
         POPT_AUTOHELP
         { nullptr, 0, 0, nullptr, 0, nullptr, nullptr }
     };
@@ -374,14 +510,18 @@ int main(int argc, const char* argv[]) {
         broken = true;
     } else if (genPairs && (argOr || argNor)) {
         std::cerr << "Orientability options cannot be used with "
-            << "-p/--genpairs.\n";
+            "-p/--genpairs.\n";
         broken = true;
     } else if (genPairs && (argFinite || argIdeal)) {
         std::cerr << "Finiteness options cannot be used with -p/--genpairs.\n";
         broken = true;
-    } else if (genPairs && sigs) {
+    } else if (genPairs && (sigs || canonical)) {
         std::cerr << "Signature output cannot be used with -p/--genpairs.\n";
         broken = true;
+    } else if (genPairs && threads != 1) {
+        std::cerr << "Multithreading options cannot be used with "
+            "-p/--genpairs.\n";
+        broken = true;
     } else if (dim2 && dim4) {
         std::cerr << "Options -2/--dim2 and -4/--dim4 cannot be used together.\n";
         broken = true;
@@ -416,31 +556,48 @@ int main(int argc, const char* argv[]) {
         broken = true;
     } else if (argBdry && minimalHyp) {
         std::cerr << "Options -b/--boundary and -h/--minhyp "
-            << "cannot be used together.\n";
+            "cannot be used together.\n";
         broken = true;
     } else if (argBdry && argNoBdry) {
         std::cerr << "Options -b/--boundary and -i/--internal "
-            << "cannot be used together.\n";
+            "cannot be used together.\n";
         broken = true;
     } else if (argOr && argNor) {
         std::cerr << "Options -o/--orientable and -n/--nonorientable "
-            << "cannot be used together.\n";
+            "cannot be used together.\n";
         broken = true;
     } else if (argFinite && minimalHyp) {
-        std::cerr << "Options -f/--finite and -h/--minhyp"
-            << "cannot be used together.\n";
+        std::cerr << "Options -f/--finite and -h/--minhyp "
+            "cannot be used together.\n";
         broken = true;
     } else if (argFinite && argIdeal) {
         std::cerr << "Options -f/--finite and -d/--ideal "
-            << "cannot be used together.\n";
+            "cannot be used together.\n";
+        broken = true;
+    } else if (allowInvalid && (argFinite || argIdeal)) {
+        std::cerr << "Option --allowinvalid cannot be used with finite/ideal "
+            "options.\n";
+        broken = true;
+    } else if (allowInvalid &&
+            (minimal || minimalPrime || minimalPrimeP2 || minimalHyp)) {
+        std::cerr << "Option --allowinvalid cannot be used with minimality "
+            "options.\n";
         broken = true;
     } else if (genPairs && usePairs) {
         std::cerr << "Options -p/--genpairs and -P/--usepairs "
-            << "cannot be used together.\n";
+            "cannot be used together.\n";
         broken = true;
-    } else if (subContainers && sigs) {
-        std::cerr << "Signatures (-s/--sigs) cannot be used with "
-            << "sub-containers (-c/--subcontainers).\n";
+    } else if (subContainers && (sigs || canonical || tight)) {
+        std::cerr << "Signatures (-s/--sigs or -S/--canonical) or "
+            "tight encodings (-e/--encodings) cannot be used with "
+            "sub-containers (-c/--subcontainers).\n";
+        broken = true;
+    } else if ((sigs && canonical) || (sigs && tight) || (canonical && tight)) {
+        std::cerr << "At most one of the options -s/--sigs, -S/--canonical "
+            "or -e/--encodings can be used.\n";
+        broken = true;
+    } else if (threads < 1) {
+        std::cerr << "The number of threads must be strictly positive.\n";
         broken = true;
     }
 
@@ -538,12 +695,20 @@ int runCensus() {
         }
 
         regina::FacetPairing<dim>::findAllPairings(nTet, boundary, nBdryFaces,
-                [](const regina::FacetPairing<dim>& pair,
-                   typename regina::FacetPairing<dim>::IsoList) {
-            if (dumpStream.get())
-                (*dumpStream) << pair.toTextRep() << std::endl;
-            else
-                std::cout << pair.toTextRep() << std::endl;
+                [](const regina::FacetPairing<dim>& pair) {
+            if (dumpStream.get()) {
+                if (tight)
+                    pair.tightEncode(*dumpStream);
+                else
+                    (*dumpStream) << pair.textRep();
+                (*dumpStream) << std::endl;
+            } else {
+                if (tight)
+                    pair.tightEncode(std::cout);
+                else
+                    std::cout << pair.textRep();
+                std::cout << std::endl;
+            }
             totPairings++;
         });
         std::cerr << "Total " << WORD_face << " pairings: "
@@ -558,7 +723,7 @@ int runCensus() {
     std::shared_ptr<regina::Packet> parent;
     std::shared_ptr<regina::Packet> census;
     std::shared_ptr<regina::Packet> desc;
-    if (sigs) {
+    if (sigs || canonical || tight) {
         sigStream.open(outFile.c_str());
         if (! sigStream) {
             std::cerr << "Signature file " << outFile
@@ -577,8 +742,8 @@ int runCensus() {
         census = std::make_shared<regina::Container>();
         census->setLabel("Triangulations");
 
-        parent->insertChildLast(desc);
-        parent->insertChildLast(census);
+        parent->append(desc);
+        parent->append(census);
     }
 
     // Start the census running.
@@ -592,6 +757,13 @@ int runCensus() {
     else if (minimal)
         whichPurge = regina::PURGE_NON_MINIMAL;
 
+    std::thread** t = nullptr;
+    if (threads > 1) {
+        t = new std::thread*[threads - 1];
+        for (int i = 0; i < threads - 1; ++i)
+            t[i] = new std::thread(processCases<dim>);
+    }
+
     if (usePairs) {
         // Only use the face pairings read from standard input.
         std::cout << "Trying " << WORD_face << " pairings..." << std::endl;
@@ -632,12 +804,12 @@ int runCensus() {
         }
 
         // Store the face pairings used with the census.
-        if (! sigs) {
+        if (! (sigs || canonical || tight)) {
             auto pairingPacket = std::make_shared<regina::Text>(pairingList);
             pairingPacket->setLabel(
                 dim4 ? "Facet Pairings" : dim2 ? "Edge Pairings" :
                 "Face Pairings");
-            parent->insertChildAfter(pairingPacket, desc);
+            parent->insert(pairingPacket, desc);
         }
     } else {
         // An ordinary all-face-pairings census.
@@ -645,12 +817,33 @@ int runCensus() {
 
         regina::FacetPairing<dim>::findAllPairings(nTet, boundary, nBdryFaces,
             foundFacePairing<dim>, census /* dest */);
-
-        std::cout << "Finished." << std::endl;
     }
 
+    if (threads > 1) {
+        {
+            std::unique_lock<std::mutex> lock(casesMutex);
+            noMoreCases = true;
+        }
+        cv.notify_all();
+
+        std::cout << "Waiting for subsearches to finish..." << std::endl;
+
+        // In most scenarios this main thread will finish long before
+        // the workers.  The main thread should offer to help out since
+        // there is nothing else left for it to do.
+        processCases<dim>();
+
+        for (int i = 0; i < threads - 1; ++i) {
+            t[i]->join();
+            delete t[i];
+        }
+        delete[] t;
+    }
+
+    std::cout << "Finished." << std::endl;
+
     // Write the completed census to file.
-    if (sigs) {
+    if (sigs || canonical || tight) {
         sigStream.close();
     } else {
         if (! parent->save(outFile.c_str())) {
